//---------------------------------------------------------------------------

#ifndef AllometryH
#define AllometryH


#include <math.h>
#include "TreePopulation.h"

/**
* Allometry Class - Version 2.0
* This handles allometric calculations for trees.  None of these functions
* check for over or underflow.
*
* Copyright 2005 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>-----------------
* <br>April 28, 2004 - Submitted as beta (LEM)
* <br>April 25, 2005 - Updated to allow user-selected allometric equations
* and updated version to 2.0 (LEM)
*/
class clAllometry {

  public:
  /**Functions for height-diam relationships.*/
  enum iHeightDiamFunction {
    standard, /**<Old standard function*/
    linear, /**<Height is a linear function of DBH*/
    reverse_linear /**<DBH is a linear function of height*/
  };


  /**
  * Constructor.  NULLs pointers.
  */
  clAllometry();

  /**
  * Destructor.  Frees memory.
  */
  ~clAllometry();

  /**
  * Reads data from the parameter file.
  * @param p_oDoc Parsed parameter file document.
  * @param p_oPop Tree population object.
  */
  void GetData(xercesc::DOMDocument *p_oDoc, clTreePopulation *p_oPop);

  /**
  * Gets the maximum tree height.
  *
  * @param iSpecies Species for which to get the maximum tree height.
  * @return Max tree height.
  * @throw Error if the species isn't valid.
  */
  float GetMaxTreeHeight(int iSpecies);

  /**
  * Calculates height according to the adult DBH-height relationship for the
  * given species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcAdultHeight( const float &fDbh, const int &iSpecies );

  /**
  * Calculates DBH according to the adult DBH-height relationship for the
  * given species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  * @throw ModelErr If the species isn't recognized or if the height is greater
  * than the maximum height for the species.
  */
  float CalcAdultDbh( float fHeight, const int &iSpecies );

  /**
  * Calculates canopy radius according to the adult allometry equation.  This
  * equation is: Canrad = C<sub>1</sub> * DBH<sup>xi</sup> where
  * Canrad is the canopy radius in meters, xi is the canopy radius
  * to DBH ratio, DBH is the DBH of the tree in cm, and C<sub>exp</sub> is the
  * canopy radius exponent.  The value of Canrad cannot exceed MAX_CANOPY_RADIUS.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Canopy radius in meters, up to a maximum value of MAX_CANOPY_RADIUS.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcAdultCanRadius( const float &fDbh, const int &iSpecies );

  /**
  * Calculates adult canopy depth according to the adult allometry equation. This
  * equation is: CanDepth = C<sub>2</sub> * Height<sup>psi</sup> where
  * CanDepth is the depth of the canopy in meters, C<sub>2</sub> is the fraction
  * of tree height in canopy, Height is the tree's height in meters, and
  * psi is the canopy depth exponent.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Adult canopy depth in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcAdultCanDepth(float fHeight, const int &iSpecies);

  /**
  * Calculates height according to the sapling DBH-height relationship for the
  * given species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSaplingHeight(const float &fDbh, const int &iSpecies);

  /**
  * Calculates DBH according to the sapling DBH-height relationship for the
  * given species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  * @throw ModelErr If the species isn't recognized or if the height is greater
  * than the maximum height for the species.
  */
  float CalcSaplingDbh( float fHeight, const int &iSpecies );

  /**
  * Calculates canopy radius according to sapling the allometry equation (which
  * is currently the same as the adult equation).  This equation is:
  * Canrad = C<sub>1</sub> * DBH<sup>xi</sup> where
  * Canrad is the canopy radius in meters, xi is the canopy radius
  * to DBH ratio, DBH is the DBH of the tree in cm, and C<sub>exp</sub> is the
  * canopy radius exponent.  The value of Canrad cannot exceed MAX_CANOPY_RADIUS.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Canopy radius in meters, up to a maximum value of MAX_CANOPY_RADIUS.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSaplingCanRadius( const float &fDbh, const int &iSpecies )
   {return CalcAdultCanRadius(fDbh, iSpecies);};

  /**
  * Calculates canopy depth according to the sapling allometry equation (which is
  * currently the same as the adult equation).  This
  * equation is: CanDepth = C<sub>2</sub> * Height<sup>psi</sup> where
  * CanDepth is the depth of the canopy in meters, C<sub>2</sub> is the fraction
  * of tree height in canopy, Height is the tree's height in meters, and
  * psi is the canopy depth exponent.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Adult canopy depth in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSaplingCanDepth(float fHeight, const int &iSpecies)
   {return CalcAdultCanDepth(fHeight, iSpecies);};

  /**
  * Calculates height according to the seedling diam<sub>10</sub>-height
  * relationship for the given species.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  * @throw ModelErr If the species isn't recognized.
  */
  float CalcSeedlingHeight( float fDiam10, const int &iSpecies );

  /**
  * Calculates diameter at 10 cm according to the seedling
  * diam<sub>10</sub>-height relationship for the given species.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Diameter at 10 cm in cm.
  * @throw ModelErr If the species isn't recognized or if the height is greater
  * than the maximum height for the species.
  */
  float CalcSeedlingDiam10( float fHeight, const int &iSpecies );

  /**
  * Converts a diameter at 10 cm value to a DBH.  The equation is:
  * DBH = diam<sub>10</sub> * R where DBH is the DBH in cm, diam<sub>10</sub> is
  * the diameter at 10 cm height in cm, and R is the DBH to diam<sub>10</sub>
  * ratio.
  * @param fDiam10 Diameter at 10 cm in cm.
  * @param iSpecies Species of tree.
  * @return DBH value in cm.
  * @throw ModelErr If the species isn't recognized.
  */
  float ConvertDiam10ToDbh( float fDiam10, const int &iSpecies );

  /**
  * Converts a DBH value to a diameter at 10 cm value.  The equation is:
  * diam<sub>10</sub> = DBH / R where DBH is the DBH in cm, diam<sub>10</sub> is
  * the diameter at 10 cm height in cm, and R is the DBH to diam<sub>10</sub>
  * ratio.
  * @param fDbh DBH in cm.
  * @param iSpecies Species of tree.
  * @return Diameter at 10 cm value in cm.
  * @throw ModelErr If the species isn't recognized.
  */
  float ConvertDbhToDiam10( const float &fDbh, const int &iSpecies );

  protected:

  /**Typedef to all allometry calculating functions*/
  typedef float (clAllometry::*Ptr2Allometry)(const float&, const int&);

  /**Function pointer array for the appropriate adult height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_AdultHeight;

  /**Function pointer array for the appropriate adult DBH-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_AdultDiam;

  /**Function pointer array for the appropriate sapling height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SaplingHeight;

  /**Function pointer array for the appropriate sapling DBH-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SaplingDiam;

  /**Function pointer array for the appropriate seedling height-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SeedlingHeight;

  /**Function pointer array for the appropriate seedling diam10-calculating
  * function.  Array size is number of species.*/
  Ptr2Allometry* mp_SeedlingDiam;

  /**Tree maximum height for each species.  Array size is total # species.
  * Values are required for all species.*/
  float *mp_fMaxTreeHeight;

  /**Canopy radius (cm) to height (m) ratio.  Array size is total # species.*/
  float *mp_fCanopyRadToDbh;

  /**Canopy radius exponent.  Array size is total # species.*/
  float *mp_fCanopyRadExp;

  /**Ratio of DBH to diameter at 10 cm.  For converting from one to the other.
  * Array size is total # species.*/
  float *mp_fDbhToDiam10Ratio;

  /**Fraction of height in canopy.  Array size is total # species.*/
  float *mp_fFractOfHeightInCan;

  /**Canopy depth exponent.  Array size is total # species.*/
  float *mp_fCanopyDepthExp;

  /**Slope of the height-diameter at 10 cm conversion relationship.  Array size
  * is total # species.*/
  float *mp_fSlopeHeightDiam10;

  /**Slope of asymptotic height.  Array size is total # species.*/
  float *mp_fSlopeAsympHeight;

  /**Slope of adult linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fAdultLinearSlope;

  /**Intercept of adult linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fAdultLinearIntercept;

  /**Slope of adult reverse linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fAdultReverseLinearSlope;

  /**Intercept of adult reverse linear height-dbh equation.  Array size is
  * total # species.*/
  float *mp_fAdultReverseLinearIntercept;

  /**Slope of sapling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSaplingLinearSlope;

  /**Intercept of sapling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSaplingLinearIntercept;

  /**Slope of sapling reverse linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSaplingReverseLinearSlope;

  /**Intercept of sapling reverse linear height-dbh equation.  Array size is
  * total # species.*/
  float *mp_fSaplingReverseLinearIntercept;

  /**Slope of seedling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSeedlingLinearSlope;

  /**Intercept of seedling linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSeedlingLinearIntercept;

  /**Slope of seedling reverse linear height-dbh equation.  Array size is total #
  * species.*/
  float *mp_fSeedlingReverseLinearSlope;

  /**Intercept of seedling reverse linear height-dbh equation.  Array size is
  * total # species.*/
  float *mp_fSeedlingReverseLinearIntercept;

  /**Total number of species*/
  int m_iNumSpecies;


  /**
  * Calculates height from DBH according to the standard sapling-adult
  * allometry function.  This equation is:
  * <br>Height = 1.35 + (H<sub>1</sub> - 1.35)*(1 - e<sup>-B * DBH</sup>)
  * <br>
  * where
  * Height is height in meters, H<sub>1</sub> is max tree height, B is slope
  * of asymptotic height, and DBH is tree DBH in cm.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcStandardSapAdultHeight(const float &fDbh, const int &iSpecies)
  {
    return (1.35 + ((mp_fMaxTreeHeight[iSpecies] - 1.35)*
          (1.0-exp((double)(-mp_fSlopeAsympHeight[iSpecies] * fDbh)))));
  }

  /**
  * Calculates height from DBH according to the linear function.  This equation
  * is:
  * <br>Height = a + b * DBH
  * <br>
  * where b is the adult linear slope and a is the adult linear intercept.
  * The height is limited to the maximum height for the species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcLinearAdultHeight(const float &fDbh, const int &iSpecies ) {
    float fHeight = fDbh * mp_fAdultLinearSlope[iSpecies] + mp_fAdultLinearIntercept[iSpecies];
    if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
    return fHeight;
  }

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>Height = (DBH - a) / b
  * <br>
  * where b is the adult reverse linear slope and a is the adult reverse linear
  * intercept.  The height is limited to the maximum height for the species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcReverseLinearAdultHeight(const float &fDbh, const int &iSpecies ) {
    float fHeight = (fDbh - mp_fAdultReverseLinearIntercept[iSpecies]) / mp_fAdultReverseLinearSlope[iSpecies];
    if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
    return fHeight;
  }

  /**
  * Calculates height from DBH according to the linear function.  This equation
  * is:
  * <br>Height = a + b * DBH
  * <br>
  * where b is the sapling linear slope and a is the sapling linear intercept.
  * The height is limited to the maximum height for the species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcLinearSaplingHeight(const float &fDbh, const int &iSpecies ) {
    float fHeight = fDbh * mp_fSaplingLinearSlope[iSpecies] + mp_fSaplingLinearIntercept[iSpecies];
    if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
    return fHeight;
  }

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>Height = (DBH - a) / b
  * <br>
  * where b is the sapling reverse linear slope and a is the sapling reverse
  * linear intercept.  The height is limited to the maximum height for the
  * species.
  * @param fDbh DBH in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcReverseLinearSaplingHeight(const float &fDbh, const int &iSpecies ) {
    float fHeight = (fDbh - mp_fSaplingReverseLinearIntercept[iSpecies]) / mp_fSaplingReverseLinearSlope[iSpecies];
    if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
    return fHeight;
  }

  /**
  * Calculates height according to standard seedling diam10-height allometry
  * equation.  This equation is:
  * <br>height = 0.1 + 30*(1 - e<sup>-alpha * diam<sub>10</sub>)</sup>)<br>
  * where height is tree height in m, alpha is the slope of the height -
  * diam<sub>10</sub> relationship, and diam<sub>10</sub> is the tree's diameter
  * at 10 cm height.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcStandardSeedlingHeight( const float &fDiam10, const int &iSpecies ) {
    return 0.1 + 30*(1.0-exp((double)
               (-mp_fSlopeHeightDiam10[iSpecies]*fDiam10)));
  }

  /**
  * Calculates height from diameter at 10 cm according to the linear function.
  * This equation is:
  * <br>Height = a + b * diam<sub>10</sub>
  * <br>
  * where b is the seedling linear slope and a is the seedling linear
  * intercept.  The height is limited to the maximum height for the species.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcLinearSeedlingHeight(const float &fDiam10, const int &iSpecies ) {
    float fHeight = fDiam10 * mp_fSeedlingLinearSlope[iSpecies] + mp_fSeedlingLinearIntercept[iSpecies];
    if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
    return fHeight;
  }

  /**
  * Calculates height from diameter at 10 cm according to the reverse linear
  * function. This equation is:
  * <br>Height = (diam<sub>10</sub> - a) / b
  * <br>
  * where b is the seedling reverse linear slope and a is the seedling reverse
  * linear intercept.
  * @param fDiam10 Diameter at 10 cm in cm for which to calculate tree height.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Height in meters.
  */
  float CalcReverseLinearSeedlingHeight(const float &fDiam10, const int &iSpecies ) {
    float fHeight = (fDiam10 - mp_fSeedlingReverseLinearIntercept[iSpecies]) / mp_fSeedlingReverseLinearSlope[iSpecies];
    if (fHeight > mp_fMaxTreeHeight[iSpecies]) return mp_fMaxTreeHeight[iSpecies];
    return fHeight;
  }

  /**
  * Calculates DBH from height according to the standard sapling-adult
  * allometry function.  This equation (which is just the height equation
  * switched around) is:
  * DBH = log(1-((Height-1.35)/(H<sub>1</sub>-1.35)))/-B where
  * Height is height in meters, H<sub>1</sub> is max tree height, B is slope
  * of asymptotic height, and DBH is tree DBH in cm.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcStandardSapAdultDbh( const float &fHeight, const int &iSpecies ) {
    return log(1-((fHeight-1.35)/(mp_fMaxTreeHeight[iSpecies]-1.35)))/
    (-mp_fSlopeAsympHeight[iSpecies]);
  }

  /**
  * Calculates height from DBH according to the linear function.  This equation
  * is:
  * <br>DBH = (Height - a) / b
  * <br>
  * where b is the adult linear slope and a is the adult linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcLinearAdultDbh(const float &fHeight, const int &iSpecies ) {
    return (fHeight - mp_fAdultLinearIntercept[iSpecies]) / mp_fAdultLinearSlope[iSpecies];
  }

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>DBH = a + b * Height
  * <br>
  * where b is the adult reverse linear slope and a is the adult reverse linear
  * intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcReverseLinearAdultDbh(const float &fHeight, const int &iSpecies ) {
    return fHeight * mp_fAdultReverseLinearSlope[iSpecies] + mp_fAdultReverseLinearIntercept[iSpecies];
  }

  /**
  * Calculates height from DBH according to the linear function.  This equation
  * is:
  * <br>DBH = (Height - a) / b
  * <br>
  * where b is the sapling linear slope and a is the sapling linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcLinearSaplingDbh(const float &fHeight, const int &iSpecies ) {
    return (fHeight - mp_fSaplingLinearIntercept[iSpecies]) / mp_fSaplingLinearSlope[iSpecies];
  }

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>DBH = a + b * Height
  * <br>
  * where b is the sapling reverse linear slope and a is the sapling reverse
  * linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcReverseLinearSaplingDbh(const float &fHeight, const int &iSpecies ) {
    return fHeight * mp_fSaplingReverseLinearSlope[iSpecies] + mp_fSaplingReverseLinearIntercept[iSpecies];
  }

  /**
  * Calculates diameter at 10 cm according to the standard seedling height -
  * diam10 allometry equation (which is just the height equation switched
  * around).  The equation is:
  * diam<sub>10</sub> = (log(1-((height-0.1)/30)))/-alpha where
  * height is tree height in m, alpha is the slope of the height -
  * diam<sub>10</sub> relationship, and diam<sub>10</sub> is the tree's diameter
  * at 10 cm height.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return Diameter at 10 cm in cm.
  */
  float CalcStandardSeedlingDiam10( const float &fHeight, const int &iSpecies ) {
    return (log(1-((fHeight-0.1)/30)))/-mp_fSlopeHeightDiam10[iSpecies];
  }

  /**
  * Calculates height from diameter at 10 according to the linear function.
  * This equation is:
  * <br>diam<sub>10</sub> = (Height - a) / b
  * <br>
  * where b is the seedling linear slope and a is the seedling linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcLinearSeedlingDiam10(const float &fHeight, const int &iSpecies ) {
    return (fHeight - mp_fSeedlingLinearIntercept[iSpecies]) / mp_fSeedlingLinearSlope[iSpecies];
  }

  /**
  * Calculates height from DBH according to the reverse linear function.  This
  * equation is:
  * <br>diam<sub>10</sub> = a + b * Height
  * <br>
  * where b is the seedling reverse linear slope and a is the seedling reverse
  * linear intercept.
  * @param fHeight Height in meters.
  * @param iSpecies Species of tree for which to calculate height.
  * @return DBH in cm.
  */
  float CalcReverseLinearSeedlingDiam10(const float &fHeight, const int &iSpecies ) {
    return fHeight * mp_fSeedlingReverseLinearSlope[iSpecies] + mp_fSeedlingReverseLinearIntercept[iSpecies];
  }

}; //end of class clAllometry


//---------------------------------------------------------------------------
#endif
