//---------------------------------------------------------------------------
#include "Establishment.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include "GridBase.h"
#include "GrowthOrg.h"
#include "Plot.h"
#include <stdio.h>
//---------------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 14, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
clEstablishment::clEstablishment( clSimManager * p_oSimManager ) : clBehaviorBase( p_oSimManager ),
     clWorkerBase( p_oSimManager )
     {
       try
       {
         strcpy( m_cNameString, "Establishment" );

         mp_iSeedGridCode = NULL;

         //Allowed file types
         m_iNumAllowedTypes = 2;
         mp_iAllowedFileTypes = new int[m_iNumAllowedTypes];
         mp_iAllowedFileTypes[0] = parfile;
         mp_iAllowedFileTypes[1] = detailed_output;

         //Versions
         m_fVersionNumber = 1;
         m_fMinimumVersionNumber = 1;
       }
       catch ( modelErr & err )
       {
         throw( err );
       }
       catch ( modelMsg & msg )
       {
         throw( msg );
       } //non-fatal error
       catch ( ... )
       {
         modelErr stcErr;
         stcErr.iErrorCode = UNKNOWN;
         strcpy( stcErr.cFunction, "clEstablishment::clEstablishment" );
         throw( stcErr );
       }
}

///////////////////////////////////////////////////////////////////////////////
// Destructor
// Edit history:
// -------------
// April 14, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
clEstablishment::~clEstablishment()
{
  delete[] mp_iSeedGridCode;
}

///////////////////////////////////////////////////////////////////////////////
// GetData
// Edit history:
// -------------
// April 14, 2005 - Created (LEM)
/////////////////////////////////////////////////////////////////////////////
void clEstablishment::GetData( xercesc::DOMDocument * p_oDoc )
{
  try
  {
    clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );
    char cLabel[MAX_GRID_LABEL_SIZE];
    short int iNumSpecies = p_oPop->GetNumberOfSpecies(), i;

    //Get the dispersed seeds grid
    mp_oSeedGrid = mp_oSimManager->GetGridObject( "Dispersed Seeds" );
    if ( NULL == mp_oSeedGrid )
    {
      modelErr stcErr;
      stcErr.iErrorCode = CANT_FIND_OBJECT;
      strcpy( stcErr.cFunction, "clEstablishment::GetData" );
      strcpy( stcErr.cMoreInfo, "Disperse behaviors are required if establishment is to be used." );
      throw( stcErr );
    }
    mp_iSeedGridCode = new short int[iNumSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    {
      sprintf( cLabel, "%s%d", "seeds_", mp_iWhatSpecies[i] );
      mp_iSeedGridCode[mp_iWhatSpecies[i]] = mp_oSeedGrid->GetFloatDataCode( cLabel );
      if ( -1 == mp_iSeedGridCode[mp_iWhatSpecies[i]] )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clEstablishment::GetData" );
        strcpy( stcErr.cMoreInfo, "A disperse behavior is required for all species using establishment." );
        throw( stcErr );
      }
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clEstablishment::GetData" );
    throw( stcErr );
  }
}

///////////////////////////////////////////////////////////////////////////////
// Action
// Edit history:
// -------------
// April 14, 2005 - Created (LEM)
///////////////////////////////////////////////////////////////////////////////
void clEstablishment::Action()
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    clPlot * p_oPlot = mp_oSimManager->GetPlotObject();
    clTree * p_oSeedling; //for the seedling efficiency routine
    float fSeedX, fSeedY, //coordinates of seed
         fXOrig, fYOrig, //coordinates of origin of a grid cell
         fXCellLength = mp_oSeedGrid->GetLengthXCells(), fYCellLength = mp_oSeedGrid->GetLengthYCells(), fNumSeeds; //number of seeds per grid
    int iNumXCells = mp_oSeedGrid->GetNumberXCells(), iNumYCells = mp_oSeedGrid->GetNumberYCells(), iSp, i, j, iX, iY; //loop counters

    //Loop through each grid cell
    for ( iX = 0; iX < iNumXCells; iX++ )
    {
      for ( iY = 0; iY < iNumYCells; iY++ )
      {

        //Get the origin of this cell (min X and Y values)
        fXOrig = fXCellLength * iX;
        fYOrig = fYCellLength * iY;

        //Loop through each behavior species
        for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
        {

          iSp = mp_iWhatSpecies[i];

          //Get the number of seeds in this cell for this species
          mp_oSeedGrid->GetValueOfCell( iX, iY, mp_iSeedGridCode[iSp], & fNumSeeds );

          //Disperse the seeds produced
          for ( j = 0; j < fNumSeeds; j++ )
          {
            //Get a random X and Y value for this seed within the grid cell
            fSeedX = p_oPlot->CorrectX( fXOrig + ( clModelMath::GetRand() * fXCellLength ) );
            fSeedY = p_oPlot->CorrectY( fYOrig + ( clModelMath::GetRand() * fYCellLength ) );

            //Create the new seedling - allow a random diameter at 10 cm
            p_oPop->CreateTree( fSeedX, fSeedY, iSp, clTreePopulation::seedling, 0 );
          } //end of for (i = 0; i < fNumSeeds; i++)
          //Set the number of seeds to 0
          fNumSeeds = 0;
          mp_oSeedGrid->SetValueOfCell( iX, iY, mp_iSeedGridCode[iSp], fNumSeeds );
        }
      }
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clEstablishment::Action" );
    throw( stcErr );
  }
}
