//---------------------------------------------------------------------------
// NZEstablishment.cpp
//---------------------------------------------------------------------------
#include "NZEstablishment.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include "ParsingFunctions.h"
#include "GridBase.h"
#include "LightOrg.h"

///////////////////////////////////////////////////////////////////////////////
//   Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
clNZEstablishment::clNZEstablishment(clSimManager * p_oSimManager) : clGLIBase(p_oSimManager), clLightBase(p_oSimManager),
     clBehaviorBase(p_oSimManager), clWorkerBase(p_oSimManager)
     {
       try
       {
         strcpy(m_cNameString, "NZ establishment");

         mp_fSubstrateFavorability = NULL;
         mp_fOptimumGLI = NULL;
         mp_fLowGLISlope = NULL;
         mp_fHighGLISlope = NULL;
         mp_iSeedGridCode = NULL;
         mp_iCounterCodes = NULL;
         mp_iZCodes = NULL;
         mp_iIndexes = NULL;
         mp_iSubstrateCodes = NULL;

         //Allowed file types
         m_iNumAllowedTypes = 2;
         mp_iAllowedFileTypes = new int[m_iNumAllowedTypes];
         mp_iAllowedFileTypes[0] = parfile;
         mp_iAllowedFileTypes[1] = detailed_output;

         //Versions
         m_fVersionNumber = 1;
         m_fMinimumVersionNumber = 1;
       }
       catch (modelErr & err)
       {
         throw(err);
       }
       catch (modelMsg & msg)
       {
         throw(msg);
       } //non-fatal error
       catch (...)
       {
         modelErr stcErr;
         stcErr.iErrorCode = UNKNOWN;
         strcpy(stcErr.cFunction, "clNZEstablishment::clNZEstablishment");
         throw(stcErr);
       }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   Destructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
clNZEstablishment::~clNZEstablishment()
{
  int i;
  if (mp_fSubstrateFavorability)
  {
    for (i = 0; i < number_substrates; i++)
    {
      delete[] mp_fSubstrateFavorability[i];
    }
  }
  delete[] mp_fSubstrateFavorability;
  delete[] mp_fOptimumGLI;
  delete[] mp_fLowGLISlope;
  delete[] mp_fHighGLISlope;
  delete[] mp_iSeedGridCode;
  delete[] mp_iCounterCodes;
  delete[] mp_iZCodes;
  delete[] mp_iIndexes;
  delete[] mp_iSubstrateCodes;

}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   SetNameData
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::SetNameData(char * cNameString)
{
  try
  {
    if (strcmp(cNameString, "NZ establishment with light") == 0)
    {
      m_bUsesLight = true;
    }
    else if (strcmp(cNameString, "NZ establishment without light") == 0)
    {
      m_bUsesLight = false;
      m_bNeedsCommonParameters = false;
    }
    else
    { //error!
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy(stcErr.cFunction, "clNZEstablishment::SetNameData");
      strcpy(stcErr.cMoreInfo, strcat("Unrecognized name ", cNameString));
      throw(stcErr);
    }
  }
  catch (modelErr & err)
  {
    throw(err);
  }
  catch (modelMsg & msg)
  {
    throw(msg);
  } //non-fatal error
  catch (...)
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clNZEstablishment::SetNameData");
    throw(stcErr);
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   GetParameterFileData
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::GetParameterFileData(xercesc::DOMDocument * p_oDoc)
{
  DOMElement * p_oElement = p_oDoc->getDocumentElement();
  clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject("treepopulation");
  clTreePopulation * p_oPop = (clTreePopulation *) p_oTemp;
  floatVal * p_fTemp = NULL; //for getting species-specific values
  short int i, j;

  //Set up our temp array - pre-load with this behavior's species
  p_fTemp = new floatVal[m_iNumBehaviorSpecies];
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    p_fTemp[i].code = mp_iWhatSpecies[i];

  //*******************************
  // Substrate favorabilities
  //*******************************
  mp_iIndexes = new short int[p_oPop->GetNumberOfSpecies()];
  //Load the indexes array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_iIndexes[mp_iWhatSpecies[i]] = i;

  //Declare the canopy array values
  mp_fSubstrateFavorability = new float * [number_substrates];
  for (i = 0; i < number_substrates; i++)
  {
    mp_fSubstrateFavorability[i] = new float[m_iNumBehaviorSpecies];
  }

  //Mound moss favorability
  FillSpeciesSpecificValue(p_oElement, "es_mossMoundFav", "es_mmfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[mound_moss] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Mound mineral soil favorability
  FillSpeciesSpecificValue(p_oElement, "es_mineralSoilMoundFav", "es_msmfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[mound_mineralsoil] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Mound fresh logs favorability
  FillSpeciesSpecificValue(p_oElement, "es_freshLogMoundFav", "es_flmfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[mound_freshlogs] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Mound mossy logs favorability
  FillSpeciesSpecificValue(p_oElement, "es_mossyLogMoundFav", "es_mlmfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[mound_mossylogs] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Mound litter favorability
  FillSpeciesSpecificValue(p_oElement, "es_litterMoundFav", "es_lmfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[mound_litter] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Ground moss favorability
  FillSpeciesSpecificValue(p_oElement, "es_mossGroundFav", "es_mgfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[ground_moss] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Ground mineral soil favorability
  FillSpeciesSpecificValue(p_oElement, "es_mineralSoilGroundFav", "es_msgfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[ground_mineralsoil] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Ground fresh logs favorability
  FillSpeciesSpecificValue(p_oElement, "es_freshLogGroundFav", "es_flgfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[ground_freshlogs] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Ground mossy logs favorability
  FillSpeciesSpecificValue(p_oElement, "es_mossyLogGroundFav", "es_mlgfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[ground_mossylogs] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Ground litter favorability
  FillSpeciesSpecificValue(p_oElement, "es_litterGroundFav", "es_lgfVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
  //Now transfer the values to the permanent array
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
    mp_fSubstrateFavorability[ground_litter] [mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

  //Proportion of the plot that is mound
  FillSingleValue(p_oElement, "es_moundProportion", & m_fMoundProportion, true);

  //Maximum number of years of respite
  FillSingleValue(p_oElement, "es_maxRespite", & m_iMaxRespiteYears, true);

  //Calculate the number of timesteps of respite
  m_iMaxRespiteTimesteps = ceil(m_iMaxRespiteYears / mp_oSimManager->GetNumberOfYearsPerTimestep());

  //Mean mound height in m
  FillSingleValue(p_oElement, "es_meanMoundHeight", & m_fMoundHeightMean, true);
  //Convert to mm
  m_fMoundHeightMean *= 1000;

  //Mound height standard deviation in m
  FillSingleValue(p_oElement, "es_moundStdDev", & m_fMoundStandardDeviation, true);
  //Convert to mm
  m_fMoundStandardDeviation *= 1000;

  //Mean fresh log height in m
  FillSingleValue(p_oElement, "es_meanFreshLogHeight", & m_fFreshLogHeightMean, true);
  //Convert to mm
  m_fFreshLogHeightMean *= 1000;

  //Fresh log height standard deviation in m
  FillSingleValue(p_oElement, "es_freshLogStdDev", & m_fFreshLogStandardDeviation, true);
  //Convert to mm
  m_fFreshLogStandardDeviation *= 1000;



  //*******************************
  // Light values
  //*******************************
  if (m_bUsesLight)
  {

    mp_fOptimumGLI = new float[m_iNumBehaviorSpecies];
    mp_fLowGLISlope = new float[m_iNumBehaviorSpecies];
    mp_fHighGLISlope = new float[m_iNumBehaviorSpecies];

    //Optimum GLI
    FillSpeciesSpecificValue(p_oElement, "es_optimumGLI", "es_ogVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
    //Now transfer the values to the permanent array
    for (i = 0; i < m_iNumBehaviorSpecies; i++)
      mp_fOptimumGLI[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Slope of decreased light favorability below optimum GLI
    FillSpeciesSpecificValue(p_oElement, "es_lowSlope", "es_lsVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
    //Now transfer the values to the permanent array
    for (i = 0; i < m_iNumBehaviorSpecies; i++)
      mp_fLowGLISlope[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Slope of decreased light favorability above optimum GLI
    FillSpeciesSpecificValue(p_oElement, "es_highSlope", "es_hsVal", p_fTemp, m_iNumBehaviorSpecies, p_oPop, true);
    //Now transfer the values to the permanent array
    for (i = 0; i < m_iNumBehaviorSpecies; i++)
      mp_fHighGLISlope[mp_iIndexes[p_fTemp[i].code]] = p_fTemp[i].val;

    //Height of light calculations
    FillSingleValue(p_oElement, "es_lightHeight", & m_fLightHeight, true);

    //Light calculation values

    //Number of alitude angles
    FillSingleValue(p_oElement, "li_numAltGrids", & m_iNumAltAng, true);
    //Number of azimuth angles
    FillSingleValue(p_oElement, "li_numAziGrids", & m_iNumAziAng, true);
    //Minimum sun angle
    FillSingleValue(p_oElement, "li_minSunAngle", & m_fMinSunAngle, true);
  }

  delete[] p_fTemp;

  ValidateData();
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   ValidateData
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::ValidateData()
{
  try
  {

    int i, j;

    for (i = 0; i < number_substrates; i++)
    {
      for (j = 0; j < m_iNumBehaviorSpecies; j++)
      {
        if (mp_fSubstrateFavorability[i] [j] < 0 || mp_fSubstrateFavorability[i] [j] > 1)
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy(stcErr.cFunction, "clNZEstablishment::ValidateData");
          strcpy(stcErr.cMoreInfo, "All substrate favorability values must be between 0 and 1.");
          throw(stcErr);
        }
      }
    }

    if (m_fMoundProportion < 0 || m_fMoundProportion > 1)
    {
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy(stcErr.cFunction, "clNZEstablishment::ValidateData");
      strcpy(stcErr.cMoreInfo, "Proportion of plot that is mound must be between 0 and 1.");
      throw(stcErr);
    }

    if (m_bUsesLight)
    {
      if (m_fLightHeight < 0)
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy(stcErr.cFunction, "clNZEstablishment::ValidateData");
        strcpy(stcErr.cMoreInfo, "Height of light calculations must be greater than 0.");
        throw(stcErr);
      }

      for (i = 0; i < m_iNumBehaviorSpecies; i++)
      {
        if (mp_fOptimumGLI[i] < 0 || mp_fOptimumGLI[i] > 100)
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy(stcErr.cFunction, "clNZEstablishment::ValidateData");
          strcpy(stcErr.cMoreInfo, "Optimum GLI must be between 0 and 100.");
          throw(stcErr);
        }
      }
    }
  }
  catch (modelErr & err)
  {
    throw(err);
  }
  catch (modelMsg & msg)
  {
    throw(msg);
  } //non-fatal error
  catch (...)
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clNZEstablishment::ValidateData");
    throw(stcErr);
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   GetData
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::GetData(xercesc::DOMDocument * p_oDoc)
{
  try
  {
    GetParameterFileData(p_oDoc);
    SetupGrids();
    SetLightFunction();
    GetTreeDataMemberCodes();
    if (m_bUsesLight)
    {
      DoLightSetup();
    }
  }
  catch (modelErr & err)
  {
    throw(err);
  }
  catch (modelMsg & msg)
  {
    throw(msg);
  } //non-fatal error
  catch (...)
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clNZEstablishment::GetData");
    throw(stcErr);
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   DoLightSetup
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::DoLightSetup()
{
  try
  {
    clBehaviorBase * p_oTempBehavior; //for going through behaviors looking for
    //light shells
    clLightBase * p_oLightShell; //light shell object
    char cBehaviorName[MAX_NAMESTRING_SIZE], //behavior namestrings
         cShellMarker[] = "lightshell";
    float fAngChunk;
    int i, j, iHalfAzi, //for partitioning the sky hemisphere
         iNumBehaviors;

    //**********************************************
    //Populate the sky brightness and photo arrays
    //**********************************************
    //Declare 'em
    mp_fBrightness = new float * [m_iNumAltAng];
    mp_fPhoto = new float * [m_iNumAltAng];
    for (i = 0; i < m_iNumAltAng; i++)
    {
      mp_fBrightness[i] = new float[m_iNumAziAng];
      mp_fPhoto[i] = new float[m_iNumAziAng];
      for (j = 0; j < m_iNumAziAng; j++)
        mp_fBrightness[i] [j] = 0;
    }

    m_fSinMinSunAng = sin(m_fMinSunAngle);
    m_iMinAngRow = floor(m_fSinMinSunAng * m_iNumAltAng);

    PopulateGLIBrightnessArray();

    m_fAziChunkConverter = m_iNumAziAng / 360.0; //force float conversion
    m_fRcpTanMinAng = 1 / (tan(m_fMinSunAngle));

    mp_fAziSlope = new float[m_iNumAziAng];
    iHalfAzi = m_iNumAziAng / 2;
    //Get the size of each azimuth chunk in radians
    fAngChunk = (2.0 * M_PI) / m_iNumAziAng;
    for (i = 0; i < iHalfAzi; i++)
    {
      //slope = tan of azimuth angle
      mp_fAziSlope[i] = 1 / (tan(fAngChunk * (i + 0.5)));
      mp_fAziSlope[i + iHalfAzi] = mp_fAziSlope[i];
    }
  }
  catch (modelErr & err)
  {
    throw(err);
  }
  catch (modelMsg & msg)
  {
    throw(msg);
  } //non-fatal error
  catch (...)
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clNZEstablishment::DoLightSetup");
    throw(stcErr);
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   GetLightFavorabilities()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::GetLightFavorabilities()
{
  GetGLIs();
  CalculateLightFavorabilities();
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   GetGLIs()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::GetGLIs()
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject("treepopulation");
    clTreePopulation * p_oPop = (clTreePopulation *) p_oTemp;
    clTreeSearch * p_oShaders; //neighborhood trees who can shade each target
    clTree * p_oNeighbor; //shading neighbors
    clPlot * p_oPlot = mp_oSimManager->GetPlotObject();
    clAllometry * p_oAllom = p_oPop->GetAllometryObject();
    char cQuery[75]; //format search strings into this
    float fX, fY, //coordinates at which to calculate light
         fGli, //global light index - end result of all this math
         //max search radius
         fMaxSearchRad = (mp_oLightOrg->GetMaxTreeHeight() - m_fLightHeight) * m_fRcpTanMinAng + MAX_CANOPY_RADIUS;
    int iNumXCells = mp_oLightGrid->GetNumberXCells(), iNumYCells = mp_oLightGrid->GetNumberYCells(), iX, iY, i, j;

    for (iX = 0; iX < iNumXCells; iX++)
    {
      for (iY = 0; iY < iNumYCells; iY++)
      {
        //First find the point at which to calculate gli - center of the tree's
        //quadrat - place in fX and fY
        mp_oLightGrid->GetPointOfCell(iX, iY, & fX, & fY);

        //Initialize photo array
        for (i = 0; i < m_iNumAltAng; i++)
          for (j = 0; j < m_iNumAziAng; j++)
            mp_fPhoto[i] [j] = 1.0;

        //Get a list of all trees that are within the search radius and taller than
        //the fish-eye photo height
        sprintf(cQuery, "%s%f%s%f%s%f%s%f", "distance=", fMaxSearchRad, " FROM x=", fX, "y=", fY, "::height=", m_fLightHeight);
        p_oShaders = p_oPop->Find(cQuery);

        p_oNeighbor = p_oShaders->NextTree();
        while (p_oNeighbor != NULL)
        {

          //Skip seedlings, which don't shade
          if (clTreePopulation::seedling != p_oNeighbor->GetType())

               //Add the effect of the neighbor to the simulated fisheye photo
                 AddTreeToGliFishEye(fX, fY, m_fLightHeight, p_oNeighbor, p_oPlot, p_oPop, p_oAllom);

          p_oNeighbor = p_oShaders->NextTree();
        } //end of while (neighbor != NULL)

        //Calculate GLI
        fGli = 0.0;
        for (i = 0; i < m_iNumAltAng; i++)
          for (j = 0; j < m_iNumAziAng; j++)
            fGli += (mp_fPhoto[i] [j] * mp_fBrightness[i] [j]);
        fGli *= 100;

        //Assign GLI to the first light favorability
        mp_oLightGrid->SetValueAtPoint(fX, fY, mp_iLightFavCodes[0], fGli);
      }
    }
  }
  catch (modelErr & err)
  {
    throw(err);
  }
  catch (modelMsg & msg)
  {
    throw(msg);
  } //non-fatal error
  catch (...)
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clNZEstablishment::GetGLIs");
    throw(stcErr);
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   SetupGrids()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::SetupGrids()
{
  char cLabel[MAX_GRID_LABEL_SIZE];
  int i;

  //************************************
  //Get the "NZ Substrate" grid and its data members
  //************************************
  mp_oSubstrateGrid = mp_oSimManager->GetGridObject("NZ Substrate");
  if (!mp_oSubstrateGrid)
  {
    modelErr stcErr;
    stcErr.iErrorCode = CANT_FIND_OBJECT;
    strcpy(stcErr.cFunction, "clNZEstablishment::SetUpGrids");
    strcpy(stcErr.cMoreInfo, "Substrate behavior required.");
    throw(stcErr);
  }

  mp_iSubstrateCodes = new short int[number_substrates];
  mp_iSubstrateCodes[mound_moss] = mp_oSubstrateGrid->GetFloatDataCode("moss");
  mp_iSubstrateCodes[mound_mineralsoil] = mp_oSubstrateGrid->GetFloatDataCode("minsoil");
  mp_iSubstrateCodes[mound_freshlogs] = mp_oSubstrateGrid->GetFloatDataCode("freshlog");
  mp_iSubstrateCodes[mound_mossylogs] = mp_oSubstrateGrid->GetFloatDataCode("mossylog");
  mp_iSubstrateCodes[mound_litter] = mp_oSubstrateGrid->GetFloatDataCode("litter");
  mp_iSubstrateCodes[ground_moss] = mp_oSubstrateGrid->GetFloatDataCode("moss");
  mp_iSubstrateCodes[ground_mineralsoil] = mp_oSubstrateGrid->GetFloatDataCode("minsoil");
  mp_iSubstrateCodes[ground_freshlogs] = mp_oSubstrateGrid->GetFloatDataCode("freshlog");
  mp_iSubstrateCodes[ground_mossylogs] = mp_oSubstrateGrid->GetFloatDataCode("mossylog");
  mp_iSubstrateCodes[ground_litter] = mp_oSubstrateGrid->GetFloatDataCode("litter");

  //Make sure that all the codes are valid
  for (i = 0; i < number_substrates; i++)
  {
    if (-1 == mp_iSubstrateCodes[i])
    {
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy(stcErr.cFunction, "clNZEstablishment::SetUpGrids");
      strcpy(stcErr.cMoreInfo, "Unexpected grid data return code for NZ Substrate.");
      throw(stcErr);
    }
  }

  //Package codes
  m_iCohFreshLogCode = mp_oSubstrateGrid->GetPackageFloatDataCode("dec_flog");
  m_iCohAgeCode = mp_oSubstrateGrid->GetPackageIntDataCode("age");
//  m_iCohAvgFreshLogDBHCode = mp_oSubstrateGrid->GetPackageFloatDataCode("avglogsize");

  //Make sure that all the codes are valid
  if (-1 == m_iCohFreshLogCode || -1 == m_iCohAgeCode)
  {
    modelErr stcErr;
    stcErr.iErrorCode = BAD_DATA;
    strcpy(stcErr.cFunction, "clNZEstablishment::SetUpGrids");
    strcpy(stcErr.cMoreInfo, "Unexpected grid data return code for NZ Substrate.");
    throw(stcErr);
  }

  //************************************
  //Get the "Dispersed Seeds" grid and its data members
  //************************************
  mp_oSeedGrid = mp_oSimManager->GetGridObject("Dispersed Seeds");
  if (NULL == mp_oSeedGrid)
  {
    modelErr stcErr;
    stcErr.iErrorCode = CANT_FIND_OBJECT;
    strcpy(stcErr.cFunction, "clNZEstablishment::SetupGrids");
    strcpy(stcErr.cMoreInfo, "Disperse behaviors are required if establishment is to be used.");
    throw(stcErr);
  }
  mp_iSeedGridCode = new short int[m_iNumBehaviorSpecies];
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
  {
    sprintf(cLabel, "%s%d", "seeds_", mp_iWhatSpecies[i]);
    mp_iSeedGridCode[i] = mp_oSeedGrid->GetFloatDataCode(cLabel);
    if (-1 == mp_iSeedGridCode[i])
    {
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy(stcErr.cFunction, "clNZEstablishment::SetupGrids");
      strcpy(stcErr.cMoreInfo, "Unexpected grid data return code for Dispersed Seeds.");
      throw(stcErr);
    }
  }

  //************************************
  //Set up the "Establishment Light" grid
  //************************************
  if (m_bUsesLight)
  {
    mp_oLightGrid = mp_oSimManager->CreateGrid("Establishment Light", //grid name
         0, //number of int data members
         m_iNumBehaviorSpecies, //number of float data members
         0, //number of char data members
         0, //number of bool data members
         mp_oSeedGrid->GetLengthXCells(), //X cell length
         mp_oSeedGrid->GetLengthYCells()); //Y cell length

    mp_iLightFavCodes = new short int[m_iNumBehaviorSpecies];
    for (i = 0; i < m_iNumBehaviorSpecies; i++)
    {
      sprintf(cLabel, "%s%d", "litefav_", mp_iWhatSpecies[i]);
      mp_iLightFavCodes[i] = mp_oLightGrid->RegisterFloat(cLabel);
    }
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   GetTreeDataMemberCodes()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::GetTreeDataMemberCodes()
{
  clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject("treepopulation");
  clTreePopulation * p_oPop = (clTreePopulation *) p_oTemp;
  char cCounter[] = "lf_count", cZ[] = "z";
  int i;

  mp_iCounterCodes = new short int[m_iNumBehaviorSpecies];
  mp_iZCodes = new short int[m_iNumBehaviorSpecies];

  //Get the values for each species for seedlings
  for (i = 0; i < m_iNumBehaviorSpecies; i++)
  {
    mp_iCounterCodes[i] = p_oPop->GetIntDataCode(cCounter, mp_iWhatSpecies[i], clTreePopulation::seedling);
    mp_iZCodes[i] = p_oPop->GetIntDataCode(cZ, mp_iWhatSpecies[i], clTreePopulation::seedling);

    //If the return code is -1, throw an error
    if (-1 == mp_iCounterCodes[i] || -1 == mp_iZCodes[i])
    {
      modelErr stcErr;
      strcpy(stcErr.cFunction, "clNZEstablishment::GetTreeDataMemberCodes");
      strcpy(stcErr.cMoreInfo, "NZ Establishment must be used with the light filter behavior.");
      stcErr.iErrorCode = BAD_DATA;
      throw(stcErr);
    }
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   CalculateLightFavorabilities()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::CalculateLightFavorabilities()
{
  float fFavorability, //favorability
       fGLI; //global light index
  int iNumXCells = mp_oLightGrid->GetNumberXCells(), iNumYCells = mp_oLightGrid->GetNumberYCells(), iX, iY, i, iSpIndex; //species index

  for (iX = 0; iX < iNumXCells; iX++)
  {
    for (iY = 0; iY < iNumYCells; iY++)
    {
      //Get the GLI
      mp_oLightGrid->GetValueOfCell(iX, iY, mp_iLightFavCodes[0], & fGLI);

      //Calculate the favorability for each species
      for (i = 0; i < m_iNumBehaviorSpecies; i++)
      {
        iSpIndex = mp_iIndexes[mp_iWhatSpecies[i]];
        if (fGLI > mp_fOptimumGLI[iSpIndex])
        {
          //GLI is greater than optimum - calculate using above-optimum slope
          fFavorability = 1 - (mp_fHighGLISlope[iSpIndex] * (fGLI - mp_fOptimumGLI[iSpIndex]));
        }
        else if (fGLI < mp_fOptimumGLI[iSpIndex])
        {
          //GLI is less than optimum - calculate using below-optimum slope
          fFavorability = 1 - (mp_fLowGLISlope[iSpIndex] * (mp_fOptimumGLI[iSpIndex] - fGLI));
        }
        else
        {
          //GLI is optimum - favorability is 1
          fFavorability = 1;
        }

        //Assign the favorability
        mp_oLightGrid->SetValueOfCell(iX, iY, mp_iLightFavCodes[iSpIndex], fFavorability);
      }
    }
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   Action()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::Action()
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject("treepopulation");
    clTreePopulation * p_oPop = (clTreePopulation *) p_oTemp;
    clTree * p_oSeedling; //newly created seedling
    float * p_fSubstrateProportions, //proportions of each type of substrate
         * p_fFreshLogProportions, //proportions of different age fresh logs
         fSeedX, fSeedY, //coordinates of seed
         fXCellLength = mp_oSeedGrid->GetLengthXCells(), //length of seed grid x
         fYCellLength = mp_oSeedGrid->GetLengthYCells(), //length of seed grid y
         fXOrig, fYOrig, //coordinates of origin of a grid cell
         fXEnd, fYEnd, //coordinates of end of a grid cell
         fNumOrigSeeds, //number of total seeds for a species for a grid
         fNumSeedsLeft, //number of seeds per grid - managing subtraction
         fNumSubstrateSeeds; //number of seeds on an individual substrate
    int iNumXCells = mp_oSeedGrid->GetNumberXCells(), //number X cells
         iNumYCells = mp_oSeedGrid->GetNumberYCells(), //number Y cells
         iX, iY, //loop counters for moving through grid cells
         iSpecies, //species of seed being dispersed
         i, j,
         iSubstr; //loop counter for substrate types

    //Calcuate light levels if using
    if (m_bUsesLight)
    {
      GetLightFavorabilities();
    }

    //Cycle through the seeds grid and deal with each seed in each one
    for (iX = 0; iX < iNumXCells; iX++)
    {
      for (iY = 0; iY < iNumYCells; iY++)
      {

        //Get the origin and end of this cell of the "Dispersed Seeds" grid
        //(min and max X and Y values)
        fXOrig = mp_oSeedGrid->GetOriginXOfCell(iX);
        fYOrig = mp_oSeedGrid->GetOriginYOfCell(iY);
        fXEnd = mp_oSeedGrid->GetEndXOfCell(iX);
        fYEnd = mp_oSeedGrid->GetEndXOfCell(iY);

        //Get the substrate proportions for this cell
        p_fSubstrateProportions = GetSubstrateProportions(fXOrig, fYOrig, fXEnd, fYEnd);

        //Get the fresh log proportions for this cell
        p_fFreshLogProportions = GetFreshLogProportions(fXOrig, fYOrig, fXEnd, fYEnd);

        //Loop through each behavior species
        for (i = 0; i < m_iNumBehaviorSpecies; i++)
        {

          iSpecies = mp_iWhatSpecies[i];

          //Get the number of seeds in this cell for this species
          mp_oSeedGrid->GetValueOfCell(iX, iY, mp_iSeedGridCode[mp_iIndexes[iSpecies]], & fNumOrigSeeds);
          fNumSeedsLeft = fNumOrigSeeds;

          //Loop through the substrate types
          for (iSubstr = 0; iSubstr < number_substrates; iSubstr++)
          {

            //How many seeds are for this substrate?
            if (0 == fNumSeedsLeft) goto nextSpecies;
            fNumSubstrateSeeds = clModelMath::RandomRound(fNumOrigSeeds * p_fSubstrateProportions[iSubstr]);
            if (fNumSubstrateSeeds > fNumSeedsLeft) fNumSubstrateSeeds = fNumSeedsLeft;
            fNumSeedsLeft -= fNumSubstrateSeeds;

            //Split up the substrate types - this saves us if statements
            //for each seed
            if (mound_freshlogs == iSubstr || ground_freshlogs == iSubstr)
            {

              //This is fresh log

              //Disperse each of the seeds
              for (j = 0; j < fNumSubstrateSeeds; j++)
              {

                //Is the seed in the proportion germinating at the cell's light
                //level (if used)?
                if ((* this.*DoesSeedSurviveLight) (iX, iY, iSpecies))
                {

                  //Is the seed in the proportion germinating on its substrate
                  //type?
                  if (DoesSeedSurviveSubstrate(iSubstr, iSpecies))
                  {

                    //This seed germinates.  So make it into a seedling.
                    //Get a random X and Y value for this seed within the grid cell
                    fSeedX = fXOrig + (clModelMath::GetRand() * fXCellLength);
                    fSeedY = fYOrig + (clModelMath::GetRand() * fYCellLength);

                    //Create the new seedling - allow a random diameter at 10 cm
                    p_oSeedling = p_oPop->CreateTree(fSeedX, fSeedY, iSpecies, clTreePopulation::seedling, 0);

                    //Give it its rooting height and fern respite counters
                    SetFreshLogZAndRespite(p_oSeedling, p_fFreshLogProportions);
                  }
                }
              }
            }
            else if (iSubstr >= mound_moss && iSubstr < ground_moss)
            {

              //This is a mound substrate
              //Disperse each of the seeds
              for (j = 0; j < fNumSubstrateSeeds; j++)
              {

                //Is the seed in the proportion germinating at the cell's light
                //level (if used)?
                if ((* this.*DoesSeedSurviveLight) (iX, iY, iSpecies))
                {

                  //Is the seed in the proportion germinating on its substrate
                  //type?
                  if (DoesSeedSurviveSubstrate(iSubstr, iSpecies))
                  {

                    //This seed germinates.  So make it into a seedling.
                    //Get a random X and Y value for this seed within the grid cell
                    fSeedX = fXOrig + (clModelMath::GetRand() * fXCellLength);
                    fSeedY = fYOrig + (clModelMath::GetRand() * fYCellLength);

                    //Create the new seedling - allow a random diameter at 10 cm
                    p_oSeedling = p_oPop->CreateTree(fSeedX, fSeedY, iSpecies, clTreePopulation::seedling, 0);

                    //Give it its rooting height and fern respite counters
                    SetMoundZAndRespite(p_oSeedling);
                  }
                }
              }
            }
            else
            {
              //This is a ground substrate
              //Disperse each of the seeds
              for (j = 0; j < fNumSubstrateSeeds; j++)
              {

                //Is the seed in the proportion germinating at the cell's light
                //level (if used)?
                if ((* this.*DoesSeedSurviveLight) (iX, iY, iSpecies))
                {

                  //Is the seed in the proportion germinating on its substrate
                  //type?
                  if (DoesSeedSurviveSubstrate(iSubstr, iSpecies))
                  {

                    //This seed germinates.  So make it into a seedling.
                    //Get a random X and Y value for this seed within the grid cell
                    fSeedX = fXOrig + (clModelMath::GetRand() * fXCellLength);
                    fSeedY = fYOrig + (clModelMath::GetRand() * fYCellLength);

                    //Create the new seedling - allow a random diameter at 10 cm
                    p_oSeedling = p_oPop->CreateTree(fSeedX, fSeedY, iSpecies, clTreePopulation::seedling, 0);

                    //Give it its rooting height and fern respite counters
                    SetGroundZAndRespite(p_oSeedling);
                  }
                }
              }
            }
          }
          nextSpecies:
        }
      }
    }
    delete[] p_fSubstrateProportions;
    delete[] p_fFreshLogProportions;
  }
  catch (modelErr & err)
  {
    throw(err);
  }
  catch (modelMsg & msg)
  {
    throw(msg);
  } //non-fatal error
  catch (...)
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy(stcErr.cFunction, "clNZEstablishment::Action");
    throw(stcErr);
  }
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   SetGroundZAndRespite()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::SetGroundZAndRespite(clTree * p_oSeedling)
{
  int iValue = 0, //value to be set
  iSp = p_oSeedling->GetSpecies(); //seedling's species
  //Set both z and lf_count to 0
  p_oSeedling->SetValue(mp_iZCodes[mp_iIndexes[iSp]], iValue);
  p_oSeedling->SetValue(mp_iCounterCodes[mp_iIndexes[iSp]], iValue);
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// SetMoundZAndRespite()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::SetMoundZAndRespite(clTree * p_oSeedling)
{
  int iValue = 0, //value to be set
  iSp = p_oSeedling->GetSpecies(); //seedling's species
  //Set lf_count to 0
  p_oSeedling->SetValue(mp_iCounterCodes[mp_iIndexes[iSp]], iValue);

  //Get a lognormally distributed number for z
  iValue = floor(m_fMoundHeightMean + clModelMath::NormalRandomDraw(m_fMoundStandardDeviation));
  if (iValue < 0) iValue = 0;
  p_oSeedling->SetValue(mp_iZCodes[mp_iIndexes[iSp]], iValue);
}

//---------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
//   SetFreshLogZAndRespite()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
void clNZEstablishment::SetFreshLogZAndRespite(clTree *p_oSeedling, float *p_fFreshLogProportions)
{
  float fRandom = clModelMath::GetRand(); //random number
  int iValue = 0, i, //value to be set
      iArraySize = m_iMaxRespiteTimesteps + 1,
      iSp = p_oSeedling->GetSpecies(); //seedling's species

  //Find the age of the fresh log cohort onto which the seed has landed
  for (i = 0; i < iArraySize; i++) {
    if (fRandom < p_fFreshLogProportions[i]) {
      iValue = i;
      break;
    }
  }

  //Subtract the age of the cohort from the max years of respite and set it
  //to the counter
  iValue = m_iMaxRespiteYears - ceil(iValue*mp_oSimManager->GetNumberOfYearsPerTimestep());
  p_oSeedling->SetValue(mp_iCounterCodes[mp_iIndexes[iSp]], iValue);

  //Get a lognormally distributed number for z
  iValue = floor(m_fFreshLogHeightMean + clModelMath::NormalRandomDraw(m_fFreshLogStandardDeviation));
  if (iValue < 0) iValue = 0;
  p_oSeedling->SetValue(mp_iZCodes[mp_iIndexes[iSp]], iValue);
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// LightTest()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
/////////////////////////////////////////////////////////////////////////////
bool clNZEstablishment::LightTest(const int & iX, const int & iY, int & iSpecies)
{
  float fFavorability;

  //Get the light favorability
  mp_oLightGrid->GetValueOfCell(iX, iY, mp_iLightFavCodes[mp_iIndexes[iSpecies]], & fFavorability);

  //Compare it to a random number to see what we return
  if (clModelMath::GetRand() <= fFavorability) return true;
  else
    return false;
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//   GetSubstrateProportions()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
float * clNZEstablishment::GetSubstrateProportions(const float & fFromX, const float & fFromY,
     const float & fToX, const float & fToY)
{
  float * p_fProportions = new float[number_substrates], //array to return
          fGroundProportion = 1 - m_fMoundProportion;
  int i; //loop counter

  //Place the substrate proportions in the array
  for (i = 0; i < number_substrates; i++)
  {
    p_fProportions[i] = mp_oSubstrateGrid->GetAverageFloatValue(fFromX, fFromY, fToX, fToY, mp_iSubstrateCodes[i]);
  }

  //Now reduce them by the mound/ground proportion
  for (i = mound_moss; i <= mound_litter; i++)
  {
    p_fProportions[i] *= m_fMoundProportion;
  }

  for (i = ground_moss; i <= ground_litter; i++)
  {
    p_fProportions[i] *= fGroundProportion;
  }

  return p_fProportions;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//   GetFreshLogProportions()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
float* clNZEstablishment::GetFreshLogProportions(const float &fFromX, const float &fFromY,
    const float &fToX, const float &fToY)
{

  clPackage *p_oCohort;
  float *p_fProportions, //what we'll return
        fTotal, //total fresh log proportion
        fValue = 0, //float-of-all-trades
        fXCellLength = mp_oSubstrateGrid->GetLengthXCells(),
        fYCellLength = mp_oSubstrateGrid->GetLengthYCells();
  int iMinXGridCell = floor(fFromX / fXCellLength), //minimum X grid
      iMinYGridCell = floor(fFromY / fYCellLength), //minimum Y grid
      iMaxXGridCell = floor(fToX / fXCellLength), //maximum X grid
      iMaxYGridCell = floor(fToY / fYCellLength), //maximum Y grid
 //     iNumCells = ((iMaxXGridCell - iMinXGridCell) + 1) * ((iMaxYGridCell - iMinYGridCell) + 1), //total number of cells
      iYGrid, iXGrid, i, //loop counters
      iAge, //age of substrate cohort
      iNumTimesteps = m_iMaxRespiteTimesteps + 1;

  //Declare the arrays and zero them out
  p_fProportions = new float[iNumTimesteps];
  for (i = 0; i < iNumTimesteps; i++) {
    p_fProportions[i] = 0;
  }
/*  p_fMeanDBHs = new float[m_iMaxRespiteTimesteps];
  for (i = 0; i < m_iMaxRespiteTimesteps; i++) {
    p_fMeanDBHs[i] = 0;
  }
*/

  //Loop through each substrate cell in this seed cell's area.  For each
  //substrate cohort we find,
  for (iXGrid = iMinXGridCell; iXGrid <= iMaxXGridCell; iXGrid++)
  {
    for (iYGrid = iMinYGridCell; iYGrid <= iMaxYGridCell; iYGrid++)
    {

      p_oCohort = mp_oSubstrateGrid->GetFirstPackageOfCell(iXGrid, iYGrid);
      while (p_oCohort) {

        //Get this cohort's age
        p_oCohort->GetValue(m_iCohAgeCode, &iAge);
        //Decrement the age 1 since substrate will have aged it in prep for
        //the next timestep and we want this timestep's age.
        iAge--;

        //Get the decayed logs value
        p_oCohort->GetValue(m_iCohFreshLogCode, &fValue);

        if (iAge < m_iMaxRespiteTimesteps) {
          //Add to the proper array place if young enough
          p_fProportions[iAge] += fValue;

          //Get the mean log DBH
//          p_oCohort->GetValue(m_iCohAvgFreshLogDBHCode, &fValue);
          //Add to the proper array place
//          p_fMeanDBHs[iAge] += fValue;
        }

        //Add it to the total of all freshlog
        fTotal += fValue;

        p_oCohort = p_oCohort->GetNextPackage();
      }
    }
  }
  //Proportions:
  //Now that we have a running total of all values, divide by that
  if (fTotal > 0) {
    for (i = 0; i < iNumTimesteps; i++) {
      p_fProportions[i] /= fTotal;
    }
  }

  //Now make it a cumulative total - divide each by the total of all values
  //and add to the previous value
  for (i = 1; i < m_iMaxRespiteTimesteps; i++) {
    p_fProportions[i] += p_fProportions[i-1];
  }
  p_fProportions[m_iMaxRespiteTimesteps] = 1;

  //Mean DBHs:
  //Divide in the number of grid cells to get the average value for each age
/*  for (i = 0; i < iNumTimesteps; i++) {
    p_fMeanDBHs[i] /= iNumCells;
  }
*/

  return p_fProportions;
}
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//   DoesSeedSurviveSubstrate()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
/**
 * Determines whether a seed survives on the substrate upon which it lands.  A
 * random number is compared to the value in the substrate favorabilities array
 * (mp_fSubstrateFavorability) at the given substrate type and species.  If the
 * random number is less than or equal to the favorability, the seed
 * germinates.
 * @param iSubstrateType The substrate type upon which the seed has landed.
 * @param iSpecies The seed's species.
 * @return True if the seed survives; false if it dies.
 */
bool clNZEstablishment::DoesSeedSurviveSubstrate(substrateType &iSubstrateType, int &iSpecies)
{
  if (clModelMath::GetRand() <= mp_fSubstrateFavorability[iSubstrateType][mp_iIndexes[iSpecies]]) return true;
  else
    return false;
}
//---------------------------------------------------------------------------

