// NZEstablishment
//---------------------------------------------------------------------------
#if !defined(NZEstablishment_H)
  #define NZEstablishment_H

#include "GLIBase.h"

class clTreePopulation;
class clTree;
class clGridBase;
class clLightOrg;
/**
* New Zealand Establishment - Version 1.0
* <p>
* This behavior germinates seeds cast by dispersal into seedlings.  Seed
* establishment can be affected by substrate only, or by substrate and light
* level.  Because of the need to calculate light levels, this class descends
* from clGLIBase.
* <p>
* This behavior works with seeds that have been deposited into the "Dispersed
* Seeds" grid.  If light level is being used, a GLI value is calculated at a
* user-specified height at the center of each cell in "Dispersed Seeds", which
* is used to calculate a proportion of all seeds which are expected to
* germinate.  Each seed in the grid then has to pass a series of tests to get
* to the germination stage.  First: is it within the proportion that germinate
* at that light level (if light is used)?  Second:  what substrate does it land
* on?  Is it within the proportion that will germinate on that substrate, given
* the substrate's favorability?
* <p>
* If a seed successfully establishes, its substrate is used to determine its
* rooting height.  If the seed lands on a log substrate, it gets a respite from
* fern shading.  The number of years of respite is set in the "lf_count" data
* member.  The number of years is determined by the age of the
* log substrate on which the seedling landed (which is determined by dividing
* the fresh logs pool up by age and doing a random draw to determine which
* it is), and that age is subtracted from the maximum respite time.
* <p>
* Requirements for using an object of this class in a run:
* <ul>
* <li>A disperse behavior is used
* <li>New Zealand substrate (clNZSubstrate) is used
* <li>Light filter (clLightFilter) is used
* <li>If light levels are being calculated, any other GLI-based light behavior
* is used
* </ul>
* <p>
* The namestring for this class is "NZ establishment".  Call this from the
* parameter file using either "NZ establishment with light" or "NZ
* establishment without light".  Apply this behavior to the desired species;
* use any type, since type will be ignored.
* <p>
* Copyright 2003 Charles D. Canham.
* @author Lora E. Murphy
*
  <br>Edit history:
  <br>-----------------
  <br>April 28, 2004 - Submitted as beta (LEM)
*/
class clNZEstablishment : virtual public clGLIBase {
//note: need the virtual keyword to avoid base class ambiguity.

   /**My unit tester - exists in the testing application.  I didn't want to
   * have to do this but I couldn't figure out a way to effectively unit test
   * any other way.*/
   friend class clTestNZEstablishment;

   public:

   /**
   * Constructor.
   *
   * @param p_oSimManager Sim Manager object.
   */
   clNZEstablishment(clSimManager *p_oSimManager);

   /**
   * Destructor.
   */
   ~clNZEstablishment();

  /**
  * Performs setup.  Calls:
  * <ul>
  * <li>GetParameterFileData()
  * <li>SetupGrids()
  * <li>DoLightSetup(), if m_bUsesLight is true
  * </ul>
  * @param p_oDoc DOM tree of parsed input file.
  */
   void GetData(xercesc::DOMDocument *p_oDoc);

  /**
  * Performs establishment.  Steps:
  * <ul>
  * <li>If m_bUsesLight is true, GetLightFavorabilities() is called.
  * <li>For each grid cell in Dispersed Seeds:
  * <ul>
  * <li>Function calls GetSubstrateProportions() to get the proportions of
  * different substrate types for the cell.
  * <li>The seeds are divided up among the substrate types according to their
  * relative proportions.
  * <li>For each seed, if m_bUsesLight is true, DoesSeedSurviveLight() is
  * called to determine whether the seed is within the proportion germinating
  * at the cell's light level.  If not, the seed dies.
  * <li>For each surviving seed, DoesSeedSurviveSubstrate() is called to see if
  * the seed is within the proportion germinating on the substrate upon which
  * it has landed.  If not, the seed dies.
  * <li>A seedling is created for each surviving seed.
  * <li>The seedling then gets a rooting height and a fern light respite
  * counter, depending on the type of substrate upon which it landed.  If it
  * landed on fresh logs, it's passed to SetFreshLogZAndRespite().  If it
  * landed on a mound substrate, it's passed to SetMoundZAndRespite().  If it
  * landed on a ground substrate, it's passed to SetGroundZAndRespite().
  * </ul>
  * </ul>
  */
   void Action();

  /**
  * Captures the behavior name passed from the parameter file.  If the name
  * called from the parameter file is "NZ establishment with light", then
  * m_bUsesLight is set to true.  If the name called is "NZ establishment
  * without light", then m_bUsesLight and m_bNeedsCommonParameters are both set
  * to false.
  * @param cNameString Behavior name from parameter file.
  */
   void SetNameData(char *cNameString);

   /**Overridden from clLightBase to do nothing*/
   void RegisterTreeDataMembers() {;};
   /**Overridden from clLightBase to do nothing*/
   float CalcLightValue(clTree *p_oTree, clTreePopulation *p_oPop) {return 0;};

   protected:

   /**List of substrate types - be extremely careful when changing these!*/
   enum substrateType {mound_moss, /**<Moss on mound*/
                       mound_mineralsoil, /**<Mineral soil on mound*/
                       mound_freshlogs, /**<Fresh logs on mound*/
                       mound_mossylogs, /**<Mossy logs on mound*/
                       mound_litter, /**<Litter on mound*/
                       ground_moss, /**<Moss on ground*/
                       ground_mineralsoil, /**<Mineral soil on ground*/
                       ground_freshlogs, /**<Fresh logs on ground*/
                       ground_mossylogs, /**<Mossy logs on ground*/
                       ground_litter, /**<Litter on ground*/
                       number_substrates /**<This is now shorthand for the
                                          * total number of substrates*/
                       };

   clGridBase
   /**Pointer to "NZ Substrate" grid, created by clNZSubstrate.*/
   *mp_oSubstrateGrid;

   /**Pointer to the "Dispersed Seeds" grid created by disperse behaviors.*/
   clGridBase *mp_oSeedGrid;

   /**This grid is created by this behavior and is named "Establishment Light".
   * It has the same resolution as the "Dispersed Seeds" grid and has one float
   * data member for each behavior species - "litefav_x", where x is the species
   * number.  This data member holds the proportion of seeds that germinate at
   * the GLI value calculated at m_fLightHeight for that cell. If m_bUsesLight
   * is false, this grid will not be created.*/
   clGridBase *mp_oLightGrid;

   /**Function pointer to eliminate if statements for light*/
  bool (clNZEstablishment::*DoesSeedSurviveLight)(const int &iX, const int &iY, int &iSpecies);

  float
     /**Substrate favorabilities.  First array index matches a substrateType
     * value and is sized number_substrates.  The second index is sized number
     * of behavior species.*/
     **mp_fSubstrateFavorability;
     /**Optimum light level at which there is no reduction in establishment -
     * one per behavior species if m_bUsesLight*/
  float *mp_fOptimumGLI;
     /**Slope of favorability dropoff below the optimum light level - one per
     * behavior species if m_bUsesLight*/
  float *mp_fLowGLISlope;
     /**Slope of favorability dropoff above the optimum light level - one per
     * behavior species if m_bUsesLight*/
  float *mp_fHighGLISlope;
     /**Height at which light is to be calculated if light factors into
     * establishment (if m_bUsesLight is true)*/
  float m_fLightHeight;
     /**Proportion of the plot that is mound*/
  float m_fMoundProportion;
     /**Mean mound height.  Read in in meters, then converted to mm*/
  float m_fMoundHeightMean;
     /**Mound height standard deviation.  Read in in meters, then converted to mm*/
  float m_fMoundStandardDeviation;
     /**Fresh log substrate mean height.  Read in in meters, then converted to mm*/
  float m_fFreshLogHeightMean;
     /**Fresh log height standard deviation.  Read in in meters, then converted to mm*/
  float m_fFreshLogStandardDeviation;

  int m_iMaxRespiteYears; /**<Maximum number of years of fern respite - from parameter file*/
  int m_iMaxRespiteTimesteps; /**<Maximum number of timesteps of fern respite - calculated from m_iMaxRespiteYears*/

  short int
        /**Data member codes for seed grid for number of seeds.  Array size is
        * # behavior species.*/
        *mp_iSeedGridCode;
        /**Holds data member codes for "lf_count" for seedlings - array size
        * is # behavior species.*/
  short int *mp_iCounterCodes;
        /**Holds data member codes for "z" for seedlings - array size is #
        * behavior species.*/
  short int *mp_iZCodes;
        /**Speeds access to favorability arrays*/
  short int *mp_iIndexes;
        /**Array of data member codes for the "NZ Substrate" grid.  It's sized
        * number_substrates; repeat substrates get the same code (i.e. mound
        * and ground fresh logs*/
  short int *mp_iSubstrateCodes;
        /**Array of data member codes for the "litefav_x" data member of the
        * "Establishment Light" grid.  Array is sized # of behavior species.*/
  short int *mp_iLightFavCodes;
        /**Data member code - package float - dec_flog*/
  short int m_iCohFreshLogCode;
        /**Data member code - package int - age*/
  short int m_iCohAgeCode;
         /**Data member code - package float - avglogsize*/
//         m_iCohAvgFreshLogDBHCode;

   bool m_bUsesLight; /**<Whether or not light level is used by this behavior*/

 /**
 * Declares arrays and fills them with parameter file data.  All favorability
 * values are required for all behavior species.  If light is being used
 * (m_bUsesLight = true), then the following things are also needed:
 * <ul>
 * <li>li_minSunAngle
 * <li>li_numAltGrids
 * <li>li_numAziGrids
 * </ul>
 * They will be searched for in the parameter file, and the first copy found
 * will be used.
 * <p>
 * After all data has been collected, ValidateData() is called to validate it.
 * @param p_oDoc Parsed parameter file.
 */
 void GetParameterFileData(xercesc::DOMDocument *p_oDoc);

 /**
 * Validates the parameter file data.
 * @throws modelErr if any of the following are true:
 * <ul>
 * <li>Any of the substrate favorabilities are not between 0 and 1,
 * inclusive;</li>
 * <li>m_fMoundProportion is not between 0 and 1, inclusive;</li>
 * <li>m_bUsesLight = true, and the value for m_fLightHeight is not zero or
 * greater;</li>
 * <li>m_bUsesLight = true, and the value for mp_fOptimumGLI is not between 0
 * and 100.</li>
 * </ul>
 */
 void ValidateData();

 /**
 * Populates required grid pointers.
 * <ul>
 * <li>Gets a pointer to the "NZ Substrate" grid and all its data members.</li>
 * <li>Gets a pointer to the "Dispersed Seeds" grid and all its data
 * members.</li>
 * <li>If m_bUsesLight is true, creates the "Establishment Light" grid.  Any
 * grid maps in the parameter file for this grid will be ignored.</li>
 * </ul>
 * @throws modelErr if either the "NZ Substrate" or "Dispersed Seeds" grid does
 * not exist.
 */
 void SetupGrids();

 /**
 * Declares and populates the values in the mp_iCounterCodes and mp_iZCodes
 * arrays, for the "lf_count" and "z" data members of seedlings, respectively.
 * @throws ModelErr if there is no code for any species which uses
 * establishment.
 */
 void GetTreeDataMemberCodes();

 /**
 * Performs calculations required of light (if m_bUsed = true).  This calculates
 * the brightness array for GLI.  It also populates the pointer for
 * mp_oLightOrg.
 * @throws modelErr if a clLightOrg object cannot be found.
 */
 void DoLightSetup();

 /**
 * Populates the "Establishment Light" grid.  First, GetGLIs() is called; then
 * CalculateLightFavorabilities().
 */
 void GetLightFavorabilities();

 /**
 * Gets the GLI for each cell in "Establishment Light".  A GLI calculation is
 * made at at the center of each grid cell, at m_fLightHeight.  This GLI value
 * is then put in the data member of the cell pointed to by mp_iLightFavCodes[0].
 */
 void GetGLIs();

 /**
 * Calculates the light favorabilities for each cell in "Establishment Light".
 * Each favorability is the proportion of the seeds germinating for that
 * species at the GLI in that cell.  This function expects GetGLIs() to have
 * been called; it retrieves the value from the data member pointed to by
 * mp_iLightFavCodes[0], which is the GLI.  This GLI is turned into a proportion
 * germinating according to the following:
 * <ul>
 * <li>If GLI = optimum GLI for a species, then proportion germinating = 1.
 * <li>If GLI < optimum GLI for a species, then proportion germinating = 1 -
 * <li>(slope below optimum*(optimum GLI - actual GLI)).
 * <li>If GLI > optimum GLI for a species, then proportion germinating = 1 -
 * (slope above optimum*(actual GLI - optimum GLI)).
 * </ul>
 */
 void CalculateLightFavorabilities();

 /**
 * Determines whether a seed in a given grid cell survives the light test.  A
 * random number is compared to the light favorability in litefav_x for the
 * "Establishment Light" grid.  If the random number is less than or equal to
 * the proportion germinating, it survives.
 * @param iX X grid cell number of seed to test.
 * @param iY Y grid cell number of seed to test.
 * @param iSpecies Species of seed to test.
 * @return True if the seed survives; false if it dies.
 */
 bool LightTest(const int &iX, const int &iY, int &iSpecies);

 /**
 * In the case of light not being used, this is a placeholder that always
 * has light surviving the light test.
 * @param iX Not used.
 * @param iY Not used.
 * @param iSpecies Not used.
 * @return True.
 */
 bool NoLightTest(const int &iX, const int &iY, int &iSpecies) {return true;};

 /**
 * Produces an array of substrate proportions.  First, the amount of
 * substrate for each substrate type is placed in the appropriate array bucket.
 * Then mound substrates are multiplied by the proportion of the total area
 * that is mound; ground substrates are multipled by 1 - that proportion.  Areas
 * are averaged over grid cells if seed grid cell sizes are bigger than
 * substrate grid cell sizes.
 * <p>
 * This function DOES NOT retain ownership of the array created.  It is up to
 * the calling function to delete the array.
 * @param fFromX Origin X coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @param fFromY Origin Y coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @param fToX End X coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @param fToY End Y coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @return Array, sized number_substrates, with the substrate
 * proportion for each value in substrateTypes.
 */
 float* GetSubstrateProportions(const float &fFromX, const float &fFromY,
    const float &fToX, const float &fToY);

 /**
 * Produces an array of cumulative fresh log proportions added for
 * m_iMaxRespiteTimesteps timesteps.
 * <p>
 * For fresh log proportions:  First, the amount of fresh log substrate
 * for each timestep is placed in the appropriate array bucket.  This value
 * comes from substrate cohort packages, in the dec_flog data member.  Then
 * the values are converted to proportion of total fresh log by dividing each
 * array bucket by the total in all buckets.  Then, to each array value, the
 * total value of all previous buckets is added so that the fresh log
 * proportions are cumulative.
 * <p>
 * <b>Note:</b>This also used to colllect fresh log mean DBHs.  This has been
 * commented out as not currently needed.
 * <p>
 * Areas are averaged over grid cells if seed grid cell sizes are bigger than
 * substrate grid cell sizes.
 * @param fFromX Origin X coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @param fFromY Origin Y coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @param fToX End X coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @param fToY End Y coordinate of the grid cell of "Dispersed Seeds" for
 * which to get substrate proportions.
 * @return Array of the fresh log proportions.  This function will declare the
 * array and size it m_iMaxRespiteTimesteps + 1, with each bucket holding the
 * cumulative proportion of fresh log of the array index's age in timesteps.
 * The last bucket is always 1.  <b>THIS WILL NOT FREE THE ARRAY'S MEMORY - IT
 * IS UP TO THE CALLING FUNCTION TO DO THAT.</b>
 */
 float* GetFreshLogProportions(const float &fFromX, const float &fFromY,
    const float &fToX, const float &fToY);

 /**
 * Determines whether a seed survives on the substrate upon which it lands.  A
 * random number is compared to the value in the substrate favorabilities array
 * (mp_fSubstrateFavorability) at the given substrate type and species.  If the
 * random number is less than or equal to the favorability, the seed
 * germinates.
 * @param iSubstrateType The substrate type upon which the seed has landed.
 * @param iSpecies The seed's species.
 * @return True if the seed survives; false if it dies.
 */
 bool DoesSeedSurviveSubstrate(substrateType &iSubstrateType, int &iSpecies);

 /**
 * Sets the rooting height of a tree in mm and a fern respite counter in years,
 * if a seedling has landed on fresh logs.  A random number is compared to
 * successive values in p_fFreshLogProportions until an array value is greater
 * than the random value.  That bucket represents the age of the fresh logs, in
 * timesteps, onto which the seed has landed.  The respite counter is
 * m_iMaxRespite minus the age of the fresh log cohort in years.
 * <p>
 * The rooting height is a normally-distributed random number using the fresh
 * log mean height and standard deviation from the parameter file.  (Fresh logs
 * overlie and obliterate the effects of mounds.)
 * @param p_oSeedling The seedling for which to set rooting height and fern
 * respite.
 * @param p_fFreshLogProportions An array holding the cumulative proportion of
 * fresh logs x timesteps ago, up to m_iMaxRespiteTimesteps.
 */
 void SetFreshLogZAndRespite(clTree *p_oSeedling, float *p_fFreshLogProportions);

 /**
 * Sets the rooting height of a seedling in mm if it has landed on a mound
 * substrate that is not fresh logs.  The rooting height is a random
 * normally-distributed value using the mound mean height (in
 * m_fMoundHeightMean) and mound standard deviation (in
 * m_fMoundStandardDeviation).  In this non-fresh-log case, the fern respite
 * counter is always 0.
 * @param p_oSeedling The seedling for which to set rooting height and fern
 * respite.
 */
 void SetMoundZAndRespite(clTree *p_oSeedling);

 /**
 * Sets the rooting height of a seedling in mm if it has landed on a ground
 * substrate that is not fresh logs.  The rooting height and fern respite
 * counters are always 0.
 * @param p_oSeedling The seedling for which to set rooting height and fern
 * respite.
 */
 void SetGroundZAndRespite(clTree *p_oSeedling);

 /**
 * Sets the function pointer DoesSeedSurviveLight.  If m_bUsesLight, set it
 * to LightTest.  Otherwise, set it to NoLightTest.
 */
 void SetLightFunction() {
   if (m_bUsesLight)
     DoesSeedSurviveLight = & clNZEstablishment::LightTest;
   else
     DoesSeedSurviveLight = & clNZEstablishment::NoLightTest;
};

};
//---------------------------------------------------------------------------
#endif // NZEstablishment_H
