//---------------------------------------------------------------------------
#include "NZSubstrate.h"
#include "ParsingFunctions.h"
#include "SimManager.h"
#include "GridBase.h"
#include "TreePopulation.h"
#include "ModelMath.h"
#include "Plot.h"
#include <stdio.h>
#include <math.h>

//#define SubstrateTest
//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
clNZSubstrate::clNZSubstrate( clSimManager * p_oSimManager ) : clBehaviorBase( p_oSimManager ), clWorkerBase( p_oSimManager )
{
  try
  {
    //Set the namestring
    strcpy( m_cNameString, "substrate" );

    //NULL our pointers
    mp_iDeadCodes = NULL;
    mp_bSubstrateApplies = NULL;
    mp_fPropOfDeadThatFall = NULL;
    mp_fPropOfFallenThatUproot = NULL;
    mp_iFLogCalcsCode = NULL;
    m_iNumTotalSpecies = 0;

    //Allowed file types
    m_iNumAllowedTypes = 2;
    mp_iAllowedFileTypes = new int[m_iNumAllowedTypes];
    mp_iAllowedFileTypes[0] = parfile;
    mp_iAllowedFileTypes[1] = detailed_output;

    //Versions
    m_fVersionNumber = 1;
    m_fMinimumVersionNumber = 1;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::clNZSubstrate" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     Destructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
clNZSubstrate::~clNZSubstrate()
{
  if ( mp_iDeadCodes && mp_bSubstrateApplies )
  {
    for ( int i = 0; i < m_iNumTotalSpecies; i++ )
    {
      delete[] mp_iDeadCodes[i];
      delete[] mp_bSubstrateApplies[i];
    }
  }

  delete[] mp_iDeadCodes;
  delete[] mp_bSubstrateApplies;
  delete[] mp_fPropOfDeadThatFall;
  delete[] mp_fPropOfFallenThatUproot;
  delete[] mp_iFLogCalcsCode;
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// GetData()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::GetData( xercesc::DOMDocument * p_oDoc )
{
  try
  {

    //Read in the values from the parameter file
    GetParameterFileData( p_oDoc );

    //Set up the substrate grid
    SetupSubstrateGrids();

    //Put the initial conditions into the substrate grid
    PopulateInitialConditions();

    //Get the codes for the "lgm" data member
    GetDeadCodes();
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::GetData" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     GetParameterFileData()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::GetParameterFileData( xercesc::DOMDocument * p_oDoc )
{
  try
  {
    DOMElement * p_oElement = p_oDoc->getDocumentElement();
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    floatVal * p_fTempValues; //for getting species-specific values
    float fTemp, fNumberYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep();
    short int i; //loop counter

    m_iNumTotalSpecies = p_oPop->GetNumberOfSpecies();

    //Declare the permanent arrays
    mp_fPropOfDeadThatFall = new float[m_iNumTotalSpecies];
    mp_fPropOfFallenThatUproot = new float[m_iNumTotalSpecies];

    //Declare the species-specific temp array and pre-load with the species that
    //this behavior affects
    p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      p_fTempValues[i].code = mp_iWhatSpecies[i];

    //*****************************************
    // Read in the values from the parameter file
    //*****************************************
    //Mineral soil decay alpha
    FillSingleValue( p_oElement, "nzs_mineralSoilDecayAlpha", & m_fMineralSoilA, true );
    //Mineral soil beta
    FillSingleValue( p_oElement, "nzs_mineralSoilDecayBeta", & m_fMineralSoilB, true );
    //Fresh log decay alpha
    FillSingleValue( p_oElement, "nzs_freshLogDecayAlpha", & m_fFreshLogA, true );
    //Fresh log decay beta
    FillSingleValue( p_oElement, "nzs_freshLogDecayBeta", & m_fFreshLogB, true );

    //Mossy log to moss/litter decay parameter
    FillSingleValue( p_oElement, "nzs_mossyLogToLitterMoss", & m_fMossyLogToLitterMoss, true );
    //Compound over the number of years per timestep
    m_fMossyLogToLitterMoss = pow( m_fMossyLogToLitterMoss, mp_oSimManager->GetNumberOfYearsPerTimestep() );

    //Maximum number of decay years
    FillSingleValue( p_oElement, "nzs_maxNumberDecayYears", & m_iMaxDecayYears, true );

    //Initial conditions - mineral soils
    FillSingleValue( p_oElement, "nzs_initialMineralSoil", & m_fInitMineralSoil, true );
    //Initial conditions - fresh logs
    FillSingleValue( p_oElement, "nzs_initialFreshLog", & m_fInitFreshLog, true );
    //Initial conditions - mossy logs
    FillSingleValue( p_oElement, "nzs_initialMossyLog", & m_fInitMossyLog, true );
    //Initial conditions - average dbh of new fresh logs
    //    FillSingleValue( p_oElement, "nzs_initialAvgDbhFreshLog", & m_fInitAvgDbhFreshLog, true );

    //Mineral soil root tip-up factor
    FillSingleValue( p_oElement, "nzs_rootTipupFactor", & m_fRootTipupFactor, true );

    //Proportion of litter/moss that is moss
    FillSingleValue( p_oElement, "nzs_mossProportion", & m_fMossProportion, true );

    //Proportion of dead that fall
    FillSpeciesSpecificValue( p_oElement, "nzs_propOfDeadFall", "nzs_podfVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer values to our permanent array
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPropOfDeadThatFall[p_fTempValues[i].code] = 1 - pow( 1 - p_fTempValues[i].val, fNumberYearsPerTimestep );

    //Proportion of fallen that uproot
    FillSpeciesSpecificValue( p_oElement, "nzs_propOfFallUproot", "nzs_pofuVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer values to our permanent array
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPropOfFallenThatUproot[p_fTempValues[i].code] = 1 - pow( 1 - p_fTempValues[i].val, fNumberYearsPerTimestep );


    //*****************************************
    // Validation and subsequent calculations - the proportions must add
    // up to less than or equal to 1.  If the don't, throw error.  If they do,
    // canopy moss is the remainder needed to make it all add up to 1.
    //*****************************************

    //Max number of timesteps
    m_iMaxDecayTimesteps = ceil( m_iMaxDecayYears / mp_oSimManager->GetNumberOfYearsPerTimestep() );

    //Initial conditions substrate proportions
    fTemp = 1 - ( m_fInitMineralSoil + m_fInitFreshLog + m_fInitMossyLog );
    if ( fTemp < 0 )
    {
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy( stcErr.cFunction, "clNZSubstrate::GetParameterFileData" );
      strcpy( stcErr.cMoreInfo, "Initial conditions substrate proportions must add up to be less than 1" );
      throw( stcErr );
    }

    delete[] p_fTempValues;

  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::GetParameterFileData" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// SetupSubstrateGrids()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// July 21, 2004 - Calcs grid can now have a grid map (LEM)
////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::SetupSubstrateGrids()
{
  try
  {
    char cLabel[15];
    int i;

    //**************************************
    //Create the main substrate grid
    //**************************************
    //Check for its pre-existence
    mp_oSubstrateGrid = mp_oSimManager->GetGridObject( "NZ Substrate" );

    if ( NULL == mp_oSubstrateGrid )
    {

      m_bParFileHasSubstrateMap = false;

      //There was no map of this grid, so create it
      //If there's no harvest grid, the cell lengths will be 0 and therefore
      //the grid defaults will be used
      mp_oSubstrateGrid = mp_oSimManager->CreateGrid( "NZ Substrate", //grid name
           0, //number of int data members
           5, //number of float data members
           0, //number of char data members
           0 ); //number of bool data members

      //Package data structure
      mp_oSubstrateGrid->ChangePackageDataStructure( 1, //number of package int data members
           //           4, //number of package float data members
           3, //number of package float data members
           0, //number of package char data members
           0 ); //number of package bool data members

      //Register data members
      m_iLitterCode = mp_oSubstrateGrid->RegisterFloat( "litter" );
      m_iMossCode = mp_oSubstrateGrid->RegisterFloat( "moss" );
      m_iMinSoilCode = mp_oSubstrateGrid->RegisterFloat( "minsoil" );
      m_iFreshLogCode = mp_oSubstrateGrid->RegisterFloat( "freshlog" );
      m_iMossyLogCode = mp_oSubstrateGrid->RegisterFloat( "mossylog" );
      m_iPkgAgeCode = mp_oSubstrateGrid->RegisterPackageInt( "age" );
      m_iPkgMinSoilCode = mp_oSubstrateGrid->RegisterPackageFloat( "minsoil" );
      m_iPkgFreshLogCode = mp_oSubstrateGrid->RegisterPackageFloat( "freshlog" );
      m_iPkgDecFreshLogCode = mp_oSubstrateGrid->RegisterPackageFloat( "dec_flog" );
      //      m_iPkgAvgLogSizeCode = mp_oSubstrateGrid->RegisterPackageFloat( "avglogsize" );

    }
    else
    {

      //The grid was created from a map file.  Validate it.
      m_bParFileHasSubstrateMap = true;

      //Get the data member codes
      m_iLitterCode = mp_oSubstrateGrid->GetFloatDataCode( "litter" );
      m_iMossCode = mp_oSubstrateGrid->GetFloatDataCode( "moss" );
      m_iMinSoilCode = mp_oSubstrateGrid->GetFloatDataCode( "minsoil" );
      m_iFreshLogCode = mp_oSubstrateGrid->GetFloatDataCode( "freshlog" );
      m_iMossyLogCode = mp_oSubstrateGrid->GetFloatDataCode( "mossylog" );

      //Throw an error if any of them do not exist - we can't register them at
      //this point
      if ( -1 == m_iLitterCode || -1 == m_iMossCode || -1 == m_iMinSoilCode || -1 == m_iFreshLogCode || -1 == m_iMossyLogCode )
      {

        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clNZSubstrate::SetupSubstrateGrid" );
        sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
             "\"Substrate\" did not contain all required grid data members." );
        throw( stcErr );
      }

      //The packages - try to find age, which would not be automatically present
      m_iPkgAgeCode = mp_oSubstrateGrid->GetPackageIntDataCode( "age" );

      if ( -1 == m_iPkgAgeCode )
      {

        //Packages were not included in the map - declare them now
        mp_oSubstrateGrid->ChangePackageDataStructure( 1, //number of package int data members
             //             4, //number of package float data members
             3, //number of package float data members
             0, //number of package char data members
             0 ); //number of package bool data members

        m_iPkgAgeCode = mp_oSubstrateGrid->RegisterPackageInt( "age" );
        m_iPkgMinSoilCode = mp_oSubstrateGrid->RegisterPackageFloat( "minsoil" );
        m_iPkgFreshLogCode = mp_oSubstrateGrid->RegisterPackageFloat( "freshlog" );
        m_iPkgDecFreshLogCode = mp_oSubstrateGrid->RegisterPackageFloat( "dec_flog" );
        //        m_iPkgAvgLogSizeCode = mp_oSubstrateGrid->RegisterPackageFloat( "avglogsize" );

      }
      else
      {

        //Packages should have been in the map - get their codes
        m_iPkgMinSoilCode = mp_oSubstrateGrid->GetPackageFloatDataCode( "minsoil" );
        m_iPkgFreshLogCode = mp_oSubstrateGrid->GetPackageFloatDataCode( "freshlog" );
        m_iPkgDecFreshLogCode = mp_oSubstrateGrid->GetPackageFloatDataCode( "dec_flog" );
        //        m_iPkgAvgLogSizeCode = mp_oSubstrateGrid->GetPackageFloatDataCode( "avglogsize" );

        //Throw an error if any are missing
        if ( -1 == m_iPkgMinSoilCode || -1 == m_iPkgMinSoilCode || -1 == m_iPkgFreshLogCode || -1 == m_iPkgDecFreshLogCode ) //||
        // -1 == m_iPkgAvgLogSizeCode)
        {

          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clNZSubstrate::SetupSubstrateGrid" );
          sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
               "\"Substrate\" did not contain all required grid data members." );
          throw( stcErr );
        }
      }
    }

    //**************************************
    //Create the calculations grid
    //**************************************
    //Check for its pre-existence
    mp_oCalcGrid = mp_oSimManager->GetGridObject( "nzsubstratecalcs" );

    if ( NULL == mp_oCalcGrid )
    {

      mp_oCalcGrid = mp_oSimManager->CreateGrid( "nzsubstratecalcs", 0, //number of ints
           1 + m_iMaxDecayTimesteps, //number of floats
           0, //number of chars
           0, //number of bools
           mp_oSubstrateGrid->GetLengthXCells(), //X cell length - must match the grid above!
           mp_oSubstrateGrid->GetLengthYCells() ); //Y cell length - must match the grid above!

      //Register data members
      m_iMinSoilCalcsCode = mp_oCalcGrid->RegisterFloat( "newminsoil" );
      //    m_iTotalDBHCalcsCode = mp_oCalcGrid->RegisterFloat( "flogdbh" );
      //    m_iNumLogsCalcsCode = mp_oCalcGrid->RegisterInt( "numlogs" );
      mp_iFLogCalcsCode = new short int[m_iMaxDecayTimesteps];
      for ( i = 0; i < m_iMaxDecayTimesteps; i++ )
      {
        sprintf( cLabel, "%s%d", "freshlog_", i );
        mp_iFLogCalcsCode[i] = mp_oCalcGrid->RegisterFloat( cLabel );
      }
    }
    else
    {
      //It was already created - make sure it was set up right
      //Make sure the cell lengths are right
      if ( ( mp_oCalcGrid->GetLengthXCells() != mp_oSubstrateGrid->GetLengthXCells() )
           || ( mp_oCalcGrid->GetLengthYCells() != mp_oSubstrateGrid->GetLengthYCells() ) )
           {

             //Throw an eror
             modelErr stcErr;
             stcErr.iErrorCode = BAD_DATA;
             strcpy( stcErr.cFunction, "clNZSubstrate::SetupSubstrateGrid" );
             sprintf( stcErr.cMoreInfo, "%s%s%s", "Grid map values for grid ",
                  "\"nzsubstratecalcs\" contain cell lengths which do not match ",
                  "the allowed values." );
             throw( stcErr );
      }

      //Get the data member codes
      m_iMinSoilCalcsCode = mp_oCalcGrid->GetFloatDataCode( "newminsoil" );
      if ( -1 == m_iMinSoilCalcsCode )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clNZSubstrate::SetupSubstrateGrid" );
        sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
             "\"nzsubstratecalcs\" did not contain all required grid data members." );
        throw( stcErr );
      }
      mp_iFLogCalcsCode = new short int[m_iMaxDecayTimesteps];
      for ( i = 0; i < m_iMaxDecayTimesteps; i++ )
      {
        sprintf( cLabel, "%s%d", "freshlog_", i );
        mp_iFLogCalcsCode[i] = mp_oCalcGrid->GetFloatDataCode( cLabel );

        if ( -1 == mp_iFLogCalcsCode[i] )
        {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clNZSubstrate::SetupSubstrateGrid" );
          sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
               "\"nzsubstratecalcs\" did not contain all required grid data members." );
          throw( stcErr );
        }
      }
    }


    //Get the area and the reciprocal of the area of a grid cell
    m_fGridCellArea = mp_oSubstrateGrid->GetLengthXCells() * mp_oSubstrateGrid->GetLengthYCells();
    m_fRecipOfGridCellArea = 1.0 / m_fGridCellArea;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::SetupSubstrateGrid" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// PopulateInitialConditions()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::PopulateInitialConditions()
{
  try
  {

    if ( m_bParFileHasSubstrateMap ) return;

    clPackage * p_oPackage; //new package
    float fLitterMoss = 1 - ( m_fInitMineralSoil + m_fInitFreshLog + m_fInitMossyLog );
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), i, j; //loop counters

    //Go through each grid cell and give it the proper initial conditions
    for ( i = 0; i < iNumXCells; i++ )
      for ( j = 0; j < iNumYCells; j++ )
      {

        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iLitterCode, fLitterMoss * ( 1 - m_fMossProportion ) );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iMossCode, fLitterMoss * m_fMossProportion );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iMinSoilCode, m_fInitMineralSoil );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iFreshLogCode, m_fInitFreshLog );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iMossyLogCode, m_fInitMossyLog );

        //Create a new package with the initial substrate proportions
        p_oPackage = mp_oSubstrateGrid->CreatePackageOfCell( i, j );

        //Package age will be incremented in DecaySubstrate, first timestep
        p_oPackage->SetValue( m_iPkgAgeCode, 0 );
        p_oPackage->SetValue( m_iPkgMinSoilCode, m_fInitMineralSoil );
        p_oPackage->SetValue( m_iPkgFreshLogCode, m_fInitFreshLog );
        //        p_oPackage->SetValue( m_iPkgAvgLogSizeCode, m_fInitAvgDbhFreshLog);
      } //end of for (j = 0; j < iNumYCells; j++)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::PopulateInitialConditions" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// GetDeadCodes()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 19, 2004 - Renamed this and pointed it at the "dead" tree data member
//                instead of "lgm" (LEM)
////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::GetDeadCodes()
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    char cLabel[] = "dead";
    short int iTotalTypes = p_oPop->GetNumberOfTypes(), i, j; //loop counters

    //Declare and initialize the return codes and substrate applies arrays
    mp_iDeadCodes = new short int * [m_iNumTotalSpecies];
    mp_bSubstrateApplies = new bool * [m_iNumTotalSpecies];
    for ( i = 0; i < m_iNumTotalSpecies; i++ )
    {
      mp_iDeadCodes[i] = new short int[iTotalTypes];
      mp_bSubstrateApplies[i] = new bool[iTotalTypes];
      for ( j = 0; j < iTotalTypes; j++ )
      {
        mp_iDeadCodes[i] [j] = -1;
        mp_bSubstrateApplies[i] [j] = false;
      }
    }

    //Now go through the species/type combos and get the code for each
    for ( i = 0; i < m_iNumSpeciesTypeCombos; i++ )
    {

      //If any type is seedling, throw an error
      if ( mp_whatSpeciesTypeCombos[i].iType == clTreePopulation::seedling )
      {
        modelErr stcErr;
        strcpy( stcErr.cFunction, "clNZSubstrate::GetDeadCodes" );
        strcpy( stcErr.cMoreInfo, "Substrate cannot be applied to seedlings." );
        stcErr.iErrorCode = BAD_DATA;
        throw( stcErr );
      }

      mp_iDeadCodes[mp_whatSpeciesTypeCombos[i].iSpecies] [mp_whatSpeciesTypeCombos[i].iType] =
           p_oPop->GetBoolDataCode( cLabel, mp_whatSpeciesTypeCombos[i].iSpecies, mp_whatSpeciesTypeCombos[i].iType );

      mp_bSubstrateApplies[mp_whatSpeciesTypeCombos[i].iSpecies] [mp_whatSpeciesTypeCombos[i].iType] = true;

      //If the return code is -1, throw an error
      if ( -1 == mp_iDeadCodes[mp_whatSpeciesTypeCombos[i].iSpecies] [mp_whatSpeciesTypeCombos[i].iType] )
      {
        modelErr stcErr;
        strcpy( stcErr.cFunction, "clNZSubstrate::GetDeadCodes" );
        sprintf( stcErr.cMoreInfo, "%s%d%s%d%s%s", "Type/species combo species=", mp_whatSpeciesTypeCombos[i].iSpecies,
             " type=", mp_whatSpeciesTypeCombos[i].iType,
             " must have a mortality behavior assigned to be compatible with substrate." );
        stcErr.iErrorCode = BAD_DATA;
        throw( stcErr );
      }
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::GetDeadCodes" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     Action()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::Action()
{
  try
  {
    //Update the ages of the fresh log array
    UpdateCalcsGrid();

    //Add new substrate due to mortality
    AddNewDeadTrees();
    AdjustSubstrateForMortality();

    //Add in any new substrate created by harvest
    if ( mp_oSimManager->GetCurrentTimestep() == 1 )
      AddInitialNewSubstrate();

    //Decay substrate packages
    DecaySubstrate();
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::Action" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     AddInitialNewSubstrate()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::AddInitialNewSubstrate()
{
  try
  {

    float fTemp;
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), iX, iY, i; //loop counters

    for ( iX = 0; iX < iNumXCells; iX++ )
      for ( iY = 0; iY < iNumYCells; iY++ )
      {
        //Add the amount of fresh logs and mineral soils
        mp_oCalcGrid->GetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], & fTemp );
        mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], m_fInitFreshLog + fTemp );

        mp_oCalcGrid->GetValueOfCell( iX, iY, m_iMinSoilCalcsCode, & fTemp );
        mp_oCalcGrid->SetValueOfCell( iX, iY, m_iMinSoilCalcsCode, m_fInitMineralSoil + fTemp );
      }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::HarvestSubstrate" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// AddNewDeadTrees()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 19, 2004 - Killed trees that fell (LEM)
////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::AddNewDeadTrees()
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    clPlot * p_oPlot = mp_oSimManager->GetPlotObject();
    clTreeSearch * p_oAllTrees; //search object for accessing all trees
    clTree * p_oTree; //one individual tree to work with
    float fAngle, //random azimuth angle for a tree fall
         fRadius, //tree's radius in meters
         fX, fY, //tree's X and Y coordinates
         fLogEndX, fLogEndY, //the coordinates of the end of the fallen log
         fDbh, //tree's dbh
         fHeight, //tree's height
         fTreeLogArea, //the new fresh log area for one tree, in square meters
         fSubstrate, //for getting and setting substrate amounts in
         //the calculations grid
         fStep, //height while stepping out trunk to portion out fresh logs
         fTrunkChunk, //amount of fresh log going into each cell
         fTrunkInterval = 0.5;
    int iTemp;
    short int iSp, iTp, //species and type of a tree
         i, //loop counter
         iStepX, iStepY, //coordinates of intervals while walking out trunk
         iNumTrunkChunks; //number of chunks that a fallen log is divided into
    bool bDead; //value in the "dead" data member

    //Access all trees one at a time and see who's dead - only those who
    //qualify for substrate will be considered.  We could also do a species/type
    //search, but I decided to do it this way because I think it will be
    //marginally faster.
    p_oAllTrees = p_oPop->Find( "all" );

    p_oTree = p_oAllTrees->NextTree();
    while ( p_oTree )
    {
      iSp = p_oTree->GetSpecies();
      iTp = p_oTree->GetType();
      if ( !mp_bSubstrateApplies[iSp] [iTp] )
        goto nextTree;
      p_oTree->GetValue( mp_iDeadCodes[iSp] [iTp], & bDead );
      if ( !bDead )
        goto nextTree;

      //If we're still here, this tree's dead.  Get a random number to see if
      //it falls over.
      if ( iTp != clTreePopulation::snag )
      {
        if ( clModelMath::GetRand() > mp_fPropOfDeadThatFall[iSp] )
          goto nextTree; //didn't fall
      }

      //*******************************
      // New fresh log
      //*******************************
      //Yep, it fell.  Get its X and Y location, dbh, and height
      p_oTree->GetValue( p_oPop->GetXCode( iSp, iTp ), & fX );
      p_oTree->GetValue( p_oPop->GetYCode( iSp, iTp ), & fY );
      p_oTree->GetValue( p_oPop->GetDbhCode( iSp, iTp ), & fDbh );
      p_oTree->GetValue( p_oPop->GetHeightCode( iSp, iTp ), & fHeight );

      //Kill this tree and force its removal
      p_oPop->KillTree( p_oTree, clTreePopulation::remove );

      //Calculate the new fresh log area for this tree.  The trunk area is
      //modeled as a cone and calculated in sq m.
      fRadius = fDbh * 0.005; //*0.01 converts from cm to m, *0.5 converts from diameter to radius
      fTreeLogArea = fRadius * fHeight;

#ifdef SubstrateTest
      //Testing code - this turns off stochastic tree fall
      mp_oCalcGrid->GetValueAtPoint( fX, fY, mp_iFLogCalcsCode[0], & fSubstrate );
      fSubstrate += fTreeLogArea;
      mp_oCalcGrid->SetValueAtPoint( fX, fY, mp_iFLogCalcsCode[0], fSubstrate );
#else

      //Divide the trunk into even chunks based on the number of 0.5 meter
      //sections of trunk
      iNumTrunkChunks = ceil( fHeight * 2 );
      fTrunkChunk = fTreeLogArea / iNumTrunkChunks;

      //Get a random azimuth direction for the tree to fall
      fAngle = 2.0 * M_PI * ( clModelMath::GetRand() );

      //Walk out the length of the trunk in 0.5 m intervals and put a chunk of
      //fresh log in each cell we hit
      //It's possible to go a little higher than the top of the tree, and
      //hit a grid cell that the actual trunk couldn't have reached.  I am
      //accepting that risk since I think this is a good way to portion up
      //the fallen trunk.
      for ( i = 0; i < iNumTrunkChunks; i++ )
      {

        //Find the X and Y grid cells of the point at the end of this 0.5 m
        //interval
        fStep = i * fTrunkInterval;
        mp_oCalcGrid->GetCellOfPoint( p_oPlot->CorrectX( p_oPlot->GetUncorrectedX( fX, fAngle, fStep ) ),
             p_oPlot->CorrectY( p_oPlot->GetUncorrectedY( fY, fAngle, fStep ) ), & iStepX, & iStepY );

        //Drop a trunk chunk in this cell
        //Get the amount of fresh log currently in this tree's grid cell from
        //the calculation grid
        mp_oCalcGrid->GetValueOfCell( iStepX, iStepY, mp_iFLogCalcsCode[0], & fSubstrate );
        fSubstrate += fTrunkChunk;
        mp_oCalcGrid->SetValueOfCell( iStepX, iStepY, mp_iFLogCalcsCode[0], fSubstrate );

        //Update the total number of logs and total DBH
        //        mp_oCalcGrid->GetValueOfCell( iStepX, iStepY, m_iTotalDBHCalcsCode, & fSubstrate );
        //        fSubstrate += fDbh;
        //        mp_oCalcGrid->SetValueOfCell( iStepX, iStepY, m_iTotalDBHCalcsCode, fSubstrate );

        //        mp_oCalcGrid->GetValueOfCell( iStepX, iStepY, m_iNumLogsCalcsCode, & iTemp );
        //        iTemp++;
        //        mp_oCalcGrid->SetValueOfCell( iStepX, iStepY, m_iNumLogsCalcsCode, iTemp );

      }
#endif

      //*******************************
      // New mineral soil
      //*******************************
      //Now see if this fallen tree uprooted and left a tip-up mound - in which
      //case we'll add the amount of new mineral soil matter to the tree's grid cell
      //in the calculation grid.  Only adults can uproot
      if ( iTp == clTreePopulation::adult )
      {
        if ( clModelMath::GetRand() <= mp_fPropOfFallenThatUproot[iSp] )
        {
          //Yes, it uprooted - add the new mineral soils exposed to this grid's new
          //mineral soils variable
          mp_oCalcGrid->GetValueAtPoint( fX, fY, m_iMinSoilCalcsCode, & fSubstrate );
          fSubstrate += M_PI * pow( fRadius * m_fRootTipupFactor, 2 );
          mp_oCalcGrid->SetValueAtPoint( fX, fY, m_iMinSoilCalcsCode, fSubstrate );
        }
      }

    nextTree:
      p_oTree = p_oAllTrees->NextTree();
    } //end of while (p_oTree)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::AddNewDeadTrees" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// AdjustSubstrateForMortality()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::AdjustSubstrateForMortality()
{
  try
  {
    clPackage * p_oPackage, //for working with substrate grid packages
    * p_oNextToLastPackage; //for getting the last package
    float fNewSubstrateCreated, //amount of new substrate created as a proportion
    fNewFreshLog, //new fresh log area created
    fNewMineralSoil, //new mineral soil matter exposed through tip-ups
    fAverageDBH, //average dbh of new fresh logs
    fTemp; //temp variable for working with grid vals
    int iTemp; //temp variable for working with grid vals
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), i, j; //loop counters
    bool bSubstrateAdded; //for whether or not new substrate proportions
    //were added to existing package

    //All the dead trees have been added to substrate.  Now go through each
    //grid cell in the substrate grid and update it with the new substrate from
    //the calculations grid

    for ( i = 0; i < iNumXCells; i++ )
      for ( j = 0; j < iNumYCells; j++ )
      {

        //Get the amount of fresh log area added for this grid cell
        mp_oCalcGrid->GetValueOfCell( i, j, mp_iFLogCalcsCode[0], & fNewFreshLog );
        mp_oCalcGrid->GetValueOfCell( i, j, m_iMinSoilCalcsCode, & fNewMineralSoil );

        if ( fNewFreshLog == 0.0 ) //no new fresh log added - nothing to do
          goto nextCell;

        //Was there more substrate area added than there is area in the grid?
        //If not, make the new values a proportion of the total area.  If so,
        //make the new values a proportion of total new substrate
        if ( fNewFreshLog + fNewMineralSoil <= m_fGridCellArea )
        {
          fNewFreshLog = fNewFreshLog * m_fRecipOfGridCellArea;
          fNewMineralSoil = fNewMineralSoil * m_fRecipOfGridCellArea;
        }
        else
        {
          fTemp = fNewFreshLog + fNewMineralSoil;
          fNewFreshLog = fNewFreshLog / fTemp;
          fNewMineralSoil = fNewMineralSoil / fTemp;
        }
        //Add the fresh log and mineral soils back to the grid
        mp_oCalcGrid->SetValueOfCell( i, j, mp_iFLogCalcsCode[0], fNewFreshLog );
        mp_oCalcGrid->SetValueOfCell( i, j, m_iMinSoilCalcsCode, fNewMineralSoil );
        //Calculate total new substrate proportion created
        fNewSubstrateCreated = fNewFreshLog + fNewMineralSoil;

        //Calculate the average DBH of new fresh logs
        //        mp_oCalcGrid->GetValueOfCell( i, j, m_iTotalDBHCalcsCode, & fAverageDBH);
        //        mp_oCalcGrid->GetValueOfCell( i, j, m_iNumLogsCalcsCode, & iTemp);
        //        fAverageDBH /= iTemp;

        //Now reduce each package's substrate proportions by the amount of new
        //substrate created
        bSubstrateAdded = false;
        p_oPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( i, j );
        while ( p_oPackage )
        {
          //Decrement the initial package according the new proportions created
          p_oPackage->GetValue( m_iPkgMinSoilCode, & fTemp );
          fTemp *= ( 1 - fNewSubstrateCreated );
          p_oPackage->SetValue( m_iPkgMinSoilCode, fTemp );

          p_oPackage->GetValue( m_iPkgFreshLogCode, & fTemp );
          fTemp *= ( 1 - fNewSubstrateCreated );
          p_oPackage->SetValue( m_iPkgFreshLogCode, fTemp );

          //If the package has an age of 0 (created this timestep), it's
          //initial conditions.  In this case, add the new substrates to
          //this package
          p_oPackage->GetValue( m_iPkgAgeCode, & iTemp );
          if ( iTemp == 0 )
          {

            p_oPackage->GetValue( m_iPkgMinSoilCode, & fTemp );
            fTemp += fNewMineralSoil;
            p_oPackage->SetValue( m_iPkgMinSoilCode, fTemp );

            p_oPackage->GetValue( m_iPkgFreshLogCode, & fTemp );
            fNewFreshLog += fTemp;
            p_oPackage->SetValue( m_iPkgFreshLogCode, fNewFreshLog );

            //            p_oPackage->SetValue( m_iPkgAvgLogSizeCode, fAverageDBH);

            bSubstrateAdded = true;
          }

          p_oPackage = p_oPackage->GetNextPackage();
        } //end of while (p_oPackage)

        //If there was no package already created for this timestep, create one
        //now and put it last; adjust all previous packages' proportions
        if ( !bSubstrateAdded )
        {
          p_oNextToLastPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( i, j );

          while ( p_oNextToLastPackage )
          {

            p_oPackage = p_oNextToLastPackage->GetNextPackage();
            if ( !p_oPackage )
            {
              p_oPackage = mp_oSubstrateGrid->CreatePackage( p_oNextToLastPackage );
              //Age will be incremented in DecaySubstrate() so set to 0
              p_oPackage->SetValue( m_iPkgAgeCode, ( int )0 );
              p_oPackage->SetValue( m_iPkgFreshLogCode, fNewFreshLog );
              p_oPackage->SetValue( m_iPkgMinSoilCode, fNewMineralSoil );
              //              p_oPackage->SetValue( m_iPkgAvgLogSizeCode, fAverageDBH);
              //Now break the loop
              p_oNextToLastPackage = NULL;
            }
            else
              p_oNextToLastPackage = p_oPackage;
          }
        } //end of if (!bSubstrateAdded)

      nextCell:;
      } //end of for (j = 0; j < iNumYCells; j++)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::AdjustSubstrateForMortality" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//DecaySubstrate()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::DecaySubstrate()
{
  try
  {
    clPackage * p_oPackage, //next two for working with packages
    * p_oNextPackage;
    float fPrevFreshLog, //previous fresh log value for a grid cell - before
    //new mortality-based values added
    fNewFreshLog, //new fresh log added this timestep
    fPrevMossyLog, //previous mossy log value for a grid cell
    fMineralSoil, //adjusted mineral soil substrate
    fFreshLogs, //adjusted fresh log substrate
    fMossyLogs, //amount of mossy log substrate
    fTotalSubstrate, //all the substrates added together
    fYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep(), fAgeInYears, //package's age in years
    fTemp; //for getting/setting grid values
    int iAge, //package age
    iTemp;
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), iX, iY, k; //loop counters


    for ( iX = 0; iX < iNumXCells; iX++ )
      for ( iY = 0; iY < iNumYCells; iY++ )
      {

        //***************************************
        // Mossy logs
        //***************************************
        //Save out the mossy log value
        mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iMossyLogCode, & fPrevMossyLog );
        fMossyLogs = fPrevMossyLog;

        //Reduce the existing mossy logs proportion by the amount of decay
        fMossyLogs *= ( 1 - m_fMossyLogToLitterMoss );

        //Now add in the amount by which all previous timesteps' fresh logs
        //have decayed
        for ( k = 1; k < m_iMaxDecayTimesteps; k++ )
        {
          mp_oCalcGrid->GetValueOfCell( iX, iY, mp_iFLogCalcsCode[k], & fTemp );
          fMossyLogs += fTemp * exp( m_fFreshLogA * ( pow( k - 1, m_fFreshLogB ) ) ) - fTemp
               * exp( m_fFreshLogA * ( pow( k, m_fFreshLogB ) ) );
        }

        //Set our finished mossy log value
        mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMossyLogCode, fMossyLogs );

        p_oPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( iX, iY );
        if ( p_oPackage )
        {

          //***************************************
          // Decay packages to get mineral soils and fresh logs
          //***************************************
          fMineralSoil = 0;
          fFreshLogs = 0;

          while ( p_oPackage )
          {
            p_oNextPackage = p_oPackage->GetNextPackage();

            p_oPackage->GetValue( m_iPkgAgeCode, & iAge );
            fAgeInYears = iAge * fYearsPerTimestep;

            //Organics
            p_oPackage->GetValue( m_iPkgMinSoilCode, & fTemp );
            fMineralSoil += fTemp * exp( m_fMineralSoilA * ( pow( fAgeInYears, m_fMineralSoilB ) ) );

            //Fresh logs
            p_oPackage->GetValue( m_iPkgFreshLogCode, & fTemp );
            //Get the amount decayed
            fTemp *= exp( m_fFreshLogA * pow( fAgeInYears, m_fFreshLogB ) );
            //Set this amount in the decayed fresh logs data member
            p_oPackage->SetValue( m_iPkgDecFreshLogCode, fTemp );
            fFreshLogs += fTemp;

            //Increment the package's age
            iAge++;
            //If the package has reached the max decay age, delete it
            if ( iAge >= m_iMaxDecayTimesteps )
            {
              mp_oSubstrateGrid->DeletePackage( p_oPackage );
            }
            else
            {
              p_oPackage->SetValue( m_iPkgAgeCode, iAge );
            }
            p_oPackage = p_oNextPackage;
          } //end of while (p_oPackage)

          //Assign the package totals back to the main substrate
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMinSoilCode, fMineralSoil );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iFreshLogCode, fFreshLogs );
        }
        else
        {

          //There were no substrate cohort packages.  Retrieve the proportions
          //for the other substrate types so moss and litter can be re-calculated.
          mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iMinSoilCode, & fMineralSoil );
          mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iFreshLogCode, & fFreshLogs );
        }

        //***************************************
        // Litter/moss and error correction
        //***************************************

        fTotalSubstrate = fMineralSoil + fFreshLogs + fMossyLogs;

        //Check that the proportions are less than 1
        if ( fTotalSubstrate <= 1 )
        {

          //Calculate the litter/moss value as the remainder needed to make the
          //total substrate proportions add up to 1 and assign
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMossCode, ( 1 - fTotalSubstrate ) * ( m_fMossProportion ) );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iLitterCode, ( 1 - fTotalSubstrate ) * ( 1 - m_fMossProportion ) );
        }
        else
        {
          //The proportions add up to more than 1 - re-proportion them and
          //re-assign to the grid cell
          fMineralSoil /= fTotalSubstrate;
          fFreshLogs /= fTotalSubstrate;
          fMossyLogs /= fTotalSubstrate;
          fTotalSubstrate = fMineralSoil + fFreshLogs + fMossyLogs;
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMinSoilCode, fMineralSoil );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iFreshLogCode, fFreshLogs );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMossyLogCode, fMossyLogs );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMossCode, ( 1 - fTotalSubstrate ) * ( m_fMossProportion ) );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iLitterCode, ( 1 - fTotalSubstrate ) * ( 1 - m_fMossProportion ) );
        }
      } //end of for (iY = 0; iY < iNumYCells; iY++)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNZSubstrate::DecaySubstrate" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//  UpdateCalcsGrid()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNZSubstrate::UpdateCalcsGrid()
{
  if ( 1 == mp_oSimManager->GetCurrentTimestep() ) return;

  float fVal;
  int iNumXCells = mp_oCalcGrid->GetNumberXCells(), iNumYCells = mp_oCalcGrid->GetNumberYCells(), iX, iY, i, iVal = 0;

  for ( iX = 0; iX < iNumXCells; iX++ )
  {
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      for ( i = m_iMaxDecayTimesteps - 2; i >= 0; i-- )
      {
        mp_oCalcGrid->GetValueOfCell( iX, iY, mp_iFLogCalcsCode[i], & fVal );
        mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[i + 1], fVal );
      }
      //Zero out this timestep's fresh log value
      fVal = 0;
      mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], fVal );

      //Zero out the mineral soils value
      mp_oCalcGrid->SetValueOfCell( iX, iY, m_iMinSoilCalcsCode, fVal );

      //Zero out the number of new fresh logs value
      //      mp_oCalcGrid->SetValueOfCell( iX, iY, m_iNumLogsCalcsCode, iVal );

      //Zero out the total dbh of fresh logs value
      //      mp_oCalcGrid->SetValueOfCell( iX, iY, m_iTotalDBHCalcsCode, fVal );
    }
  }
}
//---------------------------------------------------------------------------
