//---------------------------------------------------------------------------

#ifndef NZSubstrateH
#define NZSubstrateH
//---------------------------------------------------------------------------
#include "BehaviorBase.h"

class clGridBase;
class clSimManager;
class DOMDocument;

/**
* New Zealand Substrate - Version 1.0
* <p>
* Substrate keeps track of what the forest floor in a particular location is
* made of at any given time.  There are five kinds of substrate: fresh logs,
* mossy logs, mineral soil, litter, and moss.  This behavior tracks the
* input of new fresh logs and mineral soil through the fall of dead
* trees and the decay of all other substrate types into litter and moss.
* The amount of fresh logs and mineral soil contributed by a downed tree
* depends on the size of the tree.  Litter and moss are a common substrate pool
* in fixed proportion to one another.
* <p>
* The substrate behavior will create a grid called "NZ Substrate."  In this grid
* are contained, for each grid cell, the relative proportions of the various
* types of substrate.
* <p>
* An input of new substrate through mortality is tracked in a grid
* package.  The package keeps track of the original input amounts for the
* substrate creation event.  When the package reaches the maximum age limit it
* is removed.  The master grid cell substrate proportions are calculated by
* decaying the substrate amounts in all the packages associated with that cell
* according to the cohort's age.
* <p>
* Substrate should be applied to all the species/type combos for trees that can
* contribute to new fresh log generation.  Substrate depends on there being a
* float data member of the trees called "dead" in these trees (the absense of
* this code being previously defined will cause a fatal error).  The data member
* should have a code of < 0 if a tree is dead.  Dead trees should not yet have
* been removed from the tree population.  Substrate cannot be applied to
* seedlings; trying to do so causes a fatal error.
* <p>
* The namestring of this behavior, and its parameter file call string,
* is "NZ Substrate".
* <p>
* To test this code, #define SubstrateTest.  This turns off trees falling in a
* random direction, causing them to instead fall completely within the grid
* cell in which they rooted.  This removes the stochastic element and allows
* predictable results of all other substrate mechanisms to be tested.
* <p>
* Copyright 2003 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Changes:
* <br>-----------------
* <br>April 28, 2004 - Submitted as beta (LEM)
* <br>May 19, 2004 - Put in snag support (LEM)
*/

class clNZSubstrate : virtual public clBehaviorBase {

 public:

 /**
  Constructor.

  @param p_oSimManager Sim Manager object.
*/
 clNZSubstrate(clSimManager *p_oSimManager);

/**
  Destructor.
*/
 ~clNZSubstrate();

/**
  Reads values from parameter file.

  @param p_oDoc DOM tree of parsed input file.
*/
 void GetData(xercesc::DOMDocument *p_oDoc);

/**
* Computes substrate for a timestep.  MortalitySubstrate() adds new fresh logs
* and mineral soil from new dead trees.  Then DecaySubstrate() decays cohorts
* and comes up with the aggregate substrate amounts.
*/
 void Action();

 protected:

 //Substrate parameters from parameter file
 float m_fMineralSoilA; /**<Mineral soil alpha decay parameter.*/
 float m_fFreshLogA; /**<Fresh log alpha decay parameter.*/
 float m_fMineralSoilB; /**<Mineral soil beta decay parameter.*/
 float m_fFreshLogB; /**<Fresh log beta decay parameter.*/
 float m_fMossyLogToLitterMoss; /**<Conversion rate of mossy logs to canopy moss, compounded over years per timestep.*/
 float m_fInitMineralSoil; /**<Mineral soil initial conditions proportion*/
 float m_fInitFreshLog; /**<Fresh log initial conditions proportion*/
 float m_fInitMossyLog; /**<Mossy log initial conditions proportion*/
//       m_fInitAvgDbhFreshLog, /**<Average DBH of new fresh logs initial condition*/
 float m_fMossProportion; /**<Proportion of the litter/moss pool that is moss*/
 float m_fRootTipupFactor; /**<Amount by which the tree radius is multiplied
        when calculating the tip-up mound to account for the effects of
        root rip-out*/
 float m_fGridCellArea;          /**<Grid cell area of substrate grid*/
 float m_fRecipOfGridCellArea;   /**<Reciprocal of grid cell area - for proportion
                                 -of-area calculations*/
 float *mp_fPropOfDeadThatFall;  /**<Proportion of dead trees that fall.
       Array size is total number of species.  In the parameter file this is
       an annual value but we store it as proportion per timestep.*/
 float *mp_fPropOfFallenThatUproot; /**<Proportion of fallen dead that uproot.
 Array size is total number of species.  In the parameter file this is
 an annual value but we store it as proportion per timestep.*/


 int m_iMaxDecayYears;       /**<Mmaximum number of years a package is
                                 allowed to exist - from parameter file*/
 int m_iMaxDecayTimesteps; /**<m_iMaxDecayYears divided by the number of years
     per timestep, for setting up grid data members.*/
 short int **mp_iDeadCodes;       /**<Array (species by type) of codes for the
                                  "dead" tree bool data member*/
 short int *mp_iFLogCalcsCode;    /**<Code for using freshlog_i data member of calculations grid*/
 short int m_iNumTotalSpecies;   /**<Total number of species - for destructor*/
 short int m_iMossCode;          /**<Code for using moss grid data member*/
 short int m_iLitterCode;        /**<Code for using litter grid data member*/
 short int m_iMinSoilCode;       /**<Code for using minsoil grid data member*/
 short int m_iFreshLogCode;      /**<Code for using freshlog grid data member*/
 short int m_iMossyLogCode;      /**<Code for using mossylog grid data member*/
 short int m_iPkgAgeCode;        /**<Code for using age package data member*/
 short int m_iPkgMinSoilCode;    /**<Code for using minsoil package data member*/
 short int m_iPkgFreshLogCode;   /**<Code for using freshlog package data member*/
 short int m_iPkgDecFreshLogCode;/**<Code for using dec_flog package data member*/
           //m_iPkgAvgLogSizeCode, /**<Code for using avglogsize package data member*/
 short int m_iMinSoilCalcsCode;  /**<Code for using newminsoil data member of calculations grid*/
           //m_iTotalDBHCalcsCode, /**<Code for using flogdbh data member of calculations grid*/
           //m_iNumLogsCalcsCode;  /**<Code for using numlogs data member of calculations grid*/

 bool **mp_bSubstrateApplies;    /**<Array (species by type) of whether or not
                                 substrate applies to this tree kind*/
 bool m_bParFileHasSubstrateMap; /**<Whether or not the parameter file had a
                                 substrate map in it.  If it did, initial
                                 conditions will not be populated.*/

 /**
 Grid object containing substrate data, called "NZ Substrate".
 <p>
 If there is no user-defined grid cell resolution, then the default grid
 resolution is used.
 <p>
 Maps of this grid in the parameter file will be honored, but they must define
 all data members below (even if they don't include values for all of them).
 The grid cell record will consist of 5 float data members. They
 are:

 <table>
 <tr>
 <th>Data member name</th>      <th>Data type</th>      <th>Description</th>
 </tr>
 <tr>
 <td>litter</td>                <td>float</td>          <td>Litter, expressed as a proportion between 0 and 1.</td>
 </tr>
 <tr>
 <td>moss</td>                  <td>float</td>          <td>Moss, expressed as a proportion between 0 and 1.</td>
 </tr>
 <tr>
 <td>minsoil</td>               <td>float</td>          <td>Mineral soil, expressed as a proportion between 0 and 1.</td>
 </tr>
 <tr>
 <td>freshlog</td>              <td>float</td>          <td>Fresh logs, expressed as a proportion between 0 and 1.</td>
 </tr>
 <tr>
 <td>mossylog</td>              <td>float</td>          <td>Mossy logs, expressed as a proportion between 0 and 1.</td>
 </tr>
 </table>

 The packages will have the following structure:
 <table>
 <tr>
 <th>Data member name</th>      <th>Data type</th>      <th>Description</th>
 </tr>
 <tr>
 <td>age</td>                   <td>int</td>            <td>Age of package in timesteps</td>
 </tr>
 <tr>
 <td>minsoil</td>               <td>float</td>          <td>Mineral soil added in this substrate cohort.  Should not be changed.</td>
 </tr>
 <tr>
 <td>freshlog</td>              <td>float</td>          <td>Fresh logs added in this substrate cohort.  Should not be changed.</td>
 </tr>
 <tr>
 <td>dec_flog</td>              <td>float</td>          <td>The freshlog member, decayed for the cohort age.  This is for clNZEstablishment's use.</td>
 </tr>
 </table>

 The packages will be in the order of oldest first.
 <p>
 Deprecated package member: avglogsize - float - The average dbh of trees
 contributing fresh log to this cohort.  This is for clNZEstablishment's use.
 */
 clGridBase *mp_oSubstrateGrid;

 /**
 * Grid object for intermediate calculations.  This grid is called
 * "nzsubstratecalcs".  The grid cell resolution matches "NZ Substrate".
 * Data members:
 * <table>
 * <tr>
 * <th>Data member name</th> <th>Data type</th> <th>Description</th>
 * </tr>
 * <tr>
 * <td>newminsoil</td>       <td>float</td>     <td>New organic area by grid cell, in square meters</td>
 * </tr>
 * <tr>
 * <td>freshlog_i</td>       <td>float</td>     <td>Fresh logs i timesteps ago up to m_iMaxDecayTimesteps, as a proportion of grid cell area</td>
 * </tr>
 * </table>
 * Deprecated:
 * flogdbh - float - Total DBH of all fresh logs added to this grid cell and
 * numlogs - int - Number of fresh logs added this timestep
 */
 clGridBase *mp_oCalcGrid;

/**
* Gets the values from the parameter file.
* @param p_oDoc DOM tree of parsed input file.
*/
 void GetParameterFileData(xercesc::DOMDocument *p_oDoc);

/**
* Sets up the substrate grids and registers all their data members.
*/
 void SetupSubstrateGrids();

 /**
 * Converts new fresh logs and organics from square meters to a proportion of
 * the grid cell.  If the area of the new substrates is greater than the area
 * of the grid cell, each substrate is divided by the combined area of the two
 * to get a proportion.
 */
 void AdjustNewSubstrateToProportion();

/**
* <p>
 Gives each substrate grid cell the proportions of substrate defined as the
 initial conditions in the parameter file.  Then each cell is given a package
 with the same initial conditions proportions of substrate.  The fresh logs
 data member in the calcs grid for this timestep is set to the amount of
 initial conditions fresh logs.
 </p>
 <p>
 If a grid map was read in with the parameter file, as indicated by
 m_bParFileHasSubstrateMap = true, this function does not do anything.
 </p>
*/
 void PopulateInitialConditions();

 /**
 Populates the mp_iDeadCodes array with the data member codes for the "dead"
 data member.
 @throw Error if there is a species/type combo which
 participates in substrate but does not have a bool data member called "dead"
 registered.
*/
 void GetDeadCodes();

 /**
 * Gets the calculations grid ready for the new timestep.
 * <ul>
 * <li>Updates the ages of the fresh logs created over previous timesteps.  The
 * values are held in the data members freshlogs_i, where i is the new fresh
 * log proportion i timesteps ago.  This shifts all values back one.  The
 * value in freshlogs_2 is put in freshlogs_3, whose value is put in
 * freshlogs_4, etc.  The last value is lost.  freshlogs_0 is set to 0, ready
 * for the additions of the current timestep.
 * <li>Sets the value of the organics data member to 0.
 * <li>Sets the value of the flogdbh data member to 0.
 * <li>Sets the value of the numlogs data member to 0.
 * </ul>
 * This does not execute the first timestep because initial conditions has
 * populated a value into the first bucket of the array that should stay there.
 */
 void UpdateCalcsGrid();

/**
* Finds new dead trees to add to substrate.  Dead trees are identifiable by a
* value of true in their "dead" data member.  A random number is used to
* determine if a tree falls over (and thus becomes part of the tally of new
* fresh log substrate).  Snags that are "dead" are already considered fallen.
* For fallen trees, another random number determines if it uproots (and thus
* adds to the new organic area exposed).  Trees that fall are passed to
* clTreePopulation::KillTree() with remove as the code to force them to go away.
* This ensures they do not become snags.
* <p>
* Both the amount of new fresh log and the amount of organics added are a
* function of tree size.  New fresh log area is calculated as FA = r*h/2,
* where FA is new fresh log area in square meters, r is tree radius in meters,
* and h is tree height in meters.  New organics area is calculated as
* OA = pi * (r*F)<sup>2</sup>, where OA is the new organics area in square
* meters, r is the tree radius in meters, and F is the value in
* m_fRootTipupFactor, which accounts for the effects of root disturbance.
* <p>
* Trees that fall do so in a random direction.  The area of the log is divided
* evenly into h*2 chunks, where h is the tree height in m.  The function walks
* out the length of the log in 0.5-m intervals, and whatever grid cell the
* end of the interval falls in gets a fresh log area chunk.  If the
* tree uproots, its new mineral soil area lies completely within the cell where
* the tree was rooted.
* <p>
* The total amount of new fresh logs and new mineral soil, in square meters, are
* added to the freshlogs_0 and newminsoil data members of the calculations
* grid.
*/
 void AddNewDeadTrees();

/**
* For each grid cell in the substrate array, this adjusts the package values
* for new substrate added in AddNewDeadTrees().  First, the amounts of new
* substrate for fresh logs and organics in square meters of area is converted to
* a proportion of total substrate.  Then the proportions of all substrate cohort
* packages are reduced accordingly to make way for the new stuff.  If it is the
* first timestep, the new substrate is added to that timestep's package.
* Otherwise, a new package with the new substrate proportions is added.
* <p>
* <b>Note:</b>This code was originally written to calculate the average DBH of
* fresh logs falling in a grid cell, but that code has been commented out as
* not needed.  It's there if it's needed later.
*/
 void AdjustSubstrateForMortality();

 /**
 * Adds the amount of new fresh logs and new organics created in the initial
 * conditions to the calculations grid.  This is a separate step so that the
 * calculations for mortality don't get screwed up when the amount
 * of substrate in cohort packages is being reduced.
 */
 void AddInitialNewSubstrate();

/**
 Performs substrate decay.
 The first part of this function is about package management: removing packages
 that are too old, and adding the amounts of fresh logs, organics, and gap moss
 together from the others, decaying them appropriately according to the
 package's age.  Then these amounts are assigned to the master substrate list
 for each cell and mossy logs and moss and litter calculated.
*/
 void DecaySubstrate();
};
//---------------------------------------------------------------------------
#endif
