//---------------------------------------------------------------------------
#include "NonSpatialDisperse.h"
#include "TreePopulation.h"
#include "ParsingFunctions.h"
#include "SimManager.h"
#include "Plot.h"
#include "ModelMath.h"
#include "GridBase.h"

//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
//     Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
clNonSpatialDispersal::clNonSpatialDispersal( clSimManager * p_oSimManager ) : clDisperseBase( p_oSimManager ), clBehaviorBase( p_oSimManager ),
     clWorkerBase( p_oSimManager )
     {
       try
       {
         strcpy( m_cNameString, "non spatial disperse" );

         //Versions
         m_fVersionNumber = 1;
         m_fMinimumVersionNumber = 1;

         //Null pointers
         mp_fSlopeOfLambda = NULL;
         mp_fInterceptOfLambda = NULL;

         //Allowed file types
         m_iNumAllowedTypes = 2;
         mp_iAllowedFileTypes = new int[m_iNumAllowedTypes];
         mp_iAllowedFileTypes[0] = parfile;
         mp_iAllowedFileTypes[1] = detailed_output;
       }
       catch ( modelErr & err )
       {
         throw( err );
       }
       catch ( modelMsg & msg )
       {
         throw( msg );
       } //non-fatal error
       catch ( ... )
       {
         modelErr stcErr;
         stcErr.iErrorCode = UNKNOWN;
         strcpy( stcErr.cFunction, "clNonSpatialDispersal::clNonSpatialDispersal" );
         throw( stcErr );
       }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     Destructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

/////////////////////////////////////////////////////////////////////////////
clNonSpatialDispersal::~clNonSpatialDispersal()
{
  //Delete arrays
  delete[] mp_fSlopeOfLambda;
  delete[] mp_fInterceptOfLambda;
}

//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
//     DoShellSetup()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNonSpatialDispersal::DoShellSetup( DOMDocument * p_oDoc )
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    DOMElement * p_oElement = p_oDoc->getDocumentElement();
    floatVal * p_fTempValues; //for getting species-specific values
    float fAreaOfGridCellInSqMeters;
    int iNumTotalSpecies = p_oPop->GetNumberOfSpecies();
    short int i; //loop counter

    //Declare the temp array and populate it with the species to which this
    //behavior applies
    p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      p_fTempValues[i].code = mp_iWhatSpecies[i];

    m_fAreaOfPlotInHa = mp_oSimManager->GetPlotObject()->GetPlotArea();

    //Declare the arrays for holding the variables
    mp_fSlopeOfLambda = new float[iNumTotalSpecies];
    mp_fInterceptOfLambda = new float[iNumTotalSpecies];

    //Capture the values from the parameter file

    //Minimum DBH for reproduction - remember that this is the static array
    //sized total number of species
    FillSpeciesSpecificValue( p_oElement, "di_minDbhForReproduction", "di_mdfrVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fDbhForReproduction[p_fTempValues[i].code] = p_fTempValues[i].val;

    //Slope of lambda
    FillSpeciesSpecificValue( p_oElement, "di_nonSpatialSlopeOfLambda", "di_nssolVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fSlopeOfLambda[p_fTempValues[i].code] = p_fTempValues[i].val;

    //Intercept of lambda
    FillSpeciesSpecificValue( p_oElement, "di_nonSpatialInterceptOfLambda", "di_nsiolVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer to the appropriate array buckets
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fInterceptOfLambda[p_fTempValues[i].code] = p_fTempValues[i].val;

    //Calculate the lambda scaling factor
    //Determine the area in square meters of a grid cell
    fAreaOfGridCellInSqMeters = mp_oSeedGrid->GetLengthXCells() * mp_oSeedGrid->GetLengthYCells();
    m_fLambdaScaler = fAreaOfGridCellInSqMeters *
                      mp_oSimManager->GetNumberOfYearsPerTimestep();

  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNonSpatialDispersal::DoShellSetup" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
//     AddSeeds()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clNonSpatialDispersal::AddSeeds()
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    float fBasalArea, //basal area in sq. m./ha for a species
         fNumGridSeeds, //seeds already in one grid cell
         fLambda; //lambda - mean number of seeds per grid cell
    int iNumXCells = mp_oSeedGrid->GetNumberXCells(), //number x cells in seed grid
        iNumYCells = mp_oSeedGrid->GetNumberYCells(); //number y cells in seed grid
    short int i, iSpecies, iX, iY;

    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
    {
      iSpecies = mp_iWhatSpecies[i];

      //If the slope is not zero, calculate the basal area
      if ( mp_fSlopeOfLambda[iSpecies] != 0 )
      {
        fBasalArea = ComputeBasalArea( p_oPop, iSpecies );
      }
      else
      {
        fBasalArea = 0;
      }

      //Compute lambda
      fLambda = clModelMath::CalcPointValue( fBasalArea, mp_fSlopeOfLambda[iSpecies], mp_fInterceptOfLambda[iSpecies] );

      //Scale lambda to number of seeds per grid cell
      fLambda *= m_fLambdaScaler;

      //Loop through the grid cells of the seed grid and put seeds in each
      for ( iX = 0; iX < iNumXCells; iX++ )
        for ( iY = 0; iY < iNumYCells; iY++ )
        {

          //Place the seeds in the seed grid
          mp_oSeedGrid->GetValueOfCell( iX, iY, mp_iNumSeedsCode[iSpecies], & fNumGridSeeds );
          fNumGridSeeds += fLambda;
          mp_oSeedGrid->SetValueOfCell( iX, iY, mp_iNumSeedsCode[iSpecies], fNumGridSeeds );

        }
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNonSpatialDispersal::AddSeeds" );
    throw( stcErr );
  }
}

//----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
//     ComputeBasalArea()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
float clNonSpatialDispersal::ComputeBasalArea( clTreePopulation * p_oPop, short int iSpecies )
{
  try
  {
    clTreeSearch * p_oAll; //trees representing our query
    clTree * p_oTree; //individual tree
    char cQuery[30]; //query string for tree search
    float fDbh, //tree's dbh
         fBasalArea = 0; //basal area of species

    //Query for all trees that could possibly be of reproductive age for this
    //species
    if ( p_oPop->GetMinAdultDBH( iSpecies ) <= mp_fDbhForReproduction[iSpecies] )
    {
      sprintf( cQuery, "%s%d%s%d", "type=", clTreePopulation::adult, "::species=", iSpecies );
    }
    else
    {
      sprintf( cQuery, "%s%d%s%d%s%d", "type=", clTreePopulation::adult, ",", clTreePopulation::sapling,
           "::species=", iSpecies );
    }

    p_oAll = p_oPop->Find( cQuery );
    p_oTree = p_oAll->NextTree();
    while ( p_oTree )
    {

      //Get the dbh of this tree
      p_oTree->GetValue( p_oPop->GetDbhCode( iSpecies, p_oTree->GetType() ), & fDbh );

      if ( fDbh >= mp_fDbhForReproduction[iSpecies] )
      {
        fBasalArea += clModelMath::CalculateBasalArea( fDbh );
      }
      p_oTree = p_oAll->NextTree();
    }

    return fBasalArea / m_fAreaOfPlotInHa;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clNonSpatialDispersal::ComputeBasalArea" );
    throw( stcErr );
  }
}
//----------------------------------------------------------------------------
