//---------------------------------------------------------------------------

#ifndef OutputH
#define OutputH

#include "BehaviorBase.h"
#include "Constants.h"
#include <stdio.h>

class clGridBase;
class clSimManager;
class clTreeSearch;

/**
* OUTPUT - Version 1.0
*
* This is a behavior which saves output from the model run.  It would normally
* be run last in a timestep.
*
* This behavior supports saving tree data and generic grid data.  Tree data is
* defined separately for each type (seed, seedling, sapling, adult, or dead).
*
* For grids, a map can be saved at a specified resolution (which does not have
* to match the calculation resolution) at any given timestep interval.
*
* Filenames must be specified for the detailed output file.
*
* Note - all file writing is done using the old C library functions instead of
* the newer C++ functions (fstream and family).  The code was originally written
* with C++ functions and ran unacceptably slow.
*
* NEEDED:  Support for different grid resolutions.
*
* Copyright 2003 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Changes:
* <br>-----------------
* <br>April 28, 2004 - Submitted as beta (LEM)
*/
class clOutput : public clBehaviorBase {

 public:

/**
  Constructor.

  @param p_oSimManager Sim Manager object.
*/
 clOutput(clSimManager *p_oSimManager);

/**
  Destructor.
*/
 ~clOutput();

/**
 Performs output data collection file writing.  It
 begins by examining all of the desired output types and seeing which ones are
 ready for updating for this timestep.
*/
 void Action();

/**
  Gets the detailed output filename.
  @param cReturnFile Address to place the filename.  The filename will not
  be longer than MAX_FILENAME_SIZE.
*/

 void GetDetailedOutputFilename(char *cReturnFile)
   {sprintf(cReturnFile, "%s%s", m_cFileRoot, DETAILED_OUTPUT_FILE_EXT);};

/**
 Figures out the filename for the detailed output timestep file and places it
 in cFilename.  cFilename should be big enough to hold the maximum filename
 size; this function will not check to make sure it is.

 @param iTimestep Timestep.
 @param cFilename Place to put the completed filename.
*/
 void GetTimestepFilename(int iTimestep, char *cFilename);

protected:

/**
  Reads in the parameter file values.

  @param p_oDoc DOM tree of parsed input file.
*/
 void GetData(xercesc::DOMDocument *p_oDoc);

 //This is the filename of the detailed output file - any other files by this name
 //will be overwritten
 char m_cFileRoot[MAX_FILENAME_SIZE], /**<Root name of the detailed output file with no
    file extension on it*/
   m_cTarball[MAX_FILENAME_SIZE];  /**<Root detailed output file name plus extension for
      /tarball (gzip'ed and tar'ed)*/

/**
  Defines what tree output data to save
*/
 struct stcTreeOutputInfo {
   short int iType;          /**<Type number that we're saving for*/
   short int iSpecies;       /**<Species number that we're saving for*/
   short int iSaveFreq;      /**<How often, in timesteps, to save data*/
   short int iSumTimestep;   /**<For keeping track of save frequency*/
   short int iNumInts;       /**<How many int data members we're saving for*/
   short int iNumFloats;     /**<How many float data members we're saving for*/
   short int iNumChars;      /**<How many char data members we're saving for*/
   short int iNumBools;      /**<How mnay bool data members we're saving for*/
   short int *p_iIntCodes;   /**<Codes for int data members*/
   short int *p_iFloatCodes; /**<Codes for float data members*/
   short int *p_iCharCodes;  /**<Codes for char data members*/
   short int *p_iBoolCodes;  /**<Codes for bool data members*/
  } *mp_treeSettings;  /**<Array of stcTreeOutputInfo's, one for each tree
                        type that we're saving data for.*/
  short int m_iNumTreeSettingsToSave; /**<Number of tree output info structs*/

/**
 Data structure for defining what grid output data to save
*/
 struct stcGridOutputInfo {
   clGridBase *p_oGridPointer;     /**<Pointer to the grid object*/
   short int iSaveFreq;            /**<How often, in timesteps, to save data*/
   short int iSumTimestep;         /**<For keeping track of save frequency*/
   short int iNumInts;             /**<How many int data members we're saving*/
   short int iNumFloats;           /**<How many float data members we're saving*/
   short int iNumChars;            /**<How many char data members we're saving*/
   short int iNumBools;            /**<How mnay bool data members we're saving*/
   short int *p_iIntCodes;         /**<Codes for int data members*/
   short int *p_iFloatCodes;       /**<Codes for float data members*/
   short int *p_iCharCodes;        /**<Codes for char data members*/
   short int *p_iBoolCodes;        /**<Codes for bool data members*/
   short int iNumPackageInts;      /**<How many package int data members*/
   short int iNumPackageFloats;    /**<How many package float data members*/
   short int iNumPackageChars;     /**<How many package char data members*/
   short int iNumPackageBools;     /**<How mnay package bool data members*/
   short int *p_iPackageIntCodes;  /**<Codes for package int data members*/
   short int *p_iPackageFloatCodes;/**<Codes for package float data members*/
   short int *p_iPackageCharCodes; /**<Codes for package char data members*/
   short int *p_iPackageBoolCodes; /**<Codes for package bool data members*/
 } *mp_gridSettings; /**<array of stcGridOutputInfo's, one for each grid object
                     that we're saving data for*/
 short int m_iNumGridsToSave; /**<number of grid objects we're saving data for*/

/**
 Writes the detailed output header file.
*/
 void WriteDetailedOutputHeader();

/**
 Sets up the timestep file and writes the header data.

 @param cFilename to write to.
*/
 void WriteTimestepHeader(const char *cFilename);

/**
 Writes the closing tag for the timestep file.

 @param cFilename to write to.
*/
 void WriteTimestepFooter(const char *cFilename);

/**
 Appends the tree data to the file.

 @param cFilename to write to.
*/
 void WriteTreeData(const char *cFilename);

/**
 Appens the grid data to the file.
 Why is this here when grid reading is in the grid object?  Because we don't
 necessarily want to save the whole grid.  Output may have a subset of data
 members to save.  Subset or no, all data members are always saved in the
 settings list so that the map can be used as input.

 @param cFilename to write to.
*/
 void WriteGridData(const char *cFilename);

/**
 Extracts the output data relating to tree saving.

 @param p_oDoc DOM tree of parsed input file.
*/
 void ExtractTreeInfo(xercesc::DOMDocument *p_oDoc);

/**
 Extracts the output data relating to grid saving.

 @param p_oDoc DOM tree of parsed input file.
*/
 void ExtractGridInfo(xercesc::DOMDocument *p_oDoc);

/**
 Adds a string to a buffer string.  If the buffer is full, it is flushed to
 file before the new string is added.

 @param cBuf Buffer string.
 @param cToAdd String to add to buffer.
 @param out File to flush buffer to.
 @param iBufferSize Size of the buffer.
*/
 inline void AddToBuffer(char *cBuf, char *cToAdd, FILE *out, int iBufferSize) {
   if (strlen(cToAdd) + strlen(cBuf) > iBufferSize) { //max out buffer?
     fwrite(cBuf, strlen(cBuf), 1, out);  //flush the buffer to file
     cBuf[0] = '\0';   //reset
   }
   strcat(cBuf, cToAdd);
 }
};
//---------------------------------------------------------------------------
#endif
