//---------------------------------------------------------------------------

#ifndef StormLightH
#define StormLightH
//---------------------------------------------------------------------------
#include "BehaviorBase.h"

class clSimManager;
class clGridBase;
class clTreePopulation;

/**
* Storm Light - Version 1.0
*
* This behavior calculates light level as a function of local storm damage.
* It uses a grid to store these light values (light values are not assigned to
* individual trees).
*
* The light level is calculated as follows:
* <br><center><i>GLA = a + b * N</i></center>
* where:
* <ul>
* <li>GLA is the light level, as a value between 0 and 100</li>
* <li>a is the intercept of the function</li>
* <li>b is the slope of the function</li>
* <li>N is the number of trees heavily damaged in recent storms within a radius
* R</li>
* </ul>
*
* Trees count as heavily damaged if they are either snags with a storm counter
* (i.e. trees killed in storms) or live adults with a storm damage flag
* indicating heavy damage.  Saplings and seedlings never count.  Snags that
* were created by a process other than a storm are not counted (we can tell
* because they have no storm damage flag).  Storm-damaged trees have a counter
* set for time since the storm.  There is a maximum time, after which a tree is
* not counted in light calculations even if it is still damaged.
*
* The trees are counted within a user-set radius of R from the center of the
* grid cell.
*
* The light levels can be randomized, if desired.  The user can choose to use
* this deterministically, in which case the values equal the GLA value as
* calculated in the function above.  Alternately, this value can be used as
* the mean for a random draw on a probability distribution function.  The user
* can specify the PDF to use.
*
* This behavior creates a new grid called "Storm Light".
*
* The namestring and parameter file call string for this behavior are both
* "Storm Light".
*
* Copyright 2005 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>-----------------
* <br>May 3, 2005 - Created (LEM)
*/
class clStormLight : virtual public clBehaviorBase {

 public:

  /**Enum covering possible light level randomizations*/
  enum stochasticity {
    deterministic, /**<deterministic*/
    lognormal, /**<stochastic, lognormally distributed*/
    normal, /**<stochastic, normally distributed*/
    negative_binomial /**<stochastic, negative binomially distributed*/
  };

  /**
  * Constructor.  Sets the namestring.
  *
  * @param p_oSimManager Sim Manager object.
  */
  clStormLight(clSimManager *p_oSimManager);

  /**
  * Destructor.  Frees memory.
  */
  ~clStormLight();

  /**
  * Does setup for this behavior.  This reads parameter file values and sets
  * up the grid.
  *
  * @param p_oDoc DOM tree of parsed input file.
  * @throws modelErr if the value for stochasticity is unrecognized
  */
  void GetData(xercesc::DOMDocument *p_oDoc);

  /**
  * Performs light calculations.  This goes through each cell in "Storm Light"
  * and calculates its GLA value as described above.
  */
  void Action();

  protected:

  /**Enum to translate our types to array index*/
  enum types {adult, snag};

  /**Grid object which holds the light values.  The name of this grid is "Storm
  * Light".  It has one float data member called "Light".  It uses the default
  * grid cell resolution unless otherwise instructed in the parameter file.
  * A map of this grid in the parameter file will be ignored.
  */
  clGridBase *mp_oLightGrid;

  /**Return codes for the "stm_dmg" int tree data member.  Array index one is
  * sized m_iNumTypes; array index two is sized total number of species.*/
  int **mp_iStmDmgCodes;

  /**The max radius, in m, to search for storm-damaged neighbors.*/
  float m_fMaxRadius;

  /**The slope of the light function.*/
  float m_fSlope;

  /**The intercept of the light function.*/
  float m_fIntercept;

  /**Standard deviation if normal or lognormal distribution is desired, or
  * clumping parameter if lognormal.  Or nothing, if we are using deterministic
  * light levels.*/
  float m_fRandParameter;

  /**The max time since damage, in years, that a live tree can count toward
  * light calculations.*/
  int m_iMaxDmgTime;

  /**The max time since damage, in years, that a snag can count toward
  * light calculations.*/
  int m_iMaxSnagDmgTime;

  /**Number of types - 2.*/
  int m_iNumTypes;

  /**What stochasticity to apply to light levels*/
  stochasticity m_iStochasticity;

  /**Return code for the "Light" data member.*/
  short int m_iGridLightCode;
};
//---------------------------------------------------------------------------
#endif
