//---------------------------------------------------------------------------
#include "Substrate.h"
#include "ParsingFunctions.h"
#include "SimManager.h"
#include "GridBase.h"
#include "TreePopulation.h"
#include "ModelMath.h"
#include "Disturbance.h"
#include <stdio.h>
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
// Constructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// April 22, 2005 - Added new snag uproot parameter
////////////////////////////////////////////////////////////////////////////
clSubstrate::clSubstrate( clSimManager * p_oSimManager ) : clBehaviorBase( p_oSimManager ), clWorkerBase( p_oSimManager )
{
  try
  {
    //Set the namestring
    strcpy( m_cNameString, "substrate" );

    //NULL our pointers
    mp_iDeadCodes = NULL;
    mp_bSubstrateApplies = NULL;
    mp_fPropOfDeadThatFall = NULL;
    mp_fPropOfFallenThatUproot = NULL;
    mp_fPropOfSnagsThatUproot = NULL; //LEM 04/22/05
    mp_iFLogCalcsCode = NULL;
    m_iNumTotalSpecies = 0;

    //Allowed file types
    m_iNumAllowedTypes = 2;
    mp_iAllowedFileTypes = new int[m_iNumAllowedTypes];
    mp_iAllowedFileTypes[0] = parfile;
    mp_iAllowedFileTypes[1] = detailed_output;

    //Versions
    m_fVersionNumber = 1;
    m_fMinimumVersionNumber = 1;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::clSubstrate" );
    throw( stcErr );
  }
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// April 22, 2005 - Added new snag uproot parameter
////////////////////////////////////////////////////////////////////////////
clSubstrate::~clSubstrate()
{
  if ( mp_iDeadCodes && mp_bSubstrateApplies )
  {
    for ( int i = 0; i < m_iNumTotalSpecies; i++ )
    {
      delete[] mp_iDeadCodes[i];
      delete[] mp_bSubstrateApplies[i];
    }
  }

  delete[] mp_iDeadCodes;
  delete[] mp_bSubstrateApplies;
  delete[] mp_fPropOfDeadThatFall;
  delete[] mp_fPropOfFallenThatUproot;
  delete[] mp_fPropOfSnagsThatUproot; //LEM 04/22/05
  delete[] mp_iFLogCalcsCode;
}

/////////////////////////////////////////////////////////////////////////////
// GetData()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
////////////////////////////////////////////////////////////////////////////
void clSubstrate::GetData( xercesc::DOMDocument * p_oDoc )
{
  try
  {

    //Do this now - the presence of the Harvest grid is the proxy for
    //whether Harvest is in the behavior list
    mp_oHarvestGrid = mp_oSimManager->GetGridObject( "Harvest Results" );

    //Read in the values from the parameter file
    GetParameterFileData( p_oDoc );

    //Set up the substrate grids
    SetupSubstrateGrids();

    //Put the initial conditions into the substrate grid
    PopulateInitialConditions();

    //Get the codes for the "dead" data member
    GetDeadCodes();
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::GetData" );
    throw( stcErr );
  }
}

/////////////////////////////////////////////////////////////////////////////
// GetParameterFileData()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 14, 2004 - Fixed a bug by compounding the tree fall and uproot chance
//                by the number of years per timestep (LEM)
// April 22, 2005 - Added new snag uproot parameter
////////////////////////////////////////////////////////////////////////////
void clSubstrate::GetParameterFileData( xercesc::DOMDocument * p_oDoc )
{
  try
  {
    DOMElement * p_oElement = p_oDoc->getDocumentElement();
    clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );
    floatVal * p_fTempValues; //for getting species-specific values
    float fTemp, fNumberYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep();
    short int i; //loop counter

    m_iNumTotalSpecies = p_oPop->GetNumberOfSpecies();

    //Declare the permanent arrays
    mp_fPropOfDeadThatFall = new float[m_iNumTotalSpecies];
    mp_fPropOfFallenThatUproot = new float[m_iNumTotalSpecies];
    mp_fPropOfSnagsThatUproot = new float[m_iNumTotalSpecies];

    //Declare the species-specific temp array and pre-load with the species that
    //this behavior affects
    p_fTempValues = new floatVal[m_iNumBehaviorSpecies];
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      p_fTempValues[i].code = mp_iWhatSpecies[i];

    //*****************************************
    // Read in the values from the parameter file
    //*****************************************

    //Area organic matter exposed
    FillSingleValue( p_oElement, "su_rootTipupFactor", & m_fRootTipupFactor, true );

    //Proportion of dead that fall
    FillSpeciesSpecificValue( p_oElement, "su_propOfDeadFall", "su_podfVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer values to our permanent array, turning each value into a
    //per-timestep proportion on the way
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPropOfDeadThatFall[p_fTempValues[i].code] = 1 - pow( 1 - p_fTempValues[i].val, fNumberYearsPerTimestep );

    //Proportion of fallen that uproot
    FillSpeciesSpecificValue( p_oElement, "su_propOfFallUproot", "su_pofuVal", p_fTempValues,
         m_iNumBehaviorSpecies, p_oPop, true );
    //Transfer values to our permanent array, turning each value into a
    //per-timestep proportion on the way
    for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
      mp_fPropOfFallenThatUproot[p_fTempValues[i].code] = 1 - pow( 1 - p_fTempValues[i].val, fNumberYearsPerTimestep );

      //Proportion of snags that uproot
      FillSpeciesSpecificValue( p_oElement, "su_propOfSnagsUproot", "su_posuVal", p_fTempValues,
           m_iNumBehaviorSpecies, p_oPop, true );
      //Transfer values to our permanent array, turning each value into a
      //per-timestep proportion on the way
      for ( i = 0; i < m_iNumBehaviorSpecies; i++ )
        mp_fPropOfSnagsThatUproot[p_fTempValues[i].code] = 1 - pow( 1 - p_fTempValues[i].val, fNumberYearsPerTimestep );

    //Gap moss decay alpha
    FillSingleValue( p_oElement, "su_gapmossDecayAlpha", & m_fScarifiedSoilA, true );
    //Gap moss decay beta
    FillSingleValue( p_oElement, "su_gapmossDecayBeta", & m_fScarifiedSoilB, true );
    //Organic decay alpha
    FillSingleValue( p_oElement, "su_organicDecayAlpha", & m_fTipUpA, true );
    //Organic decay beta
    FillSingleValue( p_oElement, "su_organicDecayBeta", & m_fTipUpB, true );
    //Fresh log decay alpha
    FillSingleValue( p_oElement, "su_freshLogDecayAlpha", & m_fFreshLogA, true );
    //Fresh log decay beta
    FillSingleValue( p_oElement, "su_freshLogDecayBeta", & m_fFreshLogB, true );

    //Mossy log to canopy moss decay parameter
    FillSingleValue( p_oElement, "su_mossyLogToCanMoss", & m_fDecayedLogToForestFloor, true );
    //Compound over the number of years per timestep
    m_fDecayedLogToForestFloor = pow( m_fDecayedLogToForestFloor, mp_oSimManager->GetNumberOfYearsPerTimestep() );

    //Maximum number of decay years
    FillSingleValue( p_oElement, "su_maxNumberDecayYears", & m_iMaxDecayYears, true );

    //Initial conditions - gap moss
    FillSingleValue( p_oElement, "su_initialGapMoss", & m_fInitScarifiedSoil, true );
    //Initial conditions - organics
    FillSingleValue( p_oElement, "su_initialOrganic", & m_fInitTipUp, true );
    //Initial conditions - fresh logs
    FillSingleValue( p_oElement, "su_initialFreshLog", & m_fInitFreshLog, true );
    //Initial conditions - mossy logs
    FillSingleValue( p_oElement, "su_initialMossyLog", & m_fInitDecayedLog, true );

    //Only do the harvest conditions if harvest is in the behavior list
    if ( mp_oHarvestGrid )
    {
      //Partial cut conditions - gap moss
      FillSingleValue( p_oElement, "su_partialCutGapMoss", & m_fPartCutScarifiedSoil, true );
      //Partial cut conditions - organics
      FillSingleValue( p_oElement, "su_partialCutOrganic", & m_fPartCutTipUp, true );
      //Partial cut conditions - fresh logs
      FillSingleValue( p_oElement, "su_partialCutFreshLog", & m_fPartCutFreshLog, true );
      //Partial cut conditions - mossy logs
      FillSingleValue( p_oElement, "su_partialCutMossyLog", & m_fPartCutDecayedLog, true );

      //Gap cut conditions - gap moss
      FillSingleValue( p_oElement, "su_gapCutGapMoss", & m_fGapCutScarifiedSoil, true );
      //Gap cut conditions - organics
      FillSingleValue( p_oElement, "su_gapCutOrganic", & m_fGapCutTipUp, true );
      //Gap cut conditions - fresh logs
      FillSingleValue( p_oElement, "su_gapCutFreshLog", & m_fGapCutFreshLog, true );
      //Gap cut conditions - mossy logs
      FillSingleValue( p_oElement, "su_gapCutMossyLog", & m_fGapCutDecayedLog, true );

      //Clear cut conditions - gap moss
      FillSingleValue( p_oElement, "su_clearCutGapMoss", & m_fClearCutScarifiedSoil, true );
      //Clear cut conditions - organics
      FillSingleValue( p_oElement, "su_clearCutOrganic", & m_fClearCutTipUp, true );
      //Clear cut conditions - fresh logs
      FillSingleValue( p_oElement, "su_clearCutFreshLog", & m_fClearCutFreshLog, true );
      //Clear cut conditions - mossy logs
      FillSingleValue( p_oElement, "su_clearCutMossyLog", & m_fClearCutDecayedLog, true );
    }

    //*****************************************
    // Validation and subsequent calculations - all of the proportions must add
    // up to less than or equal to 1.  If the don't, throw error.  If they do,
    // canopy moss is the remainder needed to make it all add up to 1.
    //*****************************************

    //Max number of timesteps
    m_iMaxDecayTimesteps = ceil( m_iMaxDecayYears / mp_oSimManager->GetNumberOfYearsPerTimestep() );

    //Initial conditions substrate proportions
    fTemp = 1 - ( m_fInitScarifiedSoil + m_fInitTipUp + m_fInitFreshLog + m_fInitDecayedLog );
    if ( fTemp < 0 )
    {
      modelErr stcErr;
      stcErr.iErrorCode = BAD_DATA;
      strcpy( stcErr.cFunction, "clSubstrate::GetParameterFileData" );
      strcpy( stcErr.cMoreInfo, "Initial conditions substrate proportions must add up to be less than 1" );
      throw( stcErr );
    }

    if ( mp_oHarvestGrid )
    {

      //Partial cut substrate proportions
      fTemp = 1 - ( m_fPartCutScarifiedSoil + m_fPartCutTipUp + m_fPartCutFreshLog + m_fPartCutDecayedLog );
      if ( fTemp < 0 )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clSubstrate::GetParameterFileData" );
        strcpy( stcErr.cMoreInfo, "Partial cut substrate proportions must add up to be less than 1" );
        throw( stcErr );
      }


      //Gap cut substrate proportions
      fTemp = 1 - ( m_fGapCutScarifiedSoil + m_fGapCutTipUp + m_fGapCutFreshLog + m_fGapCutDecayedLog );
      if ( fTemp < 0 )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clSubstrate::GetParameterFileData" );
        strcpy( stcErr.cMoreInfo, "Gap cut substrate proportions must add up to be less than 1" );
        throw( stcErr );
      }

      //Clear cut substrate proportions
      fTemp = 1 - ( m_fClearCutScarifiedSoil + m_fClearCutTipUp + m_fClearCutFreshLog + m_fClearCutDecayedLog );
      if ( fTemp < 0 )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clSubstrate::GetParameterFileData" );
        strcpy( stcErr.cMoreInfo, "Clear cut substrate proportions must add up to be less than 1" );
        throw( stcErr );
      }
    }

    delete[] p_fTempValues;

  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::GetParameterFileData" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// SetupSubstrateGrids()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// July 21, 2004 - Added the ability for the calcs grid to read a map (LEM)
////////////////////////////////////////////////////////////////////////////
void clSubstrate::SetupSubstrateGrids()
{
  try
  {
    char cLabel[15];
    float fXCellLength = 0, fYCellLength = 0;
    int i;

    //See if the harvest results grid exists.  If it does, change the cell
    //length to match
    if ( mp_oHarvestGrid )
    {
      fXCellLength = mp_oHarvestGrid->GetLengthXCells();
      fYCellLength = mp_oHarvestGrid->GetLengthYCells();
    }

    //**************************************
    //Create the main substrate grid
    //**************************************
    //Check for its pre-existence
    mp_oSubstrateGrid = mp_oSimManager->GetGridObject( "Substrate" );

    if ( NULL == mp_oSubstrateGrid )
    {

      m_bParFileHasSubstrateMap = false;

      //There was no map of this grid, so create it
      //If there's no harvest grid, the cell lengths will be 0 and therefore
      //the grid defaults will be used
      mp_oSubstrateGrid = mp_oSimManager->CreateGrid( "Substrate", //grid name
           0, //number of int data members
           5, //number of float data members
           0, //number of char data members
           0, //number of bool data members
           fXCellLength, //X cell length
           fYCellLength ); //Y cell length

      //Package data structure
      mp_oSubstrateGrid->ChangePackageDataStructure( 1, //number of package int data members
           3, //number of package float data members
           0, //number of package char data members
           0 ); //number of package bool data members

      //Register data members
      m_iGapMossCode = mp_oSubstrateGrid->RegisterFloat( "gapmoss" );
      m_iCanMossCode = mp_oSubstrateGrid->RegisterFloat( "canmoss" );
      m_iOrganicCode = mp_oSubstrateGrid->RegisterFloat( "organic" );
      m_iFreshLogCode = mp_oSubstrateGrid->RegisterFloat( "freshlog" );
      m_iMossyLogCode = mp_oSubstrateGrid->RegisterFloat( "mossylog" );
      m_iPkgAgeCode = mp_oSubstrateGrid->RegisterPackageInt( "age" );
      m_iPkgGapMossCode = mp_oSubstrateGrid->RegisterPackageFloat( "gapmoss" );
      m_iPkgOrganicCode = mp_oSubstrateGrid->RegisterPackageFloat( "organic" );
      m_iPkgFreshLogCode = mp_oSubstrateGrid->RegisterPackageFloat( "freshlog" );

    }
    else
    {

      //The grid was created from a map file.  Validate it.
      m_bParFileHasSubstrateMap = true;

      //Make sure the cell lengths are right
      if ( ( fXCellLength > 0 && mp_oSubstrateGrid->GetLengthXCells() != fXCellLength )
           || ( fYCellLength > 0 && mp_oSubstrateGrid->GetLengthYCells() != fYCellLength ) )
           {

             //Throw an eror
             modelErr stcErr;
             stcErr.iErrorCode = BAD_DATA;
             strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
             sprintf( stcErr.cMoreInfo, "%s%s%s", "Grid map values for grid ",
                  "\"Substrate\" contain cell lengths which do not match the allowed ", "values." );
             throw( stcErr );
      }

      //Get the data member codes
      m_iGapMossCode = mp_oSubstrateGrid->GetFloatDataCode( "gapmoss" );
      m_iCanMossCode = mp_oSubstrateGrid->GetFloatDataCode( "canmoss" );
      m_iOrganicCode = mp_oSubstrateGrid->GetFloatDataCode( "organic" );
      m_iFreshLogCode = mp_oSubstrateGrid->GetFloatDataCode( "freshlog" );
      m_iMossyLogCode = mp_oSubstrateGrid->GetFloatDataCode( "mossylog" );

      //Throw an error if any of them do not exist - we can't register them at
      //this point
      if ( -1 == m_iGapMossCode || -1 == m_iCanMossCode || -1 == m_iOrganicCode || -1 == m_iFreshLogCode
           || -1 == m_iMossyLogCode )
           {

             modelErr stcErr;
             stcErr.iErrorCode = BAD_DATA;
             strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
             sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
                  "\"Substrate\" did not contain all required grid data members." );
             throw( stcErr );
      }

      //The packages - try to find age, which would not be automatically present
      m_iPkgAgeCode = mp_oSubstrateGrid->GetPackageIntDataCode( "age" );

      if ( -1 == m_iPkgAgeCode )
      {

        //Packages were not included in the map - declare them now
        mp_oSubstrateGrid->ChangePackageDataStructure( 1, //number of package int data members
             3, //number of package float data members
             0, //number of package char data members
             0 ); //number of package bool data members

        m_iPkgAgeCode = mp_oSubstrateGrid->RegisterPackageInt( "age" );
        m_iPkgGapMossCode = mp_oSubstrateGrid->RegisterPackageFloat( "gapmoss" );
        m_iPkgOrganicCode = mp_oSubstrateGrid->RegisterPackageFloat( "organic" );
        m_iPkgFreshLogCode = mp_oSubstrateGrid->RegisterPackageFloat( "freshlog" );

      }
      else
      {

        //Packages should have been in the map - get their codes
        m_iPkgGapMossCode = mp_oSubstrateGrid->GetPackageFloatDataCode( "gapmoss" );
        m_iPkgOrganicCode = mp_oSubstrateGrid->GetPackageFloatDataCode( "organic" );
        m_iPkgFreshLogCode = mp_oSubstrateGrid->GetPackageFloatDataCode( "freshlog" );

        //Throw an error if any are missing
        if ( -1 == m_iPkgGapMossCode || -1 == m_iPkgOrganicCode || -1 == m_iPkgFreshLogCode )
        {

          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
          sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
               "\"Substrate\" did not contain all required grid data members." );
          throw( stcErr );
        }
      }
    }

    //**************************************
    //Create the calculations grid
    //**************************************
    //Check for its pre-existence
    mp_oCalcGrid = mp_oSimManager->GetGridObject( "substratecalcs" );

    if ( NULL == mp_oCalcGrid )
    {

      mp_oCalcGrid = mp_oSimManager->CreateGrid( "substratecalcs",
           0, //number of ints
           1 + m_iMaxDecayTimesteps, //number of floats
           0, //number of chars
           0, //number of bools
           fXCellLength, //X cell length - must match the grid above!
           fYCellLength ); //Y cell length - must match the grid above!

      //Register data members
      m_iOrgCalcsCode = mp_oCalcGrid->RegisterFloat( "neworganics" );
      mp_iFLogCalcsCode = new short int[m_iMaxDecayTimesteps];
      for ( i = 0; i < m_iMaxDecayTimesteps; i++ )
      {
        sprintf( cLabel, "%s%d", "freshlog_", i );
        mp_iFLogCalcsCode[i] = mp_oCalcGrid->RegisterFloat( cLabel );
      }
    }
    else
    {
      //It was already created - make sure it was set up right
      //Make sure the cell lengths are right
      if ( (mp_oCalcGrid->GetLengthXCells() != mp_oSubstrateGrid->GetLengthXCells())
           || ( mp_oCalcGrid->GetLengthYCells() != mp_oSubstrateGrid->GetLengthYCells()) )
           {

             //Throw an eror
             modelErr stcErr;
             stcErr.iErrorCode = BAD_DATA;
             strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
             sprintf( stcErr.cMoreInfo, "%s%s%s", "Grid map values for grid ",
                  "\"substratecalcs\" contain cell lengths which do not match ",
                  "the allowed values." );
             throw( stcErr );
      }

      //Get the data member codes
      m_iOrgCalcsCode = mp_oCalcGrid->GetFloatDataCode( "neworganics" );
      if (-1 == m_iOrgCalcsCode) {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
          sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
                "\"substratecalcs\" did not contain all required grid data members." );
          throw( stcErr );
        }
      mp_iFLogCalcsCode = new short int[m_iMaxDecayTimesteps];
      for ( i = 0; i < m_iMaxDecayTimesteps; i++ )
      {
        sprintf( cLabel, "%s%d", "freshlog_", i );
        mp_iFLogCalcsCode[i] = mp_oCalcGrid->GetFloatDataCode( cLabel );

        if (-1 == mp_iFLogCalcsCode[i]) {
          modelErr stcErr;
          stcErr.iErrorCode = BAD_DATA;
          strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
          sprintf( stcErr.cMoreInfo, "%s%s", "Grid map setup values for grid ",
                "\"substratecalcs\" did not contain all required grid data members." );
          throw( stcErr );
        }
      }
    }

    //Get the area and the reciprocal of the area of a grid cell
    m_fGridCellArea = mp_oSubstrateGrid->GetLengthXCells() * mp_oSubstrateGrid->GetLengthYCells();
    m_fRecipOfGridCellArea = 1.0 / m_fGridCellArea;

    //**************************************
    //If there's a harvest grid, get the data code for harvest type
    //**************************************
    if ( mp_oHarvestGrid )
    {
      m_iHarvestTypeCode = mp_oHarvestGrid->GetIntDataCode( "Harvest Type" );
      //If the code is -1 throw an error
      if ( m_iHarvestTypeCode == -1 )
      {
        modelErr stcErr;
        stcErr.iErrorCode = BAD_DATA;
        strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
        strcpy( stcErr.cMoreInfo, "Harvest grid set up in unexpected way." );
        throw( stcErr );
      }
    }
    else
      m_iHarvestTypeCode = -1;
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::SetupSubstrateGrid" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     PopulateInitialConditions()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::PopulateInitialConditions()
{
  try
  {

    if ( m_bParFileHasSubstrateMap ) return;

    clPackage * p_oPackage; //new package
    float fInitCanMoss = 1 - ( m_fInitScarifiedSoil + m_fInitTipUp + m_fInitFreshLog + m_fInitDecayedLog );
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), i, j; //loop counters

    //Go through each grid cell and give it the proper initial conditions
    for ( i = 0; i < iNumXCells; i++ )
      for ( j = 0; j < iNumYCells; j++ )
      {

        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iGapMossCode, m_fInitScarifiedSoil );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iCanMossCode, fInitCanMoss );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iOrganicCode, m_fInitTipUp );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iFreshLogCode, m_fInitFreshLog );
        mp_oSubstrateGrid->SetValueOfCell( i, j, m_iMossyLogCode, m_fInitDecayedLog );

        //Create a new package with the initial substrate proportions
        p_oPackage = mp_oSubstrateGrid->CreatePackageOfCell( i, j );

        //Package will be incremented in DecaySubstrate, first timestep
        p_oPackage->SetValue( m_iPkgAgeCode, 0 );
        p_oPackage->SetValue( m_iPkgGapMossCode, m_fInitScarifiedSoil );
        p_oPackage->SetValue( m_iPkgOrganicCode, m_fInitTipUp );
        p_oPackage->SetValue( m_iPkgFreshLogCode, m_fInitFreshLog );
      } //end of for (j = 0; j < iNumYCells; j++)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::PopulateInitialConditions" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// GetDeadCodes()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 14, 2004 - Renamed this and pointed it at the "dead" tree data member
//                instead of "lgm" (LEM)
////////////////////////////////////////////////////////////////////////////
void clSubstrate::GetDeadCodes()
{
  try
  {
    clPopulationBase * p_oTemp = mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreePopulation * p_oPop = ( clTreePopulation * ) p_oTemp;
    char cLabel[] = "dead";
    short int iTotalTypes = p_oPop->GetNumberOfTypes(), i, j; //loop counters

    //Declare and initialize the return codes and substrate applies arrays
    mp_iDeadCodes = new short int * [m_iNumTotalSpecies];
    mp_bSubstrateApplies = new bool * [m_iNumTotalSpecies];
    for ( i = 0; i < m_iNumTotalSpecies; i++ )
    {
      mp_iDeadCodes[i] = new short int[iTotalTypes];
      mp_bSubstrateApplies[i] = new bool[iTotalTypes];
      for ( j = 0; j < iTotalTypes; j++ )
      {
        mp_iDeadCodes[i] [j] = -1;
        mp_bSubstrateApplies[i] [j] = false;
      }
    }

    //Now go through the species/type combos and get the code for each
    for ( i = 0; i < m_iNumSpeciesTypeCombos; i++ )
    {

      //If any type is seedling, throw an error
      if ( mp_whatSpeciesTypeCombos[i].iType == clTreePopulation::seedling )
      {
        modelErr stcErr;
        strcpy( stcErr.cFunction, "clSubstrate::GetDeadCodes" );
        strcpy( stcErr.cMoreInfo, "Substrate cannot be applied to seedlings." );
        stcErr.iErrorCode = BAD_DATA;
        throw( stcErr );
      }

      mp_iDeadCodes[mp_whatSpeciesTypeCombos[i].iSpecies] [mp_whatSpeciesTypeCombos[i].iType] =
           p_oPop->GetBoolDataCode( cLabel, mp_whatSpeciesTypeCombos[i].iSpecies, mp_whatSpeciesTypeCombos[i].iType );

      mp_bSubstrateApplies[mp_whatSpeciesTypeCombos[i].iSpecies] [mp_whatSpeciesTypeCombos[i].iType] = true;

      //If the return code is -1, throw an error
      if ( -1 == mp_iDeadCodes[mp_whatSpeciesTypeCombos[i].iSpecies] [mp_whatSpeciesTypeCombos[i].iType] )
      {
        modelErr stcErr;
        strcpy( stcErr.cFunction, "clSubstrate::GetDeadCodes" );
        sprintf( stcErr.cMoreInfo, "%s%d%s%d%s%s", "Type/species combo species=", mp_whatSpeciesTypeCombos[i].iSpecies,
             " type=", mp_whatSpeciesTypeCombos[i].iType,
             " must have a mortality behavior assigned to be compatible with substrate." );
        stcErr.iErrorCode = BAD_DATA;
        throw( stcErr );
      }
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::GetDeadCodes" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     Action()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::Action()
{
  try
  {
    //Update the ages of the fresh log array
    UpdateFreshLogAges();

    //Add new substrate due to harvest
    HarvestSubstrate();

    //Add new substrate due to mortality
    MortalitySubstrate();

    //Add in any new substrate created by harvest
    AddHarvestAndInitialNewSubstrate();

    //Decay substrate packages
    DecaySubstrate();
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::Action" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     HarvestSubstrate()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::HarvestSubstrate()
{
  try
  {

    //If there's no harvest, skip out
    if ( !mp_oHarvestGrid ) return;

    clPackage * p_oPackage, //for working with substrate grid packages
         * p_oNewPackage; //new package we're creating
    float fNewOrganics, //amount of new organics for harvest
         fNewGapMoss, //amount of new gap moss for harvest
         fNewFreshLog, //amount of new fresh log for harvest
         fNewMossyLog; //amount of new mossy log for harvest
    int iCutType; //the harvest cut type that occurred in a grid cell
    short int iNumXCells = mp_oHarvestGrid->GetNumberXCells(), iNumYCells = mp_oHarvestGrid->GetNumberYCells(), iX, iY, i; //loop counters

    for ( iX = 0; iX < iNumXCells; iX++ )
      for ( iY = 0; iY < iNumYCells; iY++ )
      {

        //Get the cut type for this grid cell
        mp_oHarvestGrid->GetValueOfCell( iX, iY, m_iHarvestTypeCode, & iCutType );

        //If there was no cut, skip this cell
        if ( -1 == iCutType ) goto nextCell;

        //Delete all existing packages
        p_oPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( iX, iY );
        while ( p_oPackage )
        {
          mp_oSubstrateGrid->DeletePackage( p_oPackage );
          p_oPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( iX, iY );
        }

        //Clear all existing fresh log data
        fNewFreshLog = 0;
        for ( i = 1; i < m_iMaxDecayTimesteps; i++ )
        {
          mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[i], fNewFreshLog );
        }

        //Get the new substrate amounts depending on the cut type
        if ( clDisturbance::partial == iCutType )
        { //partial cut
          fNewOrganics = m_fPartCutTipUp;
          fNewGapMoss = m_fPartCutScarifiedSoil;
          fNewFreshLog = m_fPartCutFreshLog;
          fNewMossyLog = m_fPartCutDecayedLog;

        }
        else if ( clDisturbance::gap == iCutType )
        { //gap cut
          fNewOrganics = m_fGapCutTipUp;
          fNewGapMoss = m_fGapCutScarifiedSoil;
          fNewFreshLog = m_fGapCutFreshLog;
          fNewMossyLog = m_fGapCutDecayedLog;

        }
        else
        { //clear cut
          fNewOrganics = m_fClearCutTipUp;
          fNewGapMoss = m_fClearCutScarifiedSoil;
          fNewFreshLog = m_fClearCutFreshLog;
          fNewMossyLog = m_fClearCutDecayedLog;
        }

        //Set the package proportions to match the harvest
        p_oPackage = mp_oSubstrateGrid->CreatePackageOfCell( iX, iY );
        p_oPackage->SetValue( m_iPkgGapMossCode, fNewGapMoss );
        p_oPackage->SetValue( m_iPkgOrganicCode, fNewOrganics );
        p_oPackage->SetValue( m_iPkgFreshLogCode, fNewFreshLog );
        //By setting the age to zero, the proportions will pass through the
        //decay equations in DecaySubstrate() unchanged; that function will
        //then set the values in the grid to the same proportions
        p_oPackage->SetValue( m_iPkgAgeCode, 0 );

        //Set the value for the main cell's mossy logs - since they don't have
        //a place in the cohort package, we have to stash them like this.
        //DecaySubstrate() will watch out for them.
        mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMossyLogCode, fNewMossyLog );

      nextCell:;
      } //end of for (iY = 0; iY < iNumYCells; iY++)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::HarvestSubstrate" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//     AddHarvestAndInitialNewSubstrate()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::AddHarvestAndInitialNewSubstrate()
{
  try
  {

    float fNewOrganics, //amount of new organics for harvest
    fNewFreshLog, //amount of new fresh log for harvest
    fTemp;
    int iCutType; //the harvest cut type that occurred in a grid cell
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), iX, iY, i; //loop counters

    //Do two loops to cut down the number of if statements
    if ( mp_oSimManager->GetCurrentTimestep() > 1 )
    {
      //If there's no harvest, skip out
      if ( !mp_oHarvestGrid ) return;

      for ( iX = 0; iX < iNumXCells; iX++ )
        for ( iY = 0; iY < iNumYCells; iY++ )
        {

          //Was there a harvest?
          mp_oHarvestGrid->GetValueOfCell( iX, iY, m_iHarvestTypeCode, & iCutType );

          //If there was no cut, skip this cell
          if ( -1 == iCutType ) goto nextCell;

          //Get the new substrate amounts depending on the cut type
          if ( clDisturbance::partial == iCutType )
          { //partial cut
            fNewOrganics = m_fPartCutTipUp;
            fNewFreshLog = m_fPartCutFreshLog;
          }
          else if ( clDisturbance::gap == iCutType )
          { //gap cut
            fNewOrganics = m_fGapCutTipUp;
            fNewFreshLog = m_fGapCutFreshLog;
          }
          else
          { //clear cut
            fNewOrganics = m_fClearCutTipUp;
            fNewFreshLog = m_fClearCutFreshLog;
          }
          //Add the amount of fresh logs and organics
          mp_oCalcGrid->GetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], & fTemp );
          mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], fNewFreshLog + fTemp );

          mp_oCalcGrid->GetValueOfCell( iX, iY, m_iOrgCalcsCode, & fTemp );
          mp_oCalcGrid->SetValueOfCell( iX, iY, m_iOrgCalcsCode, fNewOrganics + fTemp );

        nextCell:;
        } //end of for (iY = 0; iY < iNumYCells; iY++)
    }
    else
    {
      for ( iX = 0; iX < iNumXCells; iX++ )
        for ( iY = 0; iY < iNumYCells; iY++ )
        {

          //This is the first timestep
          iCutType = -1;
          if ( mp_oHarvestGrid )
          {
            mp_oHarvestGrid->GetValueOfCell( iX, iY, m_iHarvestTypeCode, & iCutType );
          }

          if ( -1 == iCutType )
          {
            fNewOrganics = m_fInitTipUp;
            fNewFreshLog = m_fInitFreshLog;
          }
          else if ( clDisturbance::partial == iCutType )
          { //partial cut
            fNewOrganics = m_fPartCutTipUp;
            fNewFreshLog = m_fPartCutFreshLog;
          }
          else if ( clDisturbance::gap == iCutType )
          { //gap cut
            fNewOrganics = m_fGapCutTipUp;
            fNewFreshLog = m_fGapCutFreshLog;
          }
          else
          { //clear cut
            fNewOrganics = m_fClearCutTipUp;
            fNewFreshLog = m_fClearCutFreshLog;
          }

          //Add the amount of fresh logs and organics
          mp_oCalcGrid->GetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], & fTemp );
          mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], fNewFreshLog + fTemp );

          mp_oCalcGrid->GetValueOfCell( iX, iY, m_iOrgCalcsCode, & fTemp );
          mp_oCalcGrid->SetValueOfCell( iX, iY, m_iOrgCalcsCode, fNewOrganics + fTemp );
        }
    }
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::HarvestSubstrate" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// MortalitySubstrate()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::MortalitySubstrate()
{
  try
  {

    AddNewDeadTrees();
    AdjustSubstrateForMortality();

  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::MortalitySubstrate" );
    throw( stcErr );
  }
}

/////////////////////////////////////////////////////////////////////////////
// AddNewDeadTrees()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)
// May 14, 2004 - Killed trees that fell (LEM)
// April 22, 2005 - Allowed snags to uproot with their own probability (LEM)
////////////////////////////////////////////////////////////////////////////
void clSubstrate::AddNewDeadTrees()
{
  try
  {
    clTreePopulation * p_oPop = ( clTreePopulation * ) mp_oSimManager->GetPopulationObject( "treepopulation" );
    clTreeSearch * p_oAllTrees; //search object for accessing all trees
    clTree * p_oTree; //one individual tree to work with
    float fRandom, //random number
    fRadius, //tree's radius in meters
    fX, fY, //tree's X and Y coordinates
    fDbh, //tree's dbh
    fHeight, //tree's height
    fSubstrate; //for getting and setting substrate amounts in
    //the calculations grid
    short int iSp, iTp; //species and type of a tree
    bool bDead; //value in the "dead" data member

    //Access all trees one at a time and see who's dead - only those who
    //qualify for substrate will be considered.  We could also do a species/type
    //search, but I decided to do it this way because I think it will be
    //marginally faster.
    p_oAllTrees = p_oPop->Find( "all" );

    p_oTree = p_oAllTrees->NextTree();
    while ( p_oTree )
    {
      iSp = p_oTree->GetSpecies();
      iTp = p_oTree->GetType();
      if ( !mp_bSubstrateApplies[iSp] [iTp] )
        goto nextTree;
      p_oTree->GetValue( mp_iDeadCodes[iSp] [iTp], & bDead );
      if ( !bDead )
        goto nextTree;

      //If we're still here, this tree's dead.  Get a random number to see if
      //it falls over.
      if ( iTp != clTreePopulation::snag )
      {
        fRandom = clModelMath::GetRand();
        if ( fRandom > mp_fPropOfDeadThatFall[iSp] )
          goto nextTree; //didn't fall
      }

      //Yep, it fell.  Get its X and Y location, dbh, and height
      p_oTree->GetValue( p_oPop->GetXCode( iSp, iTp ), & fX );
      p_oTree->GetValue( p_oPop->GetYCode( iSp, iTp ), & fY );
      p_oTree->GetValue( p_oPop->GetDbhCode( iSp, iTp ), & fDbh );
      p_oTree->GetValue( p_oPop->GetHeightCode( iSp, iTp ), & fHeight );

      //Kill this tree and force its removal
      p_oPop->KillTree( p_oTree, clTreePopulation::remove );

      //Get the amount of fresh log currently in this tree's grid cell from the
      //calculation grid
      mp_oCalcGrid->GetValueAtPoint( fX, fY, mp_iFLogCalcsCode[0], & fSubstrate );

      //Calculate the amount of new fresh log area created and add it to the
      //tree's grid cell in the calculation grid.  The trunk area is modeled
      //as a cone and calculated in sq m
      fRadius = fDbh * 0.005; //*0.01 converts from cm to m, *0.5 converts from diameter to radius
      fSubstrate += fRadius * fHeight;
      mp_oCalcGrid->SetValueAtPoint( fX, fY, mp_iFLogCalcsCode[0], fSubstrate );

      //Now see if this fallen tree uprooted and left a tip-up mound - in which
      //case we'll add the amount of new organic matter to the tree's grid cell
      //in the calculation grid.
      if ( iTp == clTreePopulation::adult)
      {
        fRandom = clModelMath::GetRand();
        if ( fRandom <= mp_fPropOfFallenThatUproot[iSp] )
        {
          //Yes, it uprooted - add the new organics exposed to this grid's new
          //organics variable
          mp_oCalcGrid->GetValueAtPoint( fX, fY, m_iOrgCalcsCode, & fSubstrate );
          fSubstrate += M_PI * pow( fRadius * m_fRootTipupFactor, 2 );
          mp_oCalcGrid->SetValueAtPoint( fX, fY, m_iOrgCalcsCode, fSubstrate );
        }
      }
      else if ( iTp == clTreePopulation::snag) //LEM 04/22/05
      {
        fRandom = clModelMath::GetRand();
        if ( fRandom <= mp_fPropOfSnagsThatUproot[iSp] )
        {
          //Yes, it uprooted - add the new organics exposed to this grid's new
          //organics variable
          mp_oCalcGrid->GetValueAtPoint( fX, fY, m_iOrgCalcsCode, & fSubstrate );
          fSubstrate += M_PI * pow( fRadius * m_fRootTipupFactor, 2 );
          mp_oCalcGrid->SetValueAtPoint( fX, fY, m_iOrgCalcsCode, fSubstrate );
        }
      }

    nextTree:
      p_oTree = p_oAllTrees->NextTree();
    } //end of while (p_oTree)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::AddNewDeadTrees" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// AdjustSubstrateForMortality()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::AdjustSubstrateForMortality()
{
  try
  {
    clPackage * p_oPackage, //for working with substrate grid packages
    * p_oNextToLastPackage; //for getting the last package
    float fNewSubstrateCreated, //amount of new substrate created as a proportion
    fNewFreshLog, //new fresh log area created
    fNewOrganics, //new organic matter exposed through tip-ups
    fTemp; //temp variable for working with grid vals
    int iTemp; //temp variable for working with grid vals
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), i, j; //loop counters
    bool bSubstrateAdded; //for whether or not new substrate proportions
    //were added to existing package

    //All the dead trees have been added to substrate.  Now go through each
    //grid cell in the substrate grid and update it with the new substrate from
    //the calculations grid

    for ( i = 0; i < iNumXCells; i++ )
      for ( j = 0; j < iNumYCells; j++ )
      {

        //Get the amount of fresh log area added for this grid cell
        mp_oCalcGrid->GetValueOfCell( i, j, mp_iFLogCalcsCode[0], & fNewFreshLog );
        mp_oCalcGrid->GetValueOfCell( i, j, m_iOrgCalcsCode, & fNewOrganics );

        if ( fNewFreshLog == 0.0 ) //no new fresh log added - nothing to do
          goto nextCell;

        //Was there more substrate area added than there is area in the grid?
        //If not, make the new values a proportion of the total area.  If so,
        //make the new values a proportion of total new substrate
        if ( fNewFreshLog + fNewOrganics <= m_fGridCellArea )
        {
          fNewFreshLog = fNewFreshLog * m_fRecipOfGridCellArea;
          fNewOrganics = fNewOrganics * m_fRecipOfGridCellArea;
        }
        else
        {
          fTemp = fNewFreshLog + fNewOrganics;
          fNewFreshLog = fNewFreshLog / fTemp;
          fNewOrganics = fNewOrganics / fTemp;
        }
        //Add the fresh log and organics back to the grid
        mp_oCalcGrid->SetValueOfCell( i, j, mp_iFLogCalcsCode[0], fNewFreshLog );
        mp_oCalcGrid->SetValueOfCell( i, j, m_iOrgCalcsCode, fNewOrganics );
        //Calculate total new substrate proportion created
        fNewSubstrateCreated = fNewFreshLog + fNewOrganics;

        //Now reduce each package's substrate proportions by the amount of new
        //substrate created
        bSubstrateAdded = false;
        p_oPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( i, j );
        while ( p_oPackage )
        {
          //Decrement the initial package according the new proportions created
          p_oPackage->GetValue( m_iPkgGapMossCode, & fTemp );
          fTemp *= ( 1 - fNewSubstrateCreated );
          p_oPackage->SetValue( m_iPkgGapMossCode, fTemp );

          p_oPackage->GetValue( m_iPkgOrganicCode, & fTemp );
          fTemp *= ( 1 - fNewSubstrateCreated );
          p_oPackage->SetValue( m_iPkgOrganicCode, fTemp );

          p_oPackage->GetValue( m_iPkgFreshLogCode, & fTemp );
          fTemp *= ( 1 - fNewSubstrateCreated );
          p_oPackage->SetValue( m_iPkgFreshLogCode, fTemp );

          //If the package has an age of 0 (created this timestep), it's either
          //harvest or initial conditions.  In this case, add the new substrates
          //to this package
          p_oPackage->GetValue( m_iPkgAgeCode, & iTemp );
          if ( iTemp == 0 )
          {

            p_oPackage->GetValue( m_iPkgOrganicCode, & fTemp );
            fTemp += fNewOrganics;
            p_oPackage->SetValue( m_iPkgOrganicCode, fTemp );

            p_oPackage->GetValue( m_iPkgFreshLogCode, & fTemp );
            fNewFreshLog += fTemp;
            p_oPackage->SetValue( m_iPkgFreshLogCode, fNewFreshLog );

            //Main value should also match for gapmoss
            p_oPackage->GetValue( m_iPkgGapMossCode, & fTemp );
            mp_oSubstrateGrid->SetValueOfCell( i, j, m_iGapMossCode, fTemp );

            bSubstrateAdded = true;

          }

          p_oPackage = p_oPackage->GetNextPackage();
        } //end of while (p_oPackage)

        //If there was no package already created for this timestep, create one
        //now and put it last; adjust all previous packages' proportions
        if ( !bSubstrateAdded )
        {
          p_oNextToLastPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( i, j );

          while ( p_oNextToLastPackage )
          {

            p_oPackage = p_oNextToLastPackage->GetNextPackage();
            if ( !p_oPackage )
            {
              p_oPackage = mp_oSubstrateGrid->CreatePackage( p_oNextToLastPackage );
              //Age will be incremented in DecaySubstrate() so set to 0
              p_oPackage->SetValue( m_iPkgAgeCode, ( int )0 );
              p_oPackage->SetValue( m_iPkgFreshLogCode, fNewFreshLog );
              p_oPackage->SetValue( m_iPkgOrganicCode, fNewOrganics );
              //Leave gap moss at zero
              //Now break the loop
              p_oNextToLastPackage = NULL;
            }
            else
              p_oNextToLastPackage = p_oPackage;
          }
        } //end of if (!bSubstrateAdded)

      nextCell:;
      } //end of for (j = 0; j < iNumYCells; j++)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::AdjustSubstrateForMortality" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//DecaySubstrate()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::DecaySubstrate()
{
  try
  {
    clPackage * p_oPackage, //next two for working with packages
    * p_oNextPackage;
    float fPrevFreshLog, //previous fresh log value for a grid cell - before
    //new mortality-based values added
    fNewFreshLog, //new fresh log added this timestep
    fPrevMossyLog, //previous mossy log value for a grid cell
    fGapMoss, //adjusted gap moss substrate
    fOrganics, //adjusted organic substrate
    fFreshLogs, //adjusted fresh log substrate
    fMossyLogs, //amount of mossy log substrate
    fTotalSubstrate, //all the substrates added together
    fYearsPerTimestep = mp_oSimManager->GetNumberOfYearsPerTimestep(), fAgeInYears, //package's age in years
    fTemp; //for getting/setting grid values
    int iAge, //package age
    iTemp;
    short int iNumXCells = mp_oSubstrateGrid->GetNumberXCells(), iNumYCells = mp_oSubstrateGrid->GetNumberYCells(), iX, iY, k; //loop counters


    for ( iX = 0; iX < iNumXCells; iX++ )
      for ( iY = 0; iY < iNumYCells; iY++ )
      {

        //***************************************
        // Mossy logs
        //***************************************
        //Save out the mossy log value
        mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iMossyLogCode, & fPrevMossyLog );
        fMossyLogs = fPrevMossyLog;

        //If a harvest occurred in the past timestep, the mossy log value is
        //already correct.  So only update if there was no harvest
        iTemp = -1;
        if ( mp_oHarvestGrid )
          mp_oHarvestGrid->GetValueOfCell( iX, iY, m_iHarvestTypeCode, & iTemp );
        if ( iTemp == -1 )
        {

          //Reduce the existing mossy logs proportion by the amount of decay
          fMossyLogs *= ( 1 - m_fDecayedLogToForestFloor );

          //Now add in the amount by which all previous timesteps' fresh logs
          //have decayed
          for ( k = 1; k < m_iMaxDecayTimesteps; k++ )
          {
            mp_oCalcGrid->GetValueOfCell( iX, iY, mp_iFLogCalcsCode[k], & fTemp );
            fMossyLogs += fTemp * exp( m_fFreshLogA * ( pow( k - 1, m_fFreshLogB ) ) ) - fTemp
                 * exp( m_fFreshLogA * ( pow( k, m_fFreshLogB ) ) );
          }
        }

        //Set our finished mossy log value
        mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMossyLogCode, fMossyLogs );

        p_oPackage = mp_oSubstrateGrid->GetFirstPackageOfCell( iX, iY );
        if ( p_oPackage )
        {

          //***************************************
          // Decay packages to get organics, fresh logs, and gap moss
          //***************************************
          fGapMoss = 0;
          fOrganics = 0;
          fFreshLogs = 0;

          while ( p_oPackage )
          {
            p_oNextPackage = p_oPackage->GetNextPackage();

            p_oPackage->GetValue( m_iPkgAgeCode, & iAge );
            fAgeInYears = iAge * fYearsPerTimestep;

            //Gap moss
            p_oPackage->GetValue( m_iPkgGapMossCode, & fTemp );
            fGapMoss += fTemp * exp( m_fScarifiedSoilA * ( pow( fAgeInYears, m_fScarifiedSoilB ) ) );

            //Organics
            p_oPackage->GetValue( m_iPkgOrganicCode, & fTemp );
            fOrganics += fTemp * exp( m_fTipUpA * ( pow( fAgeInYears, m_fTipUpB ) ) );

            //Fresh logs
            p_oPackage->GetValue( m_iPkgFreshLogCode, & fTemp );
            fFreshLogs += fTemp * exp( m_fFreshLogA * pow( fAgeInYears, m_fFreshLogB ) );

            //Increment the package's age
            iAge++;
            //If the package has reached the max decay age, delete it
            if ( iAge >= m_iMaxDecayTimesteps )
            {
              mp_oSubstrateGrid->DeletePackage( p_oPackage );
            }
            else
            {
              p_oPackage->SetValue( m_iPkgAgeCode, iAge );
            }
            p_oPackage = p_oNextPackage;
          } //end of while (p_oPackage)

          //Assign the package totals back to the main substrate
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iGapMossCode, fGapMoss );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iOrganicCode, fOrganics );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iFreshLogCode, fFreshLogs );
        }
        else
        {

          //There were no substrate cohort packages.  Retrieve the proportions
          //for the other substrate types so canopy moss can be re-calculated.
          mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iGapMossCode, & fGapMoss );
          mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iOrganicCode, & fOrganics );
          mp_oSubstrateGrid->GetValueOfCell( iX, iY, m_iFreshLogCode, & fFreshLogs );
        }

        //***************************************
        // Canopy moss and error correction
        //***************************************

        fTotalSubstrate = fGapMoss + fOrganics + fFreshLogs + fMossyLogs;

        //Check that the proportions are less than 1
        if ( fTotalSubstrate <= 1 )

        //Calculate the canopy moss value as the remainder needed to make the
        //total substrate proportions add up to 1 and assign
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iCanMossCode, 1 - fTotalSubstrate );

        else
        {
          //The proportions add up to more than 1 - re-proportion them and
          //re-assign to the grid cell
          fGapMoss /= fTotalSubstrate;
          fOrganics /= fTotalSubstrate;
          fFreshLogs /= fTotalSubstrate;
          fMossyLogs /= fTotalSubstrate;
          fTotalSubstrate = fGapMoss + fOrganics + fFreshLogs + fMossyLogs;
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iGapMossCode, fGapMoss );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iOrganicCode, fOrganics );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iFreshLogCode, fFreshLogs );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iMossyLogCode, fMossyLogs );
          mp_oSubstrateGrid->SetValueOfCell( iX, iY, m_iCanMossCode, 1 - fTotalSubstrate );
        }
      } //end of for (iY = 0; iY < iNumYCells; iY++)
  }
  catch ( modelErr & err )
  {
    throw( err );
  }
  catch ( modelMsg & msg )
  {
    throw( msg );
  } //non-fatal error
  catch ( ... )
  {
    modelErr stcErr;
    stcErr.iErrorCode = UNKNOWN;
    strcpy( stcErr.cFunction, "clSubstrate::DecaySubstrate" );
    throw( stcErr );
  }
}

//---------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
//UpdateFreshLogAges()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

////////////////////////////////////////////////////////////////////////////
void clSubstrate::UpdateFreshLogAges()
{
  if ( 1 == mp_oSimManager->GetCurrentTimestep() ) return;

  float fVal;
  int iNumXCells = mp_oCalcGrid->GetNumberXCells(), iNumYCells = mp_oCalcGrid->GetNumberYCells(), iX, iY, i;

  for ( iX = 0; iX < iNumXCells; iX++ )
  {
    for ( iY = 0; iY < iNumYCells; iY++ )
    {
      for ( i = m_iMaxDecayTimesteps - 2; i >= 0; i-- )
      {
        mp_oCalcGrid->GetValueOfCell( iX, iY, mp_iFLogCalcsCode[i], & fVal );
        mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[i + 1], fVal );
      }
      //Zero out this timestep's value
      fVal = 0;
      mp_oCalcGrid->SetValueOfCell( iX, iY, mp_iFLogCalcsCode[0], fVal );

      //Zero out the organics value
      mp_oCalcGrid->SetValueOfCell( iX, iY, m_iOrgCalcsCode, fVal );
    }
  }
}
//---------------------------------------------------------------------------
