#ifndef ModMathH
#define ModMathH
//---------------------------------------------------------------------------
/**
* SORTIE math library.
* This library provides some model-specific math functions.
*
* The random number generator is very important.  To make sure that no one can
* mess with the seed, I put the seed and the seed setting function in the
* private portion.  Then clSimManager is a friend class and is the only one
* that can change the seed.
*
* This used to be a library of C-style functions.  But it turns out that the
* random number generator doesn't work right unless the seed is a class member.
*
* Copyright 2005 Charles D. Canham.
* @author Lora E. Murphy
*
* <br>Edit history:
* <br>-----------------
* <br>April 28, 2004 - Submitted as beta (LEM)
* <br>October25, 2004:  Added AddBarkToDBH (LEM)
* <br>February 20, 2005:  Changed into class (LEM)
*/

class clModelMath {

  friend class clSimManager;
  friend class clTestModelMath; /**<For testing random number generator*/

  public:

  //clModelMath(); use default constructor

  //~clModelMath(); use default destructor

  /**
  * Returns the value of a linear function.
  * @param fX Point for which to calculate the function.
  * @param fSlope Slope of the function.
  * @param fIntercept Intercept of the function.
  * @return Function value at the given point.
  */
  static float CalcPointValue(float fX, float fSlope, float fIntercept=0.0);

  /**
  * Random number generator.  This is a "minimal" random number generator of
  * Park and Miller with Bays-Durham shuffle and added safeguards.  Returns a
  * uniform random deviate between 0.0 and 1.0 (exclusive of the endpoint
  * values).  The value in m_iRandomSeed, below, is used as the seed.  RNMX
  * should approximate the largest floating value that is less than 1.  Credit
  * the <a href="http://lib-www.lanl.gov/numerical/bookcpdf.html">Numerical Recipes in C</a>
  * book for this code, originally called ran1().  I have made it so it uses
  * the seed in this class rather than requiring the seed to be passed in each
  * call.
  *
  * @return a random number between 0 and 1.
  */
  static float GetRand();

  /**
  * Calculates basal area.
  * @param fDbh DBH in cm for which to calculate basal area.
  * @return Basal area in square meters.
  * @throw modelErr if DBH is less than or equal to 0.
  */
  static float CalculateBasalArea(float fDbh);

  /**
  * Rounds a number to a specified number of digits.  For this function,
  * credit www.codeproject.com.
  * @param fNumber Number to round.
  * @param iNumDigits Number of digits to round the number to.  If 0, the
  * number is rounded to the nearest integer.
  * @return Rounded number.
  */
  static float Round(float fNumber, int iNumDigits);

  /**
  * Uses a random number to decide whether to round a number up or down to the
  * next integer.  The number is split into integer and fractional parts.  A
  * random number is compared to the fractional part; if it is less than or
  * equal to it, one is added to the integer part; otherwise the integer part
  * is returned as-is.
  * @param fNumber Number to round.
  * @return Rounded number.
  */
  static int RandomRound(float fNumber);

  /**
  * Calculates the exponential of the Weibull probability distribution function
  * at a particular point.
  *
  * @param fDispersal Value of dispersal variable
  * @param fTheta Value of theta variable
  * @param fDistance Distance, in meters, at which to calculate the function
  * @return Value of function
  */
  static float CalculateWeibullFunction(float fDispersal, float fTheta, float fDistance);


  /**
  * Calculates the exponential of the Lognormal probability distribution
  * function at a particular point.
  *
  * @param fX0 Value of X<sub>0</sub> variable
  * @param fXb Value of X<sub>b</sub> variable
  * @param fDistance Distance, in meters, at which to calculate the function
  * @return Value of function
  * @throw modelErr if fX0 or fDistance are not greater than 0, or if fXb is
  * zero.
  */
  static float CalculateLognormalFunction(float fX0, float fXb, float fDistance);

  /**
  * Returns a random Poisson-distributed number.  Credit the
  * <a href="http://lib-www.lanl.gov/numerical/bookcpdf.html">Numerical Recipes in C</a>
  * book for this code (originally called poidev())- I don't claim to
  * understand it entirely).
  *
  * @param fLambda Lambda (mean) of the Poisson function.
  * @return Integer random draw.
  */
  static int PoissonRandomDraw(float fLambda);

  /**
  * Returns a random lognormally-distributed number.  The lognormal
  * distribution has the form
  *
  * p(x) dx = 1/(x * sqrt(2 pi sigma<sup>2</sup>)) exp(-(ln(x) - zeta)<sup>2</sup>/2 sigma<sup>2</sup>) dx
  *
  * for x > 0. Lognormal random numbers are the exponentials of gaussian random
  * numbers.
  *
  * This code is from the <a href="http://www.gnu.org/software/gsl"> GSL (GNU
  * Scientific Library)</a>.  I simplified it to use our random number
  * generator, and cut it from double to float precision.
  * @param fMean Mean of the distribution, or zeta in the above formula.
  * @param fStdDev Standard deviation of the distribution, or sigma in the
  * above formula.
  * @return Random lognormally-distributed number.
  */
  static float LognormalRandomDraw(float fMean, float fStdDev);

  /**
  * Returns a random normally-distributed number with mean zero and standard
  * deviation sigma. The probability distribution for normal random variates is,
  * @htmlonly p(x) dx = {1 / sqrt{2 &pi; &sigma;<sup>2</sup>}} exp (-x<sup>2</sup> / 2&sigma;<sup>2</sup>) dx @endhtmlonly for x
  * in the range -infty to +infty. Use the transformation z = mu + x on the
  * numbers returned to obtain a normal distribution with mean mu. This function
  * uses the Box-Mueller algorithm which requires two calls to the random
  * number generator.  This code is from the <a href="http://www.gnu.org/software/gsl">
  * GSL (GNU Scientific Library)</a> and was originally called gsl_ran_gaussian.
  * @param fStdDev Standard deviation.
  * @return Random normally-distributed number.
  */
  static float NormalRandomDraw(float fStdDev);

  /**
  * Returns a random negative binomially-distributed number.  The form of the
  * distribution is from Equation 3.103 from Hilborn and Mangel
  * (The Ecological Detective).  Charlie Canham created this code.
  * @param fMean Function mean.
  * @param fClumping Clumping parameter.
  * @return Random negative-binomially-distributed number.
  */
  static int NegBinomialRandomDraw(float fMean, float fClumping);

  /**
  * Adds bark to a tree diameter.  The equation is:
  * <center><i>DBH = a + b * DIB + c * DIB<sup>2</sup></i></center>
  * where:
  * <ul>
  * <li>DIB = diameter inside bark at 1.35 meters, in cm</li>
  * <li>DBH = diameter outside bark at 1.35 meters, in cm</li>
  * <li>a, b, and c are parameters</li>
  * </ul>
  *
  * @param fDIB Diameter inside bark at 1.35 meters' height, in cm
  * @param fA "a" term in equation above
  * @param fB "b" term in equation above
  * @param fC "c" term in equation above
  * @return Diameter outside the bark at 1.35 meters' height, in cm
  * @throw modelErr if DIB is less than or equal to 0, or large enough to create
  * float overflow.
  */
  static float AddBarkToDBH(float fDIB, float fA, float fB, float fC);

  /**
  * Returns a random negative binomially-distributed number.  The negative
  * binomial distribution has the form,
  *
  * prob(k) =  Gamma(n + k)/(Gamma(n) Gamma(k + 1))  p<sup>n</sup> (1-p)<sup>k</sup>
  *
  * for k = 0, 1, ... . Note that n does not have to be an integer.
  *
  * This is the Leger's algorithm (given in the answers in Knuth)
  *
  * This code is from the <a href="http://www.gnu.org/software/gsl"> GSL (GNU
  * Scientific Library)</a>.  I simplified it to use our random number
  * generator, and cut it from double to float precision.
  * @param p p.
  * @param n n.
  * @return Random negative-binomially-distributed number.
  */
// MATH_EXPORT_TYPE float NegBinomialRandomDraw(float p, float n);

  /**
  * Returns a random gamma-distributed number.  The Gamma distribution of
  * order a>0 is defined by:
  *
  * p(x) dx = {1 / \Gamma(a) b<sup>a</sup> } x<sup>{a-1}</sup> e<sup>{-x/b}</sup> dx
  *
  * for x>0.  If X and Y are independent gamma-distributed random variables of
  * order a1 and a2 with the same scale parameter b, then X+Y has gamma
  * distribution of order a1+a2.
  *
  * The algorithms below are from Knuth, vol 2, 2nd ed, p. 129.
  *
  * This code is from the <a href="http://www.gnu.org/software/gsl"> GSL (GNU
  * Scientific Library)</a>.  I simplified it to use our random number
  * generator.
  * @param a a.
  * @param b b.
  * @return Gamma-distributed random deviate.
  */
// static double GammaRandomDraw (const double a, const double b);
 /**For the use of GammaRandomDraw.  This code is from the
 * <a href="http://www.gnu.org/software/gsl">GSL (GNU Scientific Library)</a>.
*/
// static double gsl_ran_gamma_int (const unsigned int a);
 /**For the use of GammaRandomDraw.  This code is from the
 * <a href="http://www.gnu.org/software/gsl">GSL (GNU Scientific Library)</a>.
*/
// static double gamma_large (const double a);
 /**For the use of GammaRandomDraw.  This code is from the
 * <a href="http://www.gnu.org/software/gsl">GSL (GNU Scientific Library)</a>.
*/
// static double gamma_frac (const double a);

  /**
  * For the use of PoissonRandomDraw.  Returns the value ln(gamma(fX)) for
  * fX > 0.  Credit the
  * <a href="http://lib-www.lanl.gov/numerical/bookcpdf.html">Numerical Recipes in C</a>
  * book for this code, originally called gammln().
  *
  * @param fX Value for which to compute the ln of the gamma function.
  * @return Value of the function.
  */
  static float GammaLn(float fX);

  private:

  /**
  * Random seed being used by this run.
  */
  static long m_iRandomSeed;

  /**
  * Sets the random seed.  This should always be a negative number.
  * @param iSeed Seed to set.
  */
  static void SetRandomSeed(long iSeed) {
   m_iRandomSeed = iSeed;
 }
};
long clModelMath::m_iRandomSeed = -1;
//---------------------------------------------------------------------------
#endif

